/*
 * @(#)FigureSelection.java 5.1
 *
 */

package CH.ifa.draw.framework;

import CH.ifa.draw.util.*;
import java.util.*;
import java.io.*;

/**
 * FigureSelection enables to transfer the selected figures
 * to a clipboard.<p>
 * Will soon be converted to the JDK 1.1 Transferable interface.
 *
 * @see Clipboard
 */

public class FigureSelection extends Object {

    private byte[] fData; // flattened figures, ready to be resurrected

    /**
     * The type identifier of the selection.
     */
    public final static String TYPE = "CH.ifa.draw.Figures";

    /**
     * Constructes the Figure selection for the vector of figures.
     */
    public FigureSelection(Vector figures) {
	fData = writeData(figures);
    }

    public FigureSelection(Enumeration figures) {
        Vector vector = new Vector();
        while (figures.hasMoreElements()) {
	  vector.addElement(figures.nextElement());
	}
	fData = writeData(vector);
    }
  
    private byte[] writeData(Vector vector) {
        // a FigureSelection is represented as a flattened ByteStream
        // of figures.
        ByteArrayOutputStream output = new ByteArrayOutputStream(200);
	try{
	  ObjectOutputStream writer = new ObjectOutputStream(output);
	  writer.writeInt(vector.size());
	  Enumeration figures = vector.elements();
	  while (figures.hasMoreElements()) {
            Figure figure = (Figure) figures.nextElement();
            writer.writeObject(figure);
	  }
	  writer.close();
	} catch (IOException e) {
	  System.err.println("Figure serialization failed: "+e);
	}
        return output.toByteArray();
    }

    /**
     * Gets the type of the selection.
     */
    public String getType() {
        return TYPE;
    }

    /**
     * Gets the data of the selection. The result is returned
     * as a Vector of Figures.
     *
     * @return a copy of the figure selection.
     */
    public Object getData(String type) {
        if (type.equals(TYPE)) {
            InputStream input = new ByteArrayInputStream(fData);
            Vector result = new Vector(10);
            try {
	        ObjectInputStream reader = new ObjectInputStream(input);

		int numRead = 0;
                int count = reader.readInt();
                while (numRead < count) {
                    Figure newFigure = (Figure) reader.readObject();
                    result.addElement(newFigure);
                    numRead++;
                }
		reader.close();
            } catch (IOException e) {
                System.out.println("Figure deserialization failed: "+e);
            } catch (ClassNotFoundException e) {
                System.out.println("Figure deserialization failed: "+e);
            }
	    return result;
        }
        return null;
    }
}

