/*
 * @(#)FigureTransferCommand.java 5.1
 *
 */

package CH.ifa.draw.standard;

import java.util.*;
import CH.ifa.draw.util.*;
import CH.ifa.draw.framework.*;

/**
 * Common base clase for commands that transfer figures
 * between a drawing and the clipboard.
 */
abstract class FigureTransferCommand extends UndoableCommand {

    protected DrawingEditor fEditor;

   /**
    * Constructs a drawing command.
    * @param name the command name
    * @param view the target view
    */
    protected FigureTransferCommand(String name, DrawingEditor editor) {
        super(name, editor);
        fEditor = editor;
    }

   /**
    * Deletes the selection from the drawing.
    */
    protected void deleteSelection() {
       DrawingView view = fEditor.view();
       view.drawing().removeAll(view.selection());
       view.clearSelection();
    }

   /**
    * Copies the selection to the clipboard.
    */
    protected void copySelection() {
        FigureSelection selection = fEditor.view().getFigureSelection();
        Clipboard.getClipboard().setContents(selection);
    }

   /**
    * Inserts a vector of figures and translates them by the
    * given offset.
    */
    protected void insertFigures(Vector figures, int dx, int dy) {
        DrawingView view = fEditor.view();

	view.addAll(figures);
	view.addToSelectionAll(figures);

	// OK: Finally fixed that annoying double move bug.
	// We could also call moveSelection, since exactly the added
	// elements should be selected.
	StandardDrawingView.moveFigures(figures,dx,dy);
        view.checkDamage();
    }

}
