/*
 * @(#)ChangeAttributeCommand.java 5.1
 *
 */

package CH.ifa.draw.standard;

import java.awt.*;
import java.awt.event.*;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.util.*;

/**
 * Command to change a named figure attribute.
 * <p>
 * The command's effects are undoable step by step.
 * Each use of the apply button can be undone separately.
 * So this command doesn't need to inherit UndoableCommand.
 * </p>
 */
public  class QueryAttributeCommand
        extends Command {

    DrawingEditor fEditor;
    Dialog dialog;
    String fAttribute;
    TextField input;

   /**
    * Constructs a query attribute command.
    * @param name the command name
    * @param attributeName the name of the attribute to be changed
    * @param type the attribute type (class)
    * @param editor the target editor
    */
    public QueryAttributeCommand(String displayName,
				 String name, final String attributeName,
				 final Class type, final DrawingEditor editor)
    {
        super(name);
	this.fEditor=editor;
	this.fAttribute=attributeName;
	dialog=new Dialog(editor.menuFrame(),
			  "Change "+displayName+":");
	input=new TextField();
	input.addKeyListener(new KeyAdapter() {
	    public void keyPressed(KeyEvent e) {
		if (e.getKeyCode()==KeyEvent.VK_ENTER) {
		    apply(type);
		} else if (e.getKeyCode()==KeyEvent.VK_ESCAPE) {
		    dialog.setVisible(false);
		}
	    }
	});
	dialog.add(input,"Center");
	Button apply=new Button(" Apply ");
	apply.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		apply(type);
	    }
	});
	Button update=new Button(" Update ");
	update.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		execute();
	    }
	});
	Button ok=new Button(" OK ");
	ok.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		apply(type);
		dialog.setVisible(false);
	    }
	});
	Button cancel=new Button(" Cancel ");
	cancel.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		resetFocus();
		dialog.setVisible(false);
	    }
	});
	Panel panel=new Panel();
	panel.setLayout(new FlowLayout());
	panel.add(apply);
	panel.add(update);
	panel.add(ok);
	panel.add(cancel);
	dialog.add(panel,"South");
	dialog.addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {
		dialog.setVisible(false);
	    }
	});
	dialog.pack();
    }

    public void execute() {
        DrawingView view = fEditor.view();
	Object value=null;
        FigureEnumeration k = view.selectionElements();
        while (k.hasMoreElements()) {
            Figure f = k.nextFigure();
            Object val=f.getAttribute(fAttribute);
	    if (val!=null) {
		if (value!=null && !value.equals(val)) {
		    // different values, use previous value
		    value=null;
		    break;
		}
		value=val;
	    }
        }
	String valuestr="";
	if (value!=null) {
	    if (value instanceof Color) {
		valuestr="0x"+
		    Integer.toHexString(((Color)value).getRGB()).substring(2);
	    } else {
		valuestr=value.toString();
	    }
	}
	input.setText(valuestr);
	resetFocus();
 	dialog.show();
    }

    private void resetFocus() {
	input.requestFocus();
	input.setSelectionStart(0);
	input.setSelectionEnd(input.getText().length());
    }

    public boolean isExecutable() {
        return fEditor.view().selectionCount() > 0;
    }

    private boolean apply(Class type) {
	String valuestr=input.getText();
	Object value=null;
	try {
	    if (type==Integer.class) {
		value=Integer.valueOf(valuestr);
	    } else if (type==String.class) {
		value=valuestr;
	    } else if (type==Color.class) {
		value=Color.decode(valuestr);
	    }
	} catch (NumberFormatException ex) {  }
	if (value!=null) {
	    new ChangeAttributeCommand("",fAttribute,value,fEditor)
		.execute();
	}
	resetFocus();
	return value!=null;
    }

}


