package CH.ifa.draw.standard;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
//import de.renew.util.*;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.util.*;
import CH.ifa.draw.figures.*;
import CH.ifa.draw.application.*;


/** Constructs a SearchFrame or SearchAndReplaceFrame
    according to the selected mode and displays it.
*/

public class SearchReplaceFrame {

    private DrawApplication application;
    private Enumeration drawings;
    private FigureEnumeration figures;
    private Drawing drawing;
    private Frame frame;
    private boolean stringMatcherValid = false;
    private StringMatcher substringMatcher;
    private int mode;

    public static final int SEARCHMODE = 0;
    public static final int SEARCHREPLACEMODE = 1;


    public SearchReplaceFrame(DrawApplication application, int aMode) {
	String frameTitle;
	String buttonTitle;

        this.application=application;
	mode = aMode;

	if (mode == SEARCHMODE) {
	    frameTitle ="Search";
	} else {
	    frameTitle ="Search & Replace";
	} 
	

	frame=new Frame(frameTitle);

	if (!GUIProperties.avoidFrameReshape()) {
	  frame.setSize(600,200);
	} 
	GridBagLayout gridBag=new GridBagLayout();
	frame.setLayout(gridBag);

	GridBagConstraints c=new GridBagConstraints();
        c.fill=GridBagConstraints.BOTH;
	c.weightx=1;

	Label label=new Label("Search for:");
	gridBag.setConstraints(label, c);
	frame.add(label);

	c.gridwidth = GridBagConstraints.REMAINDER; //end row
	final TextField searchTextField=new TextField("",30);
	gridBag.setConstraints(searchTextField, c);
	searchTextField.addKeyListener(new KeyAdapter() {
		public void keyTyped(KeyEvent e) {
		    reset(); //search string is modified: reset search process
		}
	    });
	frame.add(searchTextField);

	final TextField replaceTextField=new TextField("",30);
	if (mode == SEARCHREPLACEMODE) {
	    c.gridwidth = 1;
	    label=new Label("Replace with:");
	    gridBag.setConstraints(label, c);
	    frame.add(label);

	    c.gridwidth = GridBagConstraints.REMAINDER; //end row
	    gridBag.setConstraints(replaceTextField, c);
	    replaceTextField.addKeyListener(new KeyAdapter() {
		    public void keyTyped(KeyEvent e) {
			stringMatcherValid = false; //replace string is modified
		}
	    });
	    frame.add(replaceTextField);
	}
	    
	c.weightx=1;


	if (mode == SEARCHMODE) {
	    buttonTitle ="Search";
	} else {
	    buttonTitle ="Search & Replace";
	} 
	Button searchButton=new Button(buttonTitle);
	gridBag.setConstraints(searchButton, c);
	frame.add(searchButton);
	searchButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    if (!stringMatcherValid) {
			if (mode == SEARCHMODE) {
			    substringMatcher= new SubstringMatcher
				(searchTextField.getText(),
				 null);
			} else {
			    substringMatcher= new SubstringMatcher
				(searchTextField.getText(),
				 replaceTextField.getText());
			} 
			stringMatcherValid = true;
		    }
		    searchInAllDrawings();
		}
	    });

	c.gridwidth = GridBagConstraints.REMAINDER; //end row
	Button cancelButton=new Button("Cancel");
	gridBag.setConstraints(cancelButton, c);
	frame.add(cancelButton);
	cancelButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    cancel();
		}
	    });

	
	frame.addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent w) {
		    frame.setVisible(false);
		}
	    });
	
	//frame.validate();

	frame.pack();

	reset();

    }

    /**
       Handles a found TextFigure wrt. the actual mode. In case of a
       SEARCHREPLACEMODE this is done by prompting the user with the
       replacement.  
    */
    private void handleFoundFigure(TextFigure foundFigure) {
	if (mode == SEARCHREPLACEMODE) {
	    String foundText = foundFigure.getText();
	    String replacement = substringMatcher.replacement(foundText);
	    YesNoCancelDialog yNCD = new YesNoCancelDialog
		(frame,"Confirm Replacement",
		 "Replace:\n"+foundText+"\nwith:\n"+replacement);
	
	    switch (yNCD.execute()) {
	    case  YesNoCancelDialog.CANCEL : 
		reset(); 
		setVisible(false);
		return;
	    case  YesNoCancelDialog.YES :
	        application.prepareUndoSnapshot();
		foundFigure.setText(replacement);
	        application.commitUndoSnapshot();
		application.view().checkDamage();
		break;
	    case  YesNoCancelDialog.NO :			
		break;
	    } 
	}
    } 
	




    /**
     * Searches for a matching string in @see CH.ifa.draw.figures.TextFigure s
     *   and calls @see handleFoundFigure(TextFigure t) for each match.
     *
     * @author  Michael Koehler, Heiko Roelke
     * 
     *
     */
    private void searchInAllDrawings() {
	TextFigure foundFigure=null;
	do {
	    if (drawing==null) { //start a new search in first/next drawing
		if (drawings.hasMoreElements()) {
		    drawing=(Drawing)drawings.nextElement();
		    figures=drawing.figures();
		} else {
		    Toolkit.getDefaultToolkit().beep();
		    return;   //no more drawings :((   
		}
	    }

	    foundFigure=searchInOneDrawing();
	    if (foundFigure!=null) {
		application.selectOffendingElements
		    (new FigureException("Heureka", "Ich hab's gefunden", 
					 drawing, foundFigure));
		if ( mode == SEARCHMODE) {
		    return;
		} else {
		    handleFoundFigure(foundFigure);		    
		} 				
	    } 
	    else {
		// no more figures in this drawing, next one.
		drawing=null;
	    } 
	    
	} while (drawings.hasMoreElements());	    
	if (!figures.hasMoreElements() & foundFigure==null) {
	    Toolkit.getDefaultToolkit().beep();
	    // optional: reset();
	}
    }


    private TextFigure searchInOneDrawing() {
	while (figures.hasMoreElements()) {
		Figure figure=figures.nextFigure();
		if (figure instanceof TextFigure) {
		    TextFigure textFigure=(TextFigure)figure;
		    String figureText=textFigure.getText(); 
		    if(substringMatcher.matches(figureText)) {
			return textFigure;//Heureka!
		    }
		}
	    }
	    return null;
    }


    public void reset() {
	drawings=application.drawings();
	drawing=null;
	figures=null;
	stringMatcherValid = false;
    };

    public void cancel() {
	this.setVisible(false);	
    }

    public void setVisible(boolean b) {
	frame.setVisible(b);
    }


}


