/*
 * @(#)SnapToGridCommand.java 5.1
 *
 */

package CH.ifa.draw.standard;

import java.util.*;
import java.awt.Point;
import CH.ifa.draw.framework.*;

/**
 * A command to toggle the snap to grid behavior.
 */
public class SnapToGridCommand extends UndoableCommand {

    protected DrawingEditor fEditor;
    private Point fGrid;

   /**
    * Constructs a snap to grid command.
    * @param name the command name
    * @param editor the editor
    * @param grid the grid size. A grid size of 1,1 turns grid snapping off.
    */
    public SnapToGridCommand(String name, DrawingEditor editor, Point grid) {
        super(name, editor);
        fEditor = editor;
        fGrid = new Point(grid.x, grid.y);
    }

    public boolean isExecutable() {
        return fEditor.view().selectionCount() > 0;
    }

    public boolean executeUndoable() {
        DrawingView view = fEditor.view();
        PointConstrainer grid = view.getConstrainer();
        if (grid == null) {
System.out.println("No active grid - using default grid.");
	    grid=new GridConstrainer(fGrid.x, fGrid.y);
        }

        FigureEnumeration selection = view.selectionElements();

        while (selection.hasMoreElements()) {
            Figure f = selection.nextFigure();
            Point c = f.center();
	    Point cc = grid.constrainPoint(f.center());
	    f.moveBy(cc.x-c.x,cc.y-c.y);
        }
        view.checkDamage();
	return true;
    }

}
