/**
 * SplineAttributeCommand.java
 *
 *
 * Created: Wed Dec 13 15:29:02 2000
 *
 * @author Friedrich Delgado Friedrichs, Lutz Kirsten
 * @version
 */

package CH.ifa.draw.standard;

import java.awt.*;
import java.awt.event.*;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.util.*;
import CH.ifa.draw.figures.PolyLineFigure;

public class SplineAttributeCommand extends UndoableCommand {

    DrawingEditor fEditor;
    String fAttribute;
    int value;

   /**
    * Constructs a spline attribute command.
    * @param name the command name
    * @param attributeName the name of the attribute to be changed
    * @param value the int value "attributeName" should be set to
    */
    public SplineAttributeCommand(String name, final String attributeName,
				  int pvalue, final DrawingEditor editor) {
	super(name, editor);
	this.fEditor=editor;
	fAttribute=attributeName;
	value=pvalue;	
    }

    public boolean isExecutable() {
	DrawingView view = fEditor.view();
	if (view.selectionCount() <= 0) {
	    return false;
	} else if (view.selectionCount() > 1) {
	    return true;
	} else {
	    return (view.selectionElements().nextElement() instanceof PolyLineFigure);
	}
    }

    public boolean executeUndoable() {
      if (isExecutable()) {
	DrawingView view = fEditor.view();
	Object val = new Integer(value);
	Object spline = new Integer(PolyLineFigure.BSPLINE_SHAPE);
	FigureEnumeration k = view.selectionElements();
        while (k.hasMoreElements()) {
          Figure f = k.nextFigure();
	  if (f instanceof PolyLineFigure) {	  
	    f.setAttribute("LineShape", spline);
	    if (fAttribute.equals("standard")) {
		f.setAttribute("BSplineSegments", 
			       new Integer(BSpline.DEFSEGMENTS));
		f.setAttribute("BSplineDegree", 
			       new Integer(BSpline.DEFDEGREE));
	    } else {
		f.setAttribute(fAttribute, val);
		// System.out.println(fAttribute + ": " + val);
	    }
	  }
	}
	view.drawAll(view.getGraphics());
	return true;
      }
      return false;
    }
    
} // SplineAttributeCommand
