package CH.ifa.draw.standard;

import CH.ifa.draw.framework.*;
import CH.ifa.draw.util.Command;

/**
 * Restores one undo or redo step.
 *
 * UndoRedoCommand.java
 * Created: Wed Dec 13  2000
 * @author Michael Duvigneau
 * 
 * @see CH.ifa.draw.framework.UndoRedoManager
 */

public class UndoRedoCommand extends Command {
  
  public static final int UNDO = 1;
  public static final int REDO = 2;

  private DrawingEditor editor;
  private int mode;

  public UndoRedoCommand(String name, DrawingEditor editor, int mode) {
    super(name);
    this.editor = editor;
    this.mode = mode;
  }

  public boolean isExecutable() {
    SnapshotHistory history = null;
    switch (mode) {
    case UNDO:
      history = editor.getUndoRedoManager().getUndoHistory(editor.drawing());
      break;
    case REDO:
      history = editor.getUndoRedoManager().getRedoHistory(editor.drawing());
      break;
    }
    return (history != null)
        && (! history.isEmpty());
  }

  public void execute() {
    if (isExecutable()) {
      editor.toolDone();
      switch (mode) {
      case UNDO:
	editor.getUndoRedoManager().restoreUndoSnapshot(editor.drawing());
	editor.selectionChanged(editor.view());
        break;
      case REDO:
	editor.getUndoRedoManager().restoreRedoSnapshot(editor.drawing());
	editor.selectionChanged(editor.view());
        break;
      }
    }
  }
} // UndoRedoCommand
