package de.renew.call;

import java.io.*;
import de.renew.util.StringUtil;

public class StubCompiler {

  private static String AUTO_COMMENT = "// This file was automatically generated. Do not modify.";

  public static void main(String[] args) {
    if (args.length==0) {
      System.err.println("Please provide the names of the stub files to be");
      System.err.println("compiled as arguments to this command. Each file");
      System.err.println("should have the extension '.stub'. The resulting");
      System.err.println("Java sources will be placed in the corresponding");
      System.err.println("'.java' files. Existing files will be overwritten");
      System.err.println("only if the \"do not modify\" warning in the first");
      System.err.println("line is unchanged.");
      
      System.exit(1);
    }
    for (int i=0;i<args.length;i++) {
      String sourceName=args[i];
      if (!"stub".equals(StringUtil.getExtension(sourceName))) {
	sourceName+=".stub";
      }

      File sourceFile=new File(sourceName);
      String canonicalSourceName=null;
      try {
	canonicalSourceName=sourceFile.getCanonicalPath();
      }
      catch (IOException e) {
	System.out.println("Source file could not be located.");
        System.exit(1);
      }
      String targetName=StringUtil.getPath(canonicalSourceName)+
	File.separator+
	StringUtil.getFilename(sourceName)+".java";

      File targetFile=new File(targetName);
      if (!sourceFile.canRead()) {
	System.err.println("Cannot read source file "+sourceName+". "+
	  "Ignoring this argument.");
      } else {
	boolean exists = false;
	boolean mayWrite = true;
	if (targetFile.exists()) {
	  exists = true;
	  mayWrite = false;
	  BufferedReader targetReader = null;
	  try {
	    targetReader=new BufferedReader(new FileReader(targetFile));
	    if (targetReader.ready()) {
	      String firstLine = targetReader.readLine();
	      if (firstLine.equals(AUTO_COMMENT)) {
		mayWrite = true;
	      } else {
		System.err.println("Target file "+targetName+" already exists and was modified. "+
		  "Ignoring this argument.");
	      }
	    } else {
	      System.err.println("Target file "+targetName+" already exists. "+
		"Ignoring this argument.");
	    }
	  } catch (Exception e) {
	    System.err.println("Target file "+targetName+" already exists and could not be checked. "+
	      "Ignoring this argument.");
	  }
	  try {
	    targetReader.close();
	  } catch (Exception ex) {
	    // This is probably due to earlier errors.
	  }
	} 
	if (mayWrite) {
	  FileReader sourceReader=null;
	  PrintWriter targetWriter=null;

	  try {
	    sourceReader=new FileReader(sourceFile);
	    targetWriter=new PrintWriter(new FileWriter(targetFile));

	    StubParser parser=new StubParser(sourceReader);
	    targetWriter.println(AUTO_COMMENT);
	    parser.setWriter(targetWriter);
	    parser.StubFile();
	    sourceReader.close();
	    targetWriter.close();
	    System.err.println("Target file "+targetName+(exists?" updated.":" created."));
	  }
	  catch (Exception e) {
	    System.err.println(
              "An error occurred during parsing the stub file.");
	    System.err.println(e.getMessage());
	    System.err.println("Trying to clean up and continue.");
	    try {
	      sourceReader.close();
	    }
	    catch (Exception ex) {
	      // This is probably due to earlier errors.
	    }
	    try {
	      targetWriter.close();
	    }
	    catch (Exception ex) {
	      // This is probably due to earlier errors.
	    }
	    try {
	      if (targetFile.exists()) {
		targetFile.delete();
	      }
	    }
	    catch (Exception ex) {
	      System.err.println("Could not remove broken file "+
		targetName+".");
	    }
	  }
	}
      }
    }
  }
}
