package de.renew.formalism.fs;

import de.renew.shadow.SyntaxException;
import de.renew.shadow.*;
import collections.*;
import java.util.*;
import java.util.Enumeration;
import de.uni_hamburg.fs.*;
import de.renew.formalism.java.*;
import de.renew.simulator.*;
import de.renew.expression.*;
import de.renew.formalism.fs.*;

public class FSNetCompiler extends JavaNetCompiler implements FSShadowCompiler {
  static final long serialVersionUID = 262012294953452116L;

  public static final String LINK="Link";
  public static final Name RCV=new Name("rcv");
  public static final Name PARAM=new Name("param");
  private static ConceptImpl linkConcept=null;
  private static Type linkType=null;

  public FSNetCompiler() {
    super(true,false,true);
  }

  public static Type getLinkType() {
    if (linkType==null && linkConcept!=null) {
	linkType=new ConjunctiveType(linkConcept);
    }
    return linkType;
  }

  protected InscriptionParser makeParser(String inscr) {
    return new FSNetParser(new java.io.StringReader(inscr));
  }

  protected CollectionEnumeration parseArcInscription(String inscr,
    ParsedDeclarationNode declaration) throws SyntaxException
  {
    if (inscr==null || inscr.equals("")) {
      inscr="[]";
    }
    return super.parseArcInscription(inscr,declaration);
  }

  public void checkConcept(String conceptName, String appropStr, ShadowNet shadowNet)
    throws SyntaxException {
 }

  // Compile a net system and return a lookup that knows
  // about the compiled nets.
  public synchronized ShadowLookup compile(ShadowNetSystem netSystem)
    throws SyntaxException {
       return compile(new FSShadowLookup(), netSystem);
  }

  public synchronized ShadowLookup compileMore(ShadowNetSystem netSystem) 
    throws SyntaxException
  {
    throw new UnsupportedOperationException(
      "FSNetCompiler: On-demand compilation not supported.");
  }

  protected ParsedDeclarationNode makeEmptyDeclarationNode(ShadowNet net) {
      if (net!=null)
	  return new ParsedFSDeclarationNode(net.getName());
      return new ParsedFSDeclarationNode();
  }

  protected void createNetElements() throws SyntaxException {
    ShadowNetSystem netSystem=getNetSystem();
    // compile the type hierarchy...
    // First, remove all (old) user concepts:
    TypeSystem ts=TypeSystem.newInstance();
    Partition topPartition=ts.getTopPartition();
    // add link concept
    linkConcept=new ConceptImpl(LINK);
    linkConcept.addApprop(RCV,new ParsedConjunctiveType(new ConceptSet(
                   ts.getJavaConcept(NetInstance.class))));
    linkConcept.addApprop(PARAM,ParsedType.PARSED_TOP);
    // second, compile all concepts from all drawings:
    CollectionEnumeration enum=netSystem.elements();
    while (enum.hasMoreElements()) {
      compileConcepts((ShadowNet)enum.nextElement(),topPartition);
    }
    // then, add all "approp"-information:
    enum=netSystem.elements();
    while (enum.hasMoreElements()) {
      compileApprops((ShadowNet)enum.nextElement());
    }
    // then, add all "isNotA"-information:
    enum=netSystem.elements();
    while (enum.hasMoreElements()) {
      compilePartitions((ShadowNet)enum.nextElement());
    }
    // last, add all "isa"-information:
    enum=netSystem.elements();
    while (enum.hasMoreElements()) {
      compileIsas((ShadowNet)enum.nextElement());
    }
    // and now:
    try {
	ts.inheritFeatures();
    } catch (UnificationFailure uff) {
	SyntaxException ex;
	if (uff instanceof TypeException) {
	    TypeException tex=(TypeException)uff;
	    String msg="Feature \""+tex.featureName
		+"\" has been redefined illegally in concept \""
		+tex.concept.getName()+"\".";
	    ex=new SyntaxException(msg,null);
	    FSShadowLookup lookup=(FSShadowLookup)getLookup();
	    ex.addObject(lookup.getApprop(tex.concept,tex.featureName));
	} else {
	    ex=new SyntaxException("Unification failure while constructing feature types.",null);
	}
	throw ex;
    }
    // all additional work has been done.
    // continue in superclass:
    super.createNetElements();
  }

  private void compileConcepts(ShadowNet net,
			       Partition topPartition) throws SyntaxException {
   FSShadowLookup lookup=(FSShadowLookup)getLookup();
   int counter=0;
   String netName=net.getName();
   CollectionEnumeration enum=net.elements();
   while (enum.hasMoreElements()) {
      ShadowNetElement elem=(ShadowNetElement)enum.nextElement();
      if (elem instanceof ShadowConcept) {
	  ShadowConcept shadowConcept=(ShadowConcept)elem;
	  if (shadowConcept.getNamespace().equals(netName)) {
	      // only compile "local" concepts
	      String conceptName=shadowConcept.getFullName();
System.out.println("Compiling concept "+conceptName+"...");
              // Concept with that name may already exist:
              if (TypeSystem.instance().hasConcept(conceptName)) {
	        SyntaxException ex=
		  new SyntaxException("The concept \""
				      +conceptName+"\" already exists.",
				      null);
	        ex.addObject(shadowConcept);
	        throw ex;
	      }
	      ConceptImpl concept=new ConceptImpl(conceptName);
	      lookup.setConcept(shadowConcept,concept);
	      if (!shadowConcept.elements().hasMoreElements()) {
	          // is this a top level concept?
	          try {
		      topPartition.addConcept(concept);
	          } catch (TypeException tex) {
		      // should never happen!
		      throw new RuntimeException(tex.toString());
	          }
	      }
	      //System.out.println("Created Concept "+conceptName);
	  }
      }
    }
  }

  private Concept lookup(ShadowConcept shadowConcept)
                  throws SyntaxException {
    // check referenced concepts:
    String conceptName=shadowConcept.getFullName();
    if (!TypeSystem.instance().hasConcept(conceptName)) {
      SyntaxException ex=new SyntaxException("Referenced Concept "+conceptName+" does not exist.",null);
      ex.addObject(shadowConcept);
      throw ex;
    }
    return TypeSystem.instance().conceptForName(conceptName);
  }

  private void compileApprops(ShadowNet net) throws SyntaxException {
    FSShadowLookup lookup=(FSShadowLookup)getLookup();
    CollectionEnumeration enum=net.elements();

    enum=net.elements();
    while (enum.hasMoreElements()) {
	ShadowNetElement elem=(ShadowNetElement)enum.nextElement();
	if (elem instanceof ShadowAssoc) {
	    ShadowAssoc shadowAssoc=(ShadowAssoc)elem;
	    ConceptImpl concept=(ConceptImpl)lookup(shadowAssoc.getConcept());
	    ConceptSet typ=new ConceptSet(lookup(shadowAssoc.getType()));
	    int mult=shadowAssoc.getMultiplicity();
	    Name featureName=shadowAssoc.getFeature();
	    ParsedType pt=new ParsedConjunctiveType(typ);
	    if (mult!=ShadowAssoc.ZERO_OR_ONE) {
		pt=new ParsedListType(mult==ShadowAssoc.ONE_OR_MORE,pt);
	    }
	    concept.addApprop(featureName,pt);
	    lookup.setApprop(concept,featureName,shadowAssoc);
    //System.out.println("Found Assoc: "+shadowAssoc.getFeature()+": "+typ);
	}
    }

    enum=net.elements();
    while (enum.hasMoreElements()) {
      ShadowNetElement elem=(ShadowNetElement)enum.nextElement();
      if (elem instanceof ShadowConcept) {
        ShadowConcept shadowConcept=(ShadowConcept)elem;
        ConceptImpl concept=(ConceptImpl)lookup(shadowConcept);
        // parse lines:
        FSInscriptionParser parser=(FSInscriptionParser)makeParser("\n"+shadowConcept.getApprop());
        parser.setLookup(lookup);
	ParsedFSDeclarationNode decl=
	    (ParsedFSDeclarationNode)makeDeclarationNode(net);
	try {
	    decl.addAccess(net.getName(),null);
	} catch (de.renew.formalism.java.ParseException e) { }
        parser.setDeclarationNode(decl);
        try {
	    OrderedTable approps=parser.parseAppropDef();
	    //concept.setApprops(approps);
	    Enumeration appropenum=approps.keys();
	    while (appropenum.hasMoreElements()) {
		Name featureName=(Name)appropenum.nextElement();
		concept.addApprop(featureName,(ParsedType)approps.at(featureName));
		lookup.setApprop(concept,featureName,
				 shadowConcept);
	    }
        } catch (de.renew.formalism.java.ParseException e) {
          SyntaxException ex=makeSyntaxException(e);
          ex.addObject(shadowConcept);
          throw ex;
        }
      }
    }
  }

  private void compilePartitions(ShadowNet net) throws SyntaxException {
    FSShadowLookup lookup=(FSShadowLookup)getLookup();
    CollectionEnumeration enum=net.elements();
    while (enum.hasMoreElements()) {
      ShadowNetElement elem=(ShadowNetElement)enum.nextElement();
      if (elem instanceof ShadowIsa) {
        ShadowIsa shadowIsa=(ShadowIsa)elem;
	if (shadowIsa.isDisjunctive()) {
	    ConceptImpl sub=(ConceptImpl)lookup(shadowIsa.getSource());
	    ConceptImpl supi=(ConceptImpl)lookup(shadowIsa.getTarget());
	    try {
		lookup.addToSubPartition(supi,sub);
	    } catch (TypeException tex) {
		SyntaxException ex=new SyntaxException("This disjunction leads to a contradiction.",null);
		ex.addObject(shadowIsa);
		throw ex;
	    }
	}
	    
      }
    }
  }

  private void compileIsas(ShadowNet net) throws SyntaxException {
    FSShadowLookup lookup=(FSShadowLookup)getLookup();
    CollectionEnumeration enum=net.elements();
    while (enum.hasMoreElements()) {
      ShadowNetElement elem=(ShadowNetElement)enum.nextElement();
      if (elem instanceof ShadowIsa) {
        ShadowIsa shadowIsa=(ShadowIsa)elem;
        ConceptImpl sub=(ConceptImpl)lookup(shadowIsa.getSource());
	ShadowConcept supi=shadowIsa.getTarget();
        ConceptImpl sup=(ConceptImpl)lookup(supi);
System.out.println("Compiling is-a from "+sub.getName()+" to "+sup.getName());
        try {
          sub.addIsa(sup);
        } catch (CyclicHierarchyException e) {
          SyntaxException ex=new SyntaxException("This is-a-relation leads to a cyclic type hierarchy.",null);
          ex.addObject(shadowIsa);
          throw ex;
        } catch (TypeException te) {
	    SyntaxException ex=new SyntaxException("This is-a-relation leads to a contradiction.",null);
	    ex.addObject(shadowIsa);
	    throw ex;
	}
      }
    }
  }

  protected ArcFactory getArcFactory(int arcType, boolean allowTime) {
  	return new FSArcFactory(arcType,allowTime);
  }

}
