package de.renew.formalism.java;

import de.renew.simulator.*;
import de.renew.expression.Expression;
import de.renew.function.*;
import de.renew.shadow.*;
import de.renew.unify.List;
import de.renew.util.Types;
import java.util.Vector;

public class FlexibleInArcFactory implements ArcFactory {
  public static FlexibleInArcFactory INSTANCE=new FlexibleInArcFactory();

  private FlexibleInArcFactory() {}

  public void emptyArcCheck() throws SyntaxException {
    throw new SyntaxException("Flexible arcs must be inscribed.",null);
  }

  public boolean allowsTime() {
    return false;
  }

  public void compileArc(Place place, Transition transition, boolean trace,
    Class placeType, TimedExpression timedExpr) throws SyntaxException
  {
    TypedExpression typedExpr=timedExpr.getExpression();

    Function forwardFunction=null;
    Function backwardFunction=null;

    Class targetType;
    if (placeType.isPrimitive()) {
      targetType=placeType;
    } else {
      targetType=null;
    }
    
    Class exprType=typedExpr.getType();
    if (exprType!=Types.UNTYPED) {
      if (exprType==null) {
				throw new SyntaxException("Null not allowed for flexible arcs.",null);
      } else if (!de.renew.unify.List.class.isAssignableFrom(exprType) &&
								 !java.util.Collection.class.isAssignableFrom(exprType) &&
								 !exprType.isArray())
				{
					throw new SyntaxException(
																		"Incorrect type for flexible arc inscription.",null);
				}
    }

    if (placeType!=Types.UNTYPED && exprType.isArray()) {
      // The place and the inscriptions and the individual elements
      // are typed. A conversion may be required.
      Class elementType=exprType.getComponentType();
      
      if (!Types.allowsLosslessWidening(placeType,elementType) &&
	!Types.allowsLosslessWidening(elementType,placeType))
      {
	throw new SyntaxException("Cannot losslessly convert "+
	  JavaHelper.makeTypeErrorString(placeType)+
	  " to "+JavaHelper.makeTypeErrorString(elementType)+
	  " or vice versa.",
	  null);
      }
      
      forwardFunction=new CastFunction(placeType);
      backwardFunction=new CastFunction(elementType);
    }
    
    FlexibleArc arc=new FlexibleArc(place,FlexibleArc.in,
      typedExpr.getExpression(),forwardFunction,backwardFunction);
    arc.setTrace(trace);
    transition.add(arc);		    
  }
}
