package de.renew.formalism.java;

import de.renew.simulator.*;
import de.renew.expression.Expression;
import de.renew.expression.ConstantExpression;
import de.renew.shadow.*;
import de.renew.util.Types;
import de.renew.util.Value;

public class SimpleArcFactory implements ArcFactory {
  private int arcType;
  private boolean allowsTime;
  
  public static Expression zeroExpression=
    new ConstantExpression(Double.TYPE,new Value(new Double(0)));
  
  public SimpleArcFactory(int arcType, boolean allowsTime) {
    this.arcType=arcType;
    this.allowsTime=allowsTime;
  }

  public void emptyArcCheck() {}

  public boolean allowsTime() {
    return allowsTime;
  }

  protected Arc getArc(Place place, int arcType,
  		       Expression expr, Expression timeExpr) {
    return new Arc(place,arcType,expr,timeExpr);
  }

  public void compileArc(Place place, Transition transition, boolean trace,
    Class placeType, TimedExpression timedExpr) throws SyntaxException
  {
    Expression expr;
    if (arcType==Arc.out) {
      expr=JavaNetHelper.makeCastedOutputExpression(placeType,
        timedExpr.getExpression());
    } else {
      expr=JavaNetHelper.makeCastedInputExpression(placeType,
        timedExpr.getExpression());
    }

    Expression timeExpr=null;
    if (allowsTime) {
      if (timedExpr.isTimed()) {
	timeExpr=timedExpr.getTime().getExpression();
      } else {
	timeExpr=zeroExpression;
      }
    }
  
    Arc arc=getArc(place,arcType,expr,timeExpr);
    arc.setTrace(trace);
    transition.add(arc);
  }
}
