package de.renew.function;

import de.renew.unify.*;
import de.renew.util.Value;

import de.renew.util.ReflectionSerializer;
import java.io.IOException;

// A cast function will attempt a cast whenever possible.
// If primitive values are casted, this might result in loss of
// precision.

public class CastFunction implements Function {
  /** 
   * This field is not really transient, but as <code>java.lang.Class
   * </code>is not always serializable, we have to store it by
   * ourselves.
   **/
  transient Class clazz;

  public CastFunction(Class clazz) {
    this.clazz=clazz;
  }

  public Object function(Object arg) throws Impossible {
    if (clazz.isPrimitive()) {
      if (arg instanceof Value) {
	Object value=((Value)arg).value;
	if (value instanceof Boolean) {
	  if (clazz==Boolean.TYPE) {
	    return arg;
	  }
	} else {
	  Number number;
	  if (value instanceof Character) {
	    char charVal=((Character)value).charValue();
	    number=new Integer((int)charVal);
	  } else {
	    number=(Number)value;
	  }
	  if (clazz==Character.TYPE) {
	    return new Value(new Character((char)number.intValue()));
	  } else if (clazz==Byte.TYPE) {
	    return new Value(new Byte(number.byteValue()));
	  } else if (clazz==Short.TYPE) {
	    return new Value(new Short(number.shortValue()));
	  } else if (clazz==Integer.TYPE) {
	    return new Value(new Integer(number.intValue()));
	  } else if (clazz==Long.TYPE) {
	    return new Value(new Long(number.longValue()));
	  } else if (clazz==Float.TYPE) {
	    return new Value(new Float(number.floatValue()));
	  } else if (clazz==Double.TYPE) {
	    return new Value(new Double(number.doubleValue()));
	  }  
	}
      }
    } else {
      if (arg==null || clazz.isInstance(arg)) {
	return arg;
      }
    }
    throw new Impossible();
  }

  /**
   * Serialization method, behaves like default writeObject
   * method. Stores the not-really-transient clazz field.
   **/
  private void writeObject(java.io.ObjectOutputStream out)
      throws IOException {
      out.defaultWriteObject();
      ReflectionSerializer.writeClass(out, clazz);
  }

  /**
   * Deserialization method, behaves like default readObject
   * method. Restores the not-really-transient clazz field.
   **/
  private void readObject(java.io.ObjectInputStream in)
      throws IOException, ClassNotFoundException {
      in.defaultReadObject();
      clazz = ReflectionSerializer.readClass(in);
  }

}
