package de.renew.function;

import de.renew.unify.*;
import de.renew.util.ReflectionSerializer;
import java.lang.reflect.*;
import java.io.IOException;


public class DynamicConstructorFunction implements Function {
  /**
   * This field is not really transient, but as <code>java.lang.Class
   * </code>is not always serializable, we have to store it by
   * ourselves.
   **/
  transient Class constr;

  public DynamicConstructorFunction(Class clazz) {
    constr=clazz;
  }

  public Object function(Object param) throws Impossible {
    Tuple tuple=(Tuple)param;

    Object[] paramArr=new Object[tuple.getArity()];
    for (int i=0;i<paramArr.length;i++) {
      paramArr[i]=tuple.getComponent(i);
    }

    try {
      return Executor.executeConstructor((Class)constr,paramArr);
    } catch (Exception e) {
			if (e instanceof InvocationTargetException) {
				if (MethodFunction.DEBUG_METHOD_CALLS) {
					((InvocationTargetException)e).getTargetException().printStackTrace();
				}
				throw new Impossible("InvocationTargetException occured during constructor call: "
					+ ((InvocationTargetException) e).getTargetException());
			} else {
				if (MethodFunction.DEBUG_METHOD_CALLS) {
					e.printStackTrace();
				}
				throw new Impossible("Exception occured during constructor call: " + e);
			}
		}
  }

  /**
   * Serialization method, behaves like default writeObject
   * method. Stores the not-really-transient constr field.
   **/
  private void writeObject(java.io.ObjectOutputStream out)
      throws IOException {
      out.defaultWriteObject();
      ReflectionSerializer.writeClass(out, constr);
  }

  /**
   * Deserialization method, behaves like default readObject
   * method. Restores the not-really-transient constr field.
   **/
  private void readObject(java.io.ObjectInputStream in)
      throws IOException, ClassNotFoundException {
      in.defaultReadObject();
      constr = ReflectionSerializer.readClass(in);
  }


  public String toString() {
    return "DynConstrFunc("+de.renew.util.Types.typeToString(constr)+")";
  }
}

