/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

package de.renew.gui;

import java.awt.*;
import java.awt.event.*;
import java.rmi.RemoteException;
import java.util.Vector;
import collections.CollectionEnumeration;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.standard.*;
import de.renew.simulator.*;
import de.renew.event.*;
import de.renew.remote.*;
import de.renew.util.StringUtil;
import CH.ifa.draw.util.Fontkit;
import javax.swing.JOptionPane;

/**
 * Class documentation.
 */
class BindingSelectionFrame implements ActionListener, ItemListener, TimeListener, RemoteTriggerable
{

	// Accesses to this field should use class synchronisation.
	private static BindingSelectionFrame instance = null;

	/**
	 * Method documentation.
	 * @param transitionInstance
	 * @param simulation
	 */
	static synchronized void open(TransitionInstanceAccessor transitionInstance, CPNSimulation simulation)
	{
		if (instance == null)
		{
			instance = new BindingSelectionFrame();
		}
		instance.setup(transitionInstance, simulation);
	}

	/**
	 * Method documentation.
	 */
	static synchronized void close()
	{
		if (instance != null)
		{
			instance.instanceClose();
		}
	}

	private Frame frame;
	private TextArea currentElement;
	private List bindingList;
	private Button fireButton;

	private BindingAccessor[] bindings;
	private String[] fullBindingText;
	private TransitionInstanceAccessor transitionInstance;
	private CPNSimulation simulation;
	private TriggerableForwarder triggerableForwarder;

	private RenewMode mode;

	private boolean dirty = false;
	private boolean inSearch = false;
	private boolean wantClose = false;
	private Object updateSynchronizer = new Object();

	/**
	 * Constructor documentation.
	 */
	private BindingSelectionFrame()
	{
		frame = new Frame();

		currentElement = new TextArea(5, 20);
		currentElement.setEditable(false);
		currentElement.setFont(Fontkit.getFont("Monospaced", Font.PLAIN, 11));
		frame.add(currentElement, "Center");

		bindingList = new List(10);
		bindingList.addActionListener(this);
		bindingList.addItemListener(this);
		frame.add(bindingList, "West");

		fireButton = new Button(" Fire ");
		fireButton.addActionListener(this);

		Button updateButton = new Button(" Update ");

		updateButton.addActionListener(new ActionListener()
		{
			/**
			 * Method documentation.
			 * @param e
			 */
			public void actionPerformed(ActionEvent e)
			{
				update();
			}

		});

		Button closeButton = new Button(" Close ");

		closeButton.addActionListener(new ActionListener()
		{
			/**
			 * Method documentation.
			 * @param e
			 */
			public void actionPerformed(ActionEvent e)
			{
				close();
			}

		});

		Panel buttonPanel = new Panel();

		buttonPanel.add(fireButton);
		buttonPanel.add(updateButton);
		buttonPanel.add(closeButton);
		frame.add(buttonPanel, "South");
		frame.addWindowListener(new WindowAdapter()
		{
			/**
			 * Method documentation.
			 * @param e
			 */
			public void windowClosing(WindowEvent e)
			{
				close();
			}

		});
		frame.pack();

		// The binding frame will remain a listener forever.
		// It just does not evaluate the notifications, if it is closed.
		SearchQueue.insertTimeListener(this);

		try
		{
			triggerableForwarder = new TriggerableForwarderImpl(this);
		}
		catch (RemoteException e)
		{
			e.printStackTrace(System.err);
			JOptionPane.showMessageDialog(frame,
				"A problem occurred: " + e + "\n"
				+ "See the console for details.",
				"Binding selection frame", JOptionPane.ERROR_MESSAGE);
		}
	}

	/**
	 * Method documentation.
	 */
	private synchronized void instanceClose()
	{
		if (inSearch)
		{
			wantClose = true;
		}
		else
		{
			closeNow();
		}
	}

	/**
	 * Method documentation.
	 */
	private void closeNow()
	{
		// The forwarder can still be useful when the
		// BindingSelectionFrame instance is reopened.
		//triggerableForwarder = null;
		frame.setVisible(false);
		bindingList.removeAll();
		transitionInstance = null;
		wantClose = false;
	}

	/**
	 * Method documentation.
	 * @param e
	 */
	public void itemStateChanged(ItemEvent e)
	{
		int type = e.getStateChange();
		String text = "";

		if (type == ItemEvent.SELECTED)
		{
			int index = bindingList.getSelectedIndex();

			if (index >= 0 && index < bindings.length)
			{
				text = fullBindingText[index];
			}
		}
		currentElement.setText(text);
		currentElement.setCaretPosition(0);
	}

	/**
	 * Method documentation.
	 * @param e
	 */
	public void actionPerformed(ActionEvent event)
	{
		int index = bindingList.getSelectedIndex();

		if (index >= 0 && index < bindings.length)
		{
			simulation.getBreakpointManager().clearLog();
			try
			{
				bindings[index].execute(new TracerAccessorImpl(
					new ConsoleTracer("Synchronously", "(Manually) ")),
					!simulation.isStrictlySequential());
			}
			catch (RemoteException e)
			{
				e.printStackTrace(System.err);
				JOptionPane.showMessageDialog(frame,
					"A problem occurred: " + e + "\n"
					+ "See the console for details.",
					"Binding selection frame", JOptionPane.ERROR_MESSAGE);
			}
			simulation.simulationRefresh();
			update();
		}
		else
		{
			fireButton.setEnabled(false);
		}
	}

	/**
	 * Method documentation.
	 * @param transitionInstance
	 * @param simulation
	 */
	private void setup(TransitionInstanceAccessor transitionInstance, CPNSimulation simulation)
	{
		this.transitionInstance = transitionInstance;
		wantClose = false;
		this.simulation = simulation;

		try
		{
			frame.setTitle(transitionInstance.asString() + "'s possible bindings");
		}
		catch (RemoteException e)
		{
			e.printStackTrace(System.err);
			JOptionPane.showMessageDialog(frame,
				"A problem occurred: " + e + "\n"
				+ "See the console for details.",
				"Binding selection frame", JOptionPane.ERROR_MESSAGE);
		}

		update();
	}

	/**
	 * This event is issued whenever the corresponding TransitionInstance
	 * may have become enabled or disabled. The triggerable has to check
	 * enabledness itself.
	 */
	public void proposeSearch()
	{
		update();
	}

	/**
	 * This event is issued whenever the global simulation time
	 * changes. The triggerable has to check enabledness, because
	 * it only keeps track of currently activated bindings.
	 */
	public void timeAdvanced()
	{
		update();
	}

	/**
	 * Updates the binding list thread-safely.
	 * This is done in a separate thread, because of
	 * a deadlock problem when executing synchronously.
	 */
	public void update()
	{
		new BindingListUpdater().start();
	}

	/**
	 * Updates the binding list thread-safely.
	 */
	private class BindingListUpdater extends Thread
	{
		/**
		 * Creates a new updater.
		 */
		private BindingListUpdater()
		{
		}

		/**
		 * Updates the binding list thread-safely.
		 * This method is encapsulated in a separate thread, because of
		 * a deadlock problem when executing synchronously.
		 */
		public void run()
		{
			synchronized (updateSynchronizer)
			{
				if (inSearch)
				{
					// In a different thread there is a search prozess running.
					// Notify it about possible new bindings and forget about it.
					dirty = true;
					return;
				}
				else if (wantClose)
				{
					closeNow();
					return;
				}
				else
				{
					// Request the start of a new search process.
					dirty = false;
					inSearch = true;
				}
			}

			// When I reach this point, I have aquired the inSearch flag.
			// I am supposed to do updates until the dirty flag remains
			// false.
			while (true)
			{
				// Was the window already closed?
				if (transitionInstance != null)
				{
					update();
					frame.setVisible(true);
				}

				synchronized (updateSynchronizer)
				{
					if (wantClose)
					{
						// Close the frame and make sure that the search terminates.
						closeNow();
						dirty = false;
					}
					if (!dirty)
					{
						// No more requests. Free the inSearch lock and
						// finally exit from this method.
						inSearch = false;
						return;
					}
					else
					{
						// Start another round.
						dirty = false;
					}
				}
			}
		}

		/**
		 * Does the AWT operations themselves.
		 */
		public void update()
		{
			try
			{
				bindings = transitionInstance.findAllBindings(triggerableForwarder);
			}
			catch (RemoteException e)
			{
				e.printStackTrace(System.err);
				JOptionPane.showMessageDialog(frame,
					"A problem occurred: " + e + "\n"
					+ "See the console for details.",
					"Binding selection frame", JOptionPane.ERROR_MESSAGE);
				bindings = new BindingAccessor[0];
			}

			fullBindingText = new String[bindings.length];
			bindingList.removeAll();
			for (int i=0; i<bindings.length; ++i)
			{
				try
				{
					fullBindingText[i] = bindings[i].getDescription();
				}
				catch (RemoteException e)
				{
					e.printStackTrace(System.err);
					JOptionPane.showMessageDialog(frame,
						"A problem occurred: " + e + "\n"
						+ "See the console for details.",
						"Binding selection frame", JOptionPane.ERROR_MESSAGE);
					fullBindingText[i] = e.toString();
				}

				String oneLineText = StringUtil.unspace(fullBindingText[i]);
				if (oneLineText.length() > 50)
				{
					oneLineText = oneLineText.substring(0, 47) + "...";
				}
				bindingList.add(oneLineText);
			}
			String text = "";

			if (bindings.length > 0)
			{
				bindingList.select(0);
				text = fullBindingText[0];
			}
			currentElement.setText(text);
			currentElement.setCaretPosition(0);

			fireButton.setEnabled(bindings.length > 0);
		}
	}
}
