package de.renew.gui;

import CH.ifa.draw.application.*;
import CH.ifa.draw.figures.*;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.standard.*;
import CH.ifa.draw.util.*;
import de.renew.database.*;
import de.renew.database.entitylayer.*;
import de.renew.remote.*;
import de.renew.shadow.*;
import de.renew.simulator.*;
import de.renew.util.ParameteredCommandLine;
import de.renew.util.PostscriptWriter;
import de.renew.util.RenewObjectInputStream;
import de.renew.util.RenewObjectOutputStream;
import de.renew.util.StringUtil;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.lang.reflect.*;
import java.net.*;
import java.rmi.RemoteException;
import java.sql.*;
import java.util.*;
import java.util.List;
import javax.swing.JOptionPane;


/**
 * Class documentation.
 */
public class CPNApplication extends DrawApplication
{
	static public final String CPNIMAGES = "/de/renew/gui/images/";
	static private final Color INSTANCE_COLOR = new Color(200, 200, 255);
	static private final Color TOKEN_BAG_COLOR = new Color(230, 230, 255);

	private ToolButton fInscrTB, fNameTB, fDeclTB, fAuxTB, fConnTextTB, fTextTB;
	protected RenewMode fMode;
	protected XMLFormat fXMLFormat;
	protected CPNSimulation fSimulation;	// cache of fMode.getSimulation()
	private BreakpointManager bpManager = null;
	private CPNDrawingLoader fDrawingLoader;

	/**
	 * Constructor documentation.
	 */
	protected CPNApplication()
	{
		this(new String[0]);
	}

	/**
	 * Constructor documentation.
	 * @param args
	 */
	protected CPNApplication(String[] args)
	{
		super("Reference Net Workshop", "rnw", args, CPNIMAGES + "RENEW.gif");
		loadSimulationStateFromDatabase();
	}

	/**
	 * Method documentation.
	 * @return
	 */
	public XMLFormat getXMLFormat()
	{
		if (fXMLFormat == null)
		{
			String formatStr = System.getProperty("de.renew.XMLFormat");

			if (formatStr == null)
			{
				formatStr = "de.renew.gui.xml.XRNFormat";
			}
			try
			{
				Class clazz = Class.forName(formatStr);	   // No reload required.

				fXMLFormat = (XMLFormat) clazz.newInstance();
			}
			catch (ThreadDeath e)
			{

				// This exception must be rethrown.
				throw e;
			}
			catch (Throwable e)
			{

				// XML is not installed.
				fXMLFormat = new NullXMLFormat();
			}
		}
		return fXMLFormat;
	}

	/**
	 * Method documentation.
	 * @return
	 */
	public RenewMode getMode()
	{
		if (fMode == null)
		{
			String modeStr = System.getProperty("de.renew.mode");

			if (modeStr != null)
			{

				// System.err.println("Trying to switch to mode "+modeStr+"...");
				try
				{
					Class clazz = Class.forName(modeStr);	 // No reload required.

					fMode = (RenewMode) clazz.newInstance();
				}
				catch (Exception e)
				{
					e.printStackTrace();
					System.err.println("Switch failed:" + e);
				}
			}
			if (fMode == null)
			{
				System.err.println("Using default mode...");
				fMode = new de.renew.gui.JavaMode();
			}
			else
			{
				System.err.println("Using mode " + modeStr + "...");
			}
			fDrawingLoader = new CPNDrawingLoader(this);
			fMode.setDrawingLoader(fDrawingLoader);
			fSimulation = fMode.getSimulation();
		}
		return fMode;
	}


	/**
	 * Method documentation.
	 * @return
	 */
	protected Drawing createDrawing()
	{
		return new CPNDrawing();
	}

	/**
	 * Method documentation.
	 * @return
	 */
	protected DrawApplication newWindow()
	{
		return new CPNApplication();
	}

	/**
	 * Creates the standard menus. Clients override this
	 * method to add additional menus.
	 */
	protected void createMenus(MenuBar mb)
	{
		mb.add(createFileMenu());
		mb.add(createEditMenu());
		mb.add(createAlignmentMenu());
		mb.add(createAttributesMenu());
		mb.add(createNetMenu());
		mb.add(createSimulationMenu());
		mb.add(createDrawingsMenu());
		mb.add(createHelpMenu());
		getMode().createAdditionalMenus(this, mb);
	}


	/**
	 * Method documentation.
	 * @param title
	 * @param parentClass
	 * @param type
	 * @param selectMode
	 * @return
	 */
	protected SelectCommand createSelectCommand(String title, final Class parentClass,
												final int type, int selectMode)
	{
		return new SelectCommand(title, this, new FigureFilter()
		{
			/**
			 * Method documentation.
			 * @param fig
			 * @return
			 */
			public boolean isUsed(Figure fig)
			{
				return fig instanceof CPNTextFigure
					   && parentClass.isInstance(((ChildFigure) fig).parent())
					   && ((CPNTextFigure) fig).getType() == type;
			}

		}, selectMode);
	}


	/**
	 * Method documentation.
	 * @return
	 */
	protected Menu createEditMenu()
	{
		CommandMenu menu = (CommandMenu) super.createEditMenu();
		Menu selectMenu = createSelectMenu(SelectCommand.SELECT);
		Menu addSelectMenu = createSelectMenu(SelectCommand.ADD);
		Menu removeSelectMenu = createSelectMenu(SelectCommand.REMOVE);
		Menu restrictSelectMenu = createSelectMenu(SelectCommand.RESTRICT);

		menu.add(selectMenu);
		menu.add(addSelectMenu);
		menu.add(removeSelectMenu);
		menu.add(restrictSelectMenu);
		return menu;
	}

	/**
	 * Method documentation.
	 * @param selectMode
	 * @return
	 */
	protected Menu createSelectMenu(int selectMode)
	{

		String selectMenuTitle;

		switch (selectMode)
		{
			case SelectCommand.SELECT:
				selectMenuTitle = "Select";
				break;
			case SelectCommand.ADD:
				selectMenuTitle = "Add To Selection";
				break;
			case SelectCommand.REMOVE:
				selectMenuTitle = "Remove From Selection";
				break;
			case SelectCommand.RESTRICT:
				selectMenuTitle = "Restrict Selection";
				break;
			default:
				selectMenuTitle = "??? Select mode " + selectMode + " ???";
		}

		CommandMenu selectMenu = createCommandMenu(selectMenuTitle);

		CommandMenu nodeMenu = createCommandMenu("Nodes");

		nodeMenu.add(new SelectCommand("All", this, NodeFigure.class, selectMode));
		nodeMenu.add(new SelectCommand("Transitions", this, TransitionFigure.class, selectMode));
		nodeMenu.add(new SelectCommand("Places", this, PlaceFigure.class, selectMode));
		selectMenu.add(nodeMenu);

		selectMenu.add(new SelectCommand("Arcs", this, ArcConnection.class, selectMode));
		selectMenu.add(new SelectCommand("Nodes and Arcs", this, InscribableFigure.class,
										 selectMode));

		CommandMenu textChildMenu = createCommandMenu("Text Children");

		textChildMenu.add(new SelectCommand("All", this, CPNTextFigure.class,
											InscribableFigure.class, selectMode));
		textChildMenu.add(new SelectCommand("of Transitions", this, CPNTextFigure.class,
											TransitionFigure.class, selectMode));
		textChildMenu.add(new SelectCommand("of Places", this, CPNTextFigure.class,
											PlaceFigure.class, selectMode));
		textChildMenu.add(new SelectCommand("of Arcs", this, CPNTextFigure.class,
											ArcConnection.class, selectMode));

		selectMenu.add(textChildMenu);

		CommandMenu inscrMenu = createCommandMenu("Inscriptions");

		inscrMenu.add(createSelectCommand("All", InscribableFigure.class,
										  CPNTextFigure.INSCRIPTION, selectMode));
		inscrMenu.add(createSelectCommand("of Transitions", TransitionFigure.class,
										  CPNTextFigure.INSCRIPTION, selectMode));
		inscrMenu.add(createSelectCommand("of Places", PlaceFigure.class,
										  CPNTextFigure.INSCRIPTION, selectMode));
		inscrMenu.add(createSelectCommand("of Arcs", ArcConnection.class,
										  CPNTextFigure.INSCRIPTION, selectMode));
		selectMenu.add(inscrMenu);

		CommandMenu nameMenu = createCommandMenu("Names");

		nameMenu.add(createSelectCommand("All", InscribableFigure.class, CPNTextFigure.NAME,
										 selectMode));
		nameMenu.add(createSelectCommand("of Transitions", TransitionFigure.class,
										 CPNTextFigure.NAME, selectMode));
		nameMenu.add(createSelectCommand("of Places", PlaceFigure.class, CPNTextFigure.NAME,
										 selectMode));
		selectMenu.add(nameMenu);

		return selectMenu;
	}

	/**
	 * Method documentation.
	 * @return
	 */
	protected Menu createNetMenu()
	{
		CommandMenu menu = createCommandMenu("Net");

		menu.add(new SplitCommand("Split transition/place", this));
		menu.add(new CoarseCommand("Coarsen subnet", this));
		menu.addSeparator();

		CommandMenu traceMenu = createCommandMenu("Trace");

		traceMenu.add(new ChangeAttributeCommand("on", "TraceMode", Boolean.TRUE, this));
		traceMenu.add(new ChangeAttributeCommand("off", "TraceMode", Boolean.FALSE, this));
		menu.add(traceMenu);

		CommandMenu markingMenu = createCommandMenu("Marking");

		markingMenu.add(new ChangeAttributeCommand("highlight only", "MarkingAppearance",
												   new Integer(PlaceFigure.HIGHLIGHT), this));
		markingMenu.add(new ChangeAttributeCommand("Cardinality", "MarkingAppearance",
												   new Integer(PlaceFigure.CARDINALITY), this));
		markingMenu.add(new ChangeAttributeCommand("Tokens", "MarkingAppearance",
												   new Integer(PlaceFigure.TOKENS), this));
		markingMenu.add(new ChangeAttributeCommand("expanded Tokens", "MarkingAppearance",
												   new Integer(PlaceFigure.EXPANDED_TOKENS), this));
		menu.add(markingMenu);

		CommandMenu breakpointMenu = getBreakpointManager().createSetMenu("Breakpoint",
				ToggleBreakpointCommand.PRESET);

		breakpointMenu.addSeparator();
		breakpointMenu.add(new ToggleBreakpointCommand("none", this, getBreakpointManager(),
				ToggleBreakpointCommand.REMOVE, ToggleBreakpointCommand.PRESET));
		menu.add(breakpointMenu);

		menu.addSeparator();

		menu.add(new SetIconFigureCommand("Set Selection as Icon", this));

		menu.add(new HighlightAssociateCommand("Associate Highlight", this));
		menu.add(new HighlightSelectCommand("Select Highlight(s)", this));
		menu.add(new HighlightUnassociateCommand("Unassociate Highlight", this));

		menu.addSeparator();

		menu.add(createMenuItem("Syntax check", new ActionListener()
		{
			/**
			 * Method documentation.
			 * @param event
			 */
			public void actionPerformed(ActionEvent event)
			{
				syntaxCheck();
			}

		}));
		addLintChecks(menu);
		menu.add(new LayoutCheckCommand("Layout check", this));

		return menu;
	}

	/**
	 * Create the alignment menu.
	 */
	protected Menu createAlignmentMenu()
	{
		CommandMenu menu = (CommandMenu) super.createAlignmentMenu();

		CommandMenu sizeMenu = createCommandMenu("Figure Size");

		sizeMenu.add(new ChangeSizeCommand("copy within selection", this,
										   ChangeSizeCommand.FIRST_FIGURE));
		sizeMenu.add(new ChangeSizeCommand("reset to default", this,
										   ChangeSizeCommand.DEFAULT_SIZE));
		menu.insert(sizeMenu, 5);

		menu.addSeparator();
		menu.add(new LayoutCommand("Automatic Net Layout...", this));
		CommandMenu fixMenu = createCommandMenu("Location");

		fixMenu.add(new ChangeAttributeCommand("dynamic", "Location", Boolean.FALSE, this));
		fixMenu.add(new ChangeAttributeCommand("fixed", "Location", Boolean.TRUE, this));
		menu.add(fixMenu);
		return menu;
	}

	/**
	 * Create the simulation menu.
	 */
	protected Menu createSimulationMenu()
	{

		CommandMenu menu = createCommandMenu("Simulation");

		menu.add(createMenuItem("Run simulation", new MenuShortcut('R'), new ActionListener()
		{
			public void actionPerformed(ActionEvent event)
			{
				if (!fSimulation.isSimulationActive())
				{
					initSimulation();
				}
				if (fSimulation.isSimulationActive())
				{
					showStatus("Simulation running.");
					fSimulation.simulationRun();
				}
			}

		}));

		menu.add(createMenuItem("Simulation Step", new MenuShortcut('I'), new ActionListener()
		{
			public void actionPerformed(ActionEvent event)
			{
				new Thread()
				{
					/**
					 * Method documentation.
					 */
					public void run()
					{
						if (!fSimulation.isSimulationActive())
						{
							initSimulation();
						}
						else
						{
							showStatus("Searching for a binding...");
							int status = fSimulation.simulationStep();

							switch (status)
							{
								case Simulator.statusStopped:
									if (fSimulation.isSimulationActive())
									{
										showStatus("Simulation halted.");
									}
									else
									{
										showStatus("Simulation terminated.");
									}
									break;
								case Simulator.statusStepComplete:
									showStatus("Simulation step completed.");
									break;
								case Simulator.statusLastComplete:
									showStatus("Simulation step completed. "
											   + "No more enabled bindings.");
									break;
								case Simulator.statusCurrentlyDisabled:
									showStatus("No enabled bindings found.");
									break;
								case Simulator.statusDisabled:
									showStatus("No more enabled bindings.");
									break;
							}
						}
					}

				}.start();
			}

		}));

		menu.add(new Command("Halt simulation")
		{
			public boolean isExecutable() {
				return fSimulation.isSimulationActive();
			}
			public void execute() {
				fSimulation.simulationStop();
				showStatus("Simulation halted.");
			}
		}, new MenuShortcut('H'));

		menu.add(new Command("Terminate simulation")
		{
			public boolean isExecutable() {
				return fSimulation.isSimulationActive();
			}
			public void execute() {
				fSimulation.simulationTerminate();
				showStatus("Simulation terminated.");
				BindingSelectionFrame.close();
				closeAllSimulationDrawings();
			}
		}, new MenuShortcut('T'));

		menu.addSeparator();

		menu.add(createBreakpointMenu());

		menu.addSeparator();

		menu.add(new Command("Save simulation state...")
		{
			public boolean isExecutable() {
				return fSimulation.isSimulationActive();
			}
			public void execute() {
				promptSaveSimulationState();
			}
		});


		menu.add(new Command("Load simulation state...")
		{
			public void execute()			{
				promptLoadSimulationState();
			}
		});

		menu.addSeparator();

		menu.add(new ConnectCommand("Connect to server...", this));

		return menu;
	}

	/**
	 * Adds menu entries for mode-specific net checks to the
	 * given menu.
	 *
	 * @param menu   the command menu to be extended
	 */
	private void addLintChecks(CommandMenu menu)
	{

		String[] lintNames = getMode().getCompiler().getLintNames();

		for (int i = 0; i < lintNames.length; i++)
		{
			final int lintNr = i;
			final String name = lintNames[i];
			final String upperName = StringUtil.firstToUpperCase(name);

			menu.add(createMenuItem(upperName, new ActionListener()
			{
				/**
				 * Method documentation.
				 * @param event
				 */
				public void actionPerformed(ActionEvent event)
				{
					if (fSimulation.isSimulationActive())
					{
						showStatus("Terminate the current simulation " + "before a " + name + ".");
					}
					else
					{
						try
						{

							// Start with an ordinary syntax check. This way,
							// fewer checks have to be made within the lints,
							// all data structures in the simulation are
							// updated and the class source is set properly.
							fSimulation.syntaxCheckOnly();
							ShadowNet selectedNet = null, lastSelectedNet = null;
							Drawing drawing = drawing();

							if (drawing instanceof CPNDrawing)
							{
								selectedNet = ((CPNDrawing) drawing).getShadow();
							}
							DrawingView view = previousView();

							if (view != null && view.drawing() instanceof CPNDrawing)
							{
								lastSelectedNet = ((CPNDrawing) view.drawing()).getShadow();

								// Now make the check.
							}
							ShadowNetSystem sns = fSimulation.getNetSystem();

							sns.getCompiler().lint(lintNr, sns, selectedNet, lastSelectedNet);
							showStatus(upperName + " successful.");
							closeSyntaxErrorFrame();
							return;
						}
						catch (SyntaxException e)
						{
							processSyntaxException(FigureExceptionFactory.createFigureException(e),
												   true);
						}
					}
				}

			}));
		}
	}

	/**
	 * Method documentation.
	 * @return
	 */
	protected BreakpointManager getBreakpointManager()
	{
		if (bpManager == null)
		{

			// I use getMode().getSimulation() instead of fSimulation
			// because I'm not sure if this is the first time the mode
			// is requested. fSimulation can be uninitialized.
			bpManager = new BreakpointManager(getMode().getSimulation(), this);
			fSimulation.setBreakpointManager(bpManager);
		}
		return bpManager;
	}

	/**
	 * Method documentation.
	 * @return
	 */
	protected Menu createBreakpointMenu()
	{
		return getBreakpointManager().getMenu();
	}

	/**
	 * Method documentation.
	 * @return
	 */
	protected Menu createHelpMenu()
	{

		Menu menu = createMenu("Help");
		final InfoDialog about = new InfoDialog(menuFrame, "About Renew",
												Version.version
												+ "\n(c) 2000\nKummer, Wienberg, Duvigneau");

		menu.add(createMenuItem("About...", new ActionListener()
		{
			/**
			 * Method documentation.
			 * @param event
			 */
			public void actionPerformed(ActionEvent event)
			{
				about.show();
			}

		}));
		return menu;

	}

	/**
	 * Method documentation.
	 */
	private void closeAllSimulationDrawings()
	{
		boolean goon;

		do
		{
			goon = false;
			Enumeration drawenum = drawings();

			while (drawenum.hasMoreElements())
			{
				Drawing drawing = (Drawing) drawenum.nextElement();

				if (drawing instanceof CPNInstanceDrawing || drawing instanceof TokenBagDrawing)
				{
					closeDrawing(drawing);
					goon = true;
					break;
				}
			}
		}
		while (goon);
	}

	/**
	 * Method documentation.
	 * @param width
	 * @param height
	 * @return
	 */
	protected StandardDrawingView createDrawingView(int width, int height)
	{
		return new CPNDrawingView(this, width, height);
	}


	/**
	 * Method documentation.
	 * @return
	 */
	protected Menu createFileMenu()
	{
		Menu menu = super.createFileMenu();

		menu.insert(createExportsMenu(), menu.getItemCount() - 2);

		menu.insert(createImportsMenu(), menu.getItemCount() - 2);

		return menu;
	}

	/**
	 * Method documentation.
	 * @return
	 */
	protected Menu createExportsMenu()
	{
		Menu menu = new Menu("Export");

		menu.add(createMenuItem("PostScript...", new ActionListener()
		{
			/**
			 * Method documentation.
			 * @param event
			 */
			public void actionPerformed(ActionEvent event)
			{
				promptExportPostscript();
			}

		}));

		menu.add(createMenuItem("EPS...", new MenuShortcut('E'), new ActionListener()
		{
			/**
			 * Method documentation.
			 * @param event
			 */
			public void actionPerformed(ActionEvent event)
			{
				promptExportEPS();
			}

		}));

		menu.addSeparator();

		if (getXMLFormat().canExport())
		{
			menu.add(createMenuItem("XML...", new ActionListener()
			{
				/**
				 * Method documentation.
				 * @param event
				 */
				public void actionPerformed(ActionEvent event)
				{
					promptExportXML();
				}

			}));
			menu.addSeparator();
		}

		menu.add(createMenuItem("to Macao...", new ActionListener()
		{
			/**
			 * Method documentation.
			 * @param event
			 */
			public void actionPerformed(ActionEvent event)
			{
				promptExportMacao();
			}

		}));

		menu.add(createMenuItem("to Woflan...", new ActionListener()
		{
			/**
			 * Method documentation.
			 * @param event
			 */
			public void actionPerformed(ActionEvent event)
			{
				promptExportWoflan();
			}

		}));

		menu.addSeparator();

		menu.add(createMenuItem("all as Shadow Net System...", new ActionListener()
		{
			public void actionPerformed(ActionEvent event)
			{
				promptExportShadow();
			}
		}));

		menu.add(createMenuItem("all as Shadow Nets", new ActionListener()
		{
			public void actionPerformed(ActionEvent event)
			{
				exportSingleShadowFiles();
			}
		}));


		return menu;
	}

	/**
	 * Method documentation.
	 * @return
	 */
	protected Menu createImportsMenu()
	{
		Menu menu = new Menu("Import");

		if (getXMLFormat().canExport())
		{
			menu.add(createMenuItem("XML...", new ActionListener()
			{
				/**
				 * Method documentation.
				 * @param event
				 */
				public void actionPerformed(ActionEvent event)
				{
					promptImportXML();
				}

			}));
		}

		menu.add(createMenuItem("Shadow Net System...", new ActionListener()
		{
			/**
			 * Method documentation.
			 * @param event
			 */
			public void actionPerformed(ActionEvent event)
			{
				promptImportShadow();
			}

		}));

		return menu;
	}

	/**
	 * Method documentation.
	 * @return
	 */
	protected Menu createAttributesMenu()
	{
		Menu menu = super.createAttributesMenu();

		CommandMenu arrowTipMenu = createCommandMenu("Arrow Shape");

		arrowTipMenu.add(new ChangeAttributeCommand("normal", "ArrowTip", ArrowTip.class.getName(),
													this));
		arrowTipMenu.add(new ChangeAttributeCommand("double", "ArrowTip",
													DoubleArrowTip.class.getName(), this));
		arrowTipMenu.add(new ChangeAttributeCommand("Lines", "ArrowTip",
													de.renew.gui.fs.AssocArrowTip.class.getName(),
													this));
		arrowTipMenu.add(new ChangeAttributeCommand("Triangle", "ArrowTip",
													de.renew.gui.fs.IsaArrowTip.class.getName(),
													this));
		menu.insert(arrowTipMenu, 4);

		CommandMenu textTypeMenu = createCommandMenu("Text Type");

		textTypeMenu.add(new ChangeAttributeCommand("Label", "TextType",
													new Integer(CPNTextFigure.LABEL), this));
		textTypeMenu.add(new ChangeAttributeCommand("Inscription", "TextType",
													new Integer(CPNTextFigure.INSCRIPTION), this));
		textTypeMenu.add(new ChangeAttributeCommand("Name", "TextType",
													new Integer(CPNTextFigure.NAME), this));
		menu.add(textTypeMenu);

		return menu;
	}

	/**
	 * Method documentation.
	 * @param statusLine
	 * @return
	 */
	protected Component wrapStatusLine(Component statusLine)
	{
		if (getMode().wantClock())
		{
			Panel panel = new Panel();
			GridBagLayout layout = new GridBagLayout();

			panel.setLayout(layout);

			GridBagConstraints constraints = new GridBagConstraints();

			constraints.fill = GridBagConstraints.HORIZONTAL;

			constraints.weightx = 1;
			layout.setConstraints(statusLine, constraints);
			panel.add(statusLine);
			if (statusLine instanceof TextField)
			{

				// make original status line a bit smaller
				TextField textStatusLine = (TextField) statusLine;

				textStatusLine.setColumns(textStatusLine.getColumns() - 23);
			}
			final TextField clock = new TextField("0.0", 23);

			SearchQueue.insertTimeListener(new TimeListener()
			{
				/**
				 * Method documentation.
				 */
				public void timeAdvanced()
				{
					clock.setText(String.valueOf(SearchQueue.getTime()));
				}

			});
			clock.setEditable(false);
			constraints.weightx = 0;
			layout.setConstraints(clock, constraints);
			panel.add(clock);

			return panel;
		}
		else
		{
			return statusLine;
		}
	}

	/**
	 * Method documentation.
	 * @param drawing
	 */
	protected void drawingAdded(Drawing drawing)
	{
		super.drawingAdded(drawing);
		if (drawing instanceof CPNDrawing)
		{
			getMode(); // This is to make sure the mode (and with
			           // it the drawing loader) has been instantiated.
			fDrawingLoader.addDrawing((CPNDrawing) drawing);
		}
		if (drawing instanceof CPNInstanceDrawing || drawing instanceof TokenBagDrawing)
		{
			noUndoHistoryFor(drawing);
		}

	}

	/**
	 * Method documentation.
	 * @param drawing
	 */
	protected void drawingReleased(Drawing drawing)
	{
		super.drawingReleased(drawing);
		if (drawing instanceof CPNDrawing)
		{
			fDrawingLoader.releaseDrawing((CPNDrawing) drawing);
		}
	}

	private SyntaxExceptionFrame syntaxFrame = null;

	/**
	 * Method documentation.
	 * @return
	 */
	private boolean syntaxCheck()
	{
		if (fSimulation.isSimulationActive())
		{
			showStatus("Terminate the current simulation " + "before a syntax check.");
		}
		else
		{
			try
			{
				fSimulation.syntaxCheckOnly();
				showStatus("Syntax check successful.");
				closeSyntaxErrorFrame();
				return true;
			}
			catch (SyntaxException e)
			{
				processSyntaxException(FigureExceptionFactory.createFigureException(e), true);
			}
		}
		return false;
	}


	/**
	 * Method documentation.
	 */
	public void closeSyntaxErrorFrame()
	{
		if (syntaxFrame != null)
		{
			syntaxFrame.dispose();
			syntaxFrame = null;
		}
	}

	/**
	 * Method documentation.
	 * @param e
	 * @param displayImmediately
	 */
	void processSyntaxException(FigureException e, boolean displayImmediately)
	{
		processFigureException(e, displayImmediately);

		// Print the error message.
		showStatus("A syntax error occurred.");
	}

	/**
	 * Method documentation.
	 * @param e
	 * @param displayImmediately
	 */
	void processFigureException(FigureException e, boolean displayImmediately)
	{
		if (syntaxFrame == null)
		{
			syntaxFrame = new SyntaxExceptionFrame(this);
		}

		syntaxFrame.displayException(e, displayImmediately);
	}

	/**
	 * Loads the simulation state from the database
	 * and reopens all instance drawings having been open
	 * when the simulation was inproperly terminated.
	 */
	private void loadSimulationStateFromDatabase()
	{
		List netInstances = SetupHelper.setup(fSimulation, false).getNetInstances();
		Iterator iterator = netInstances.iterator();
		while (iterator.hasNext())
		{
			openInstanceDrawing((NetInstance) iterator.next());
		}
	}

	/**
	 * Opens an instance drawing for a given net instance.
	 * @param instance The net instance.
	 */
	public void openInstanceDrawing(NetInstance instance)
	{
		try
		{
			openInstanceDrawing(new NetInstanceAccessorImpl(instance));
		}
		catch (RemoteException e)
		{
			e.printStackTrace(System.err);
			JOptionPane.showMessageDialog(null,
				"A problem occurred: " + e + "\n"
				+ "See the console for details.", "Renew", JOptionPane.ERROR_MESSAGE);
		}
	}

	/**
	 * Opens an instance drawing for a given net instance accessor.
	 * @param instance The net instance accessor.
	 */
	public void openInstanceDrawing(NetInstanceAccessor instance)
	{
		try
		{
			CPNInstanceDrawing instDraw = CPNInstanceDrawing.getInstanceDrawing(getMode(), instance);

			// If the instance drawing could not be created, don't open a window...
			if (instDraw != null)
			{
				boolean newWindow = getViewFrame(instDraw) == null;

				// new:
				// position instance net drawing to where net drawing is located.
				// Should provide an explicit getName() for Net.
				CPNDrawing drawing = fDrawingLoader.getDrawing(instance.getNet().asString());
				DrawingViewFrame netFrame = getViewFrame(drawing);
				DrawingViewFrame viewFrame = showDrawingViewFrame(instDraw, netFrame.getLocation(),
						netFrame.getSize());

				if (newWindow)
				{
					viewFrame.onDiscardRelease();
					viewFrame.view().setBackground(INSTANCE_COLOR);
					viewFrame.validate();
				}
			}
		}
		catch (RemoteException e)
		{
			e.printStackTrace(System.err);
			JOptionPane.showMessageDialog(null,
				"A problem occurred: " + e + "\n"
				+ "See the console for details.", "Renew", JOptionPane.ERROR_MESSAGE);
		}
	}

	/**
	 * Method documentation.
	 * @param pi
	 * @exception RemoteException If an RMI problem occurred.
	 */
	public void openTokenBagDrawing(PlaceInstanceAccessor pi)
		throws RemoteException
	{
		TokenBagDrawing tokenDraw = TokenBagDrawing.getTokenBagDrawing(getMode(), pi);
		boolean newWindow = getViewFrame(tokenDraw) == null;

		CPNInstanceDrawing instDraw = CPNInstanceDrawing.getInstanceDrawing(getMode(),
				pi.getNetInstance());
		DrawingViewFrame netFrame = getViewFrame(instDraw);
		DrawingViewFrame viewFrame = showDrawingViewFrame(tokenDraw, netFrame.getLocation(), null);

		if (newWindow)
		{
			viewFrame.onDiscardRelease();
			viewFrame.view().setBackground(TOKEN_BAG_COLOR);
		}
	}

	/**
	 * Initialize the simulation.
	 */
	void initSimulation()
	{
		if (!(drawing() instanceof CPNDrawing))
		{
			showStatus("Cannot start new simulation with this drawing! Please activate a net window.");
			return;
		}

		try
		{
			NetInstance primaryInstance = fSimulation.initSimulation(drawing().getName());

			if (primaryInstance != null)
			{
				openInstanceDrawing(primaryInstance);

				if (fSimulation.isSimulationActive())
				{
					showStatus("Simulation initialized.");
				}
				else
				{
					showStatus("Simulation initialized. " + "No enabled bindings.");
				}
			}
			else
			{
				showStatus("The simulation could not be initialized.");
			}

			// closeSyntaxErrorFrame();
		}
		catch (SyntaxException e)
		{
			processSyntaxException(FigureExceptionFactory.createFigureException(e), true);
		}
	}

	/**
	 * Shows a file dialog and saves drawing as postscript.
	 */
	public void promptExportPostscript()
	{
		toolDone();
		String path = getSavePath("Export PostScript...", drawing().getName() + ".ps", new String[]
		{
			"ps"
		});

		if (path != null)
		{
			exportPostscript(path, false);
		}
	}

	/**
	 * Method documentation.
	 */
	public void promptExportEPS()
	{
		toolDone();
		String path = getSavePath("Export EPS...", drawing().getName() + ".eps", new String[]
		{
			"eps"
		});

		if (path != null)
		{
			exportPostscript(path, true);
		}
	}

	/**
	 * Method documentation.
	 */
	public void promptExportXML()
	{
		toolDone();
		String path = getSavePath("Export XML...", drawing().getName() + ".xrn", new String[]
		{
			"xrn"
		});

		if (path != null)
		{
			exportXML(path);
		}
	}

	/**
	 * Method documentation.
	 */
	public void promptExportMacao()
	{
		Drawing drawing = drawing();

		if (drawing instanceof CPNDrawing)
		{
			toolDone();
			String path = getSavePath("Export to Macao...", drawing().getName(), null);

			if (path != null)
			{
				try
				{
					PrintWriter p = new PrintWriter(new FileOutputStream(path));

					((CPNDrawing) drawing).exportMacao(p);
					p.close();
				}
				catch (Exception e)
				{
					showStatus(e.toString());
				}
			}
		}
		else
		{
			showStatus("This drawing is not a net!");
		}
	}

	/**
	 * Method documentation.
	 */
	public void promptExportWoflan()
	{
		Drawing drawing = drawing();

		if (drawing instanceof CPNDrawing)
		{
			toolDone();
			String path = getSavePath("Export to Woflan...", drawing.getName(), null);

			if (path != null)
			{
				fSimulation.buildAllShadows();
				try
				{
					PrintWriter p = new PrintWriter(new FileOutputStream(path));

					((CPNDrawing) drawing).exportWoflan(p);
					p.close();
				}
				catch (Exception e)
				{
					showStatus(e.toString());
				}
			}
		}
		else
		{
			showStatus("This drawing is not a net!");
		}
	}

	/**
	 * Method documentation.
	 * @param file
	 */
	public synchronized void loadAndOpenDrawing(String file)
	{
		if ("sns".equals(StringUtil.getExtension(file)))
		{
			importShadow(file);
		}
		else
		{
			super.loadAndOpenDrawing(file);
		}
	}

	/**
	 * Closes the specified drawing.
	 * Checks if instance drawings depend on the drawing to be
	 * closed and asks the user, if all dependent drawings should
	 * be closed, too.
	 *
	 * @return <code>true</code>,  if the drawing was closed
	 * - <code>false</code>, if the operation was aborted
	 */
	protected boolean closeViewFrame(DrawingViewFrame viewFrame)
	{
		Drawing drawing = viewFrame.view().drawing();

		if (drawing instanceof CPNDrawing)
		{
			Enumeration dependentDrawings =
				CPNInstanceDrawing.getDependentInstanceDrawings((CPNDrawing) drawing);

			if (dependentDrawings != null)
			{
				showDrawingViewFrame(drawing);

				int answer = MessageDialog.invokeDialog(menuFrame(), "Renew: Confirm Close",
														new String[]
				{
					"The drawing \"" + drawing.getName() + "\" " + "you are about to close",
					"is needed to display one " + "or more instance drawings."
				}, MessageDialog.LEFT, new String[]
				{
					" Close instance drawings ", " Cancel "
				});

				if (answer == 1)
				{
					return false;
				}
				while (dependentDrawings.hasMoreElements())
				{
					closeDrawing((CPNInstanceDrawing) dependentDrawings.nextElement());
				}
			}
		}
		return super.closeViewFrame(viewFrame);
	}

	/**
	 * Exits the application.
	 * (DrawApplication asks to never override this method,
	 * but this method calls its super class.
	 * It just closes all simulation windows before.)
	 */
	public void exit()
	{
		closeAllSimulationDrawings();
		super.exit();
	}

	/**
	 * Shows a file dialog and imports a shadow net system.
	 */
	public void promptImportShadow()
	{
		String path = getLoadPath("Open Shadow File...", null, new String[]
		{
			"sns", "sha"
		});

		if (path != null)
		{
			importShadow(path);
		}
	}

	/**
	 * Method documentation.
	 * @param filename
	 */
	private void importShadow(String filename)
	{
		showStatus("Importing " + filename + " ...");
		CPNDrawing[] drawings = ShadowNetSystemRenderer.render(filename);

		for (int i = 0; i < drawings.length; ++i)
		{
			newDrawingViewFrame(drawings[i]);
		}
		showStatus("Imported " + filename + ".");
	}

	/**
	 * Method documentation.
	 */
	public void promptExportShadow()
	{
		toolDone();
		if (syntaxCheck())
		{
			String path = getSavePath("Export Shadow Net System...", null, new String[]
			{
				"sns", "sha"
			});

			if (path != null)
			{
				try
				{
					FileOutputStream stream = new FileOutputStream(path);
					ObjectOutput output = new ObjectOutputStream(stream);

					output.writeObject(fSimulation.getNetSystem());
					output.close();
				}
				catch (Exception e)
				{
					showStatus(e.toString());
				}
			}
		}
	}

	/**
	 * Method documentation.
	 */
	public void exportSingleShadowFiles()
	{
		toolDone();
		if (syntaxCheck())
		{
			Iterator drawings = fDrawingLoader.loadedDrawings();
			int count = 0;
			StringBuffer skipped = new StringBuffer();
			showStatus("Exporting...");

			// Build new shadows of all nets, one net per net system.
			while (drawings.hasNext()) {
				CPNDrawing currentDrawing = (CPNDrawing) drawings.next();
				ShadowNetSystem netSystem = new ShadowNetSystem(fMode.getCompiler());
				currentDrawing.buildShadow(netSystem);

				String name = currentDrawing.getName();
				String path = currentDrawing.getFilename();
				if (path == null) {
					System.out.println("Skipping net "+name+" (no filename).");
					if (skipped.length() == 0) {
						skipped.append("; skipped ");
					} else {
						skipped.append(", ");
					}
					skipped.append(name);
				} else {
					try {
						path = StringUtil.makeCanonical(path);
						path = StringUtil.getPath(path) + File.separator + name + ".sns";
						System.out.print("Exporting net "+name+" to " + path + " ... ");
						
						FileOutputStream stream = new FileOutputStream(path);
						ObjectOutput output = new ObjectOutputStream(stream);
						output.writeObject(netSystem);
						output.close();
						System.out.println("OK");
						count++;
					} catch (Exception e) {
						System.out.println(e);
					}
				}
				currentDrawing.discardShadow();
			}
			showStatus("Exported "+count+" shadow net"+(count==1?"":"s")+skipped.toString()+".");
		}
	}

	/**
	 * Method documentation.
	 */
	public void promptImportXML()
	{
		String path = getLoadPath("Open XML File...", null, new String[]
		{
			"xrn"
		});

		if (path != null)
		{
			importXML(path);
		}
	}

	/**
	 * Method documentation.
	 * @param file
	 */
	private void importXML(String file)
	{
		try
		{
			showStatus("Importing " + file + " ...");
			FileInputStream stream = new FileInputStream(file);
			CPNDrawing drawing = getXMLFormat().parse(stream);

			stream.close();
			newDrawingViewFrame(drawing);
			showStatus("Imported " + file + ".");
		}
		catch (Exception e)
		{

			// e.printStackTrace();
			showStatus(e.toString());
		}
	}

	/**
	 * Method documentation.
	 * @param file
	 */
	private void exportXML(String file)
	{
		try
		{
			showStatus("Exporting " + file + " ...");
			FileOutputStream stream = new FileOutputStream(file);

			getXMLFormat().write(stream, (CPNDrawing) drawing(), getMode().getCompiler());
			stream.close();
			showStatus("Exported " + file + ".");
		}
		catch (Exception e)
		{

			// e.printStackTrace();
			showStatus(e.toString());
		}
	}

	/**
	 * Method documentation.
	 * @param file
	 * @param wantEPS
	 */
	private void exportPostscript(String file, boolean wantEPS)
	{
		try
		{
			showStatus("Exporting " + file + " ...");
			PostscriptWriter ps = null;
			Dimension pageSize = new Dimension(535, 788);
			Rectangle drawingBox = drawing().getBounds();

			if (wantEPS)
			{

				// produce EPS:
				ps = new PostscriptWriter(file, drawingBox);
			}
			else
			{
				ps = new PostscriptWriter(file);
				ps.translate(32, -(32 + pageSize.height));
				pageSize.width = Math.max(pageSize.width, drawingBox.x + drawingBox.width);
				pageSize.height = Math.max(pageSize.height, drawingBox.y + drawingBox.height);

				// ps.translate(-32,-32+pageSize.height);
				// ps.setClip(64,64,pageSize.width-128,pageSize.height-128);
				// replaced by:
				ps.setClip(0, 0, pageSize.width, pageSize.height);
				double scale = Math.min(535.0 / pageSize.width, 778.0 / pageSize.height);

				if (scale < 1.0)
				{
					System.out.println("Scaling PS output by " + scale);
					ps.scale(scale, scale);
				}
			}
			ps.setColorConversion(ColorMap.BACKGROUND, Color.white);
			view().drawDrawing(ps);

			// view().drawHandles(ps);
			ps.dispose();
			showStatus("Exported " + file + ".");
		}
		catch (java.io.IOException e)
		{
			System.out.println(e.toString());
			showStatus(e.toString());
		}
	}

	/**
	 * Shows a save file dialog and stores the current
	 * simulation state.
	 *
	 * added Jan 20 2000  Michael Duvigneau
	 */
	private void promptSaveSimulationState()
	{

		if (!fSimulation.isSimulationActive())
		{
			showStatus("Need running simulation to save state.");
		}
		else
		{

			String path = getSavePath("Save Simulation State...", null, new String[]
			{
				"rst"
			});

			if (path != null)
			{

				String message = null;
				Stack problemLocation = null;
				RenewObjectOutputStream output = null;

				try
				{
					showStatus("Exporting " + path + " ...");

					// As the recursion depth seems to be too high to
					// fit in the Java VM stack space, we try to cut
					// it down by using de.renew.util.RenewObjectOutputStream.
					FileOutputStream stream = new FileOutputStream(path);

					output = new RenewObjectOutputStream(stream);

					fSimulation.saveState(output);
					showStatus("Exported " + path + ".");
				}
				catch (NotSerializableException e)
				{
					message = "Some object or class is not serializable: " + e.getMessage()
							  + ".\nWriting of file " + path + " aborted.";
					problemLocation = output.getDomainTrace();
				}
				catch (Exception e)
				{
					message = "Problem encountered while writing file " + path + ": " + e;
				}
				catch (StackOverflowError e)
				{
					message = "StackOverflowError: Probably the recursion in Java's "
							  + "serialization mechanism went too deep.\n"
							  + "If you are using JDK 1.1, try setting the java options "
							  + " -ss... and/or -oss... to increase native and java stack "
							  + "sizes. Setting -ss1m was appropriate for us.";
				}
				try
				{
					if (output != null)
					{
						output.close();
					}
					if (message == null)
					{
						message = "Simulation state saved to " + path + ".";
					}
				}
				catch (Exception e)
				{
					if (message == null)
					{
						message = "Problem encountered while " + "closing file " + path + ": " + e;
					}
				}
				if (problemLocation != null)
				{
					displaySaveSimError(problemLocation, message);
				}
				else
				{
					showStatus(message);
					System.err.println(message);
				}
			}
		}
	}

	/**
	 * Method documentation.
	 * @param objectLocation
	 * @param message
	 */
	private void displaySaveSimError(Stack objectLocation, String message)
	{
		try
		{
			NetAccessor net = null;
			NetInstanceAccessor netInstance = null;
			int id = FigureWithID.NOID;
			String thisShouldNotHappen = null;
			String specialReason = null;
			Object netElement = null;

			try
			{
				netElement = objectLocation.peek();

				if (netElement instanceof PlaceInstanceAccessor)
				{
					PlaceInstanceAccessor placeInst = (PlaceInstanceAccessor) netElement;

						id = placeInst.getPlace().getID().getFigureID();
						netInstance = placeInst.getNetInstance();
						net = netInstance.getNet();
				}
				else if (netElement instanceof TransitionInstanceAccessor)
				{
					TransitionInstanceAccessor transitionInst = (TransitionInstanceAccessor) netElement;

					id = transitionInst.getTransition().getID().getFigureID();
					netInstance = transitionInst.getNetInstance();
					net = netInstance.getNet();
				}
				else if (netElement instanceof PlaceAccessor)
				{
					thisShouldNotHappen = "place " + netElement;
					id = ((PlaceAccessor) netElement).getID().getFigureID();
				}
				else if (netElement instanceof TransitionAccessor)
				{
					thisShouldNotHappen = "transition " + netElement;
					id = ((TransitionAccessor) netElement).getID().getFigureID();
				}
				else if (netElement instanceof NetAccessor)
				{
					thisShouldNotHappen = "net " + netElement;
					net = (NetAccessor) netElement;
				}
				else if (netElement instanceof NetInstanceAccessor)
				{
					thisShouldNotHappen = "net instance " + netElement;
					netInstance = (NetInstanceAccessor) netElement;
					net = netInstance.getNet();
				}
				else if (netElement instanceof IDRegistry)
				{
					thisShouldNotHappen = "a token ID registry";
				}
				else if (netElement == CPNInstanceDrawing.class)
				{
					thisShouldNotHappen = "instance drawings";
				}
				else if (netElement == Net.class)
				{
					thisShouldNotHappen = "compiled nets";
				}
				else if (netElement == SearchQueue.class)
				{
					thisShouldNotHappen = "search queue entries";
					if (message.indexOf("de.renew.call.SynchronisationRequest") >= 0)
					{
						specialReason = "You tried to save the simulation state "
										+ "while a Java call to a net method was still "
										+ "active. Wait until all methods have completed.";
					}
				}
				else
				{
					thisShouldNotHappen = "(DON'T KNOW - TRACE CONTAINS OBJECT OF UNEXPECTED TYPE: "
										  + netElement.getClass().getName() + ")";
				}
			}
			catch (EmptyStackException e)
			{
				thisShouldNotHappen = "(DON'T KNOW - TRACE IS EMPTY!?)";
			}

			if (thisShouldNotHappen == null)
			{
				String netName = net.getName();

				message = message + "\nThe problem lies around net element " + netElement + ".";
				CPNDrawing drawing = fDrawingLoader.getDrawing(netName);

				if (drawing == null)
				{
					showStatus(message);
					System.err.println(message);
					System.err.println("Sorry, cannot show the location of the problem "
									   + "because net " + netName + " is not loaded.");
				}
				else
				{
					openInstanceDrawing(netInstance);

					CPNInstanceDrawing instanceDrawing =
						CPNInstanceDrawing.getInstanceDrawing(getMode(), netInstance);
					FigureWithID figure = drawing.getFigureWithID(id);
					InstanceFigure instanceFigure = instanceDrawing.getInstanceFigure(figure);

					showStatus(message);
					System.err.println(message);
					processFigureException(new FigureException("Renew: Save simulation error",
						message, instanceDrawing, instanceFigure), false);
				}
			}
			else
			{
				if (specialReason == null)
				{
					message = message + "\nDid you try to save a state while the "
							  + "simulation was running (concurrent transition "
							  + "firing or binding calculation is not allowed)?";
				}
				else
				{
					message = message + "\n" + specialReason;
				}
				message = message
						  + ("\nThe problem manifested itself while writing " + thisShouldNotHappen
							 + ".");
				message = message + ("\nSerialization trace:");
				try
				{
					while (!objectLocation.isEmpty())
					{
						netElement = objectLocation.pop();
						message = message + ("\n    " + netElement);

						// Take a guess, to which net the offending figure belongs.
						// With chance, the net is listed in the trace.
						if (net == null && netInstance == null)
						{
							if (netElement instanceof NetAccessor)
							{
								net = (NetAccessor) netElement;
							}
							else if (netElement instanceof NetInstanceAccessor)
							{
								netInstance = (NetInstanceAccessor) netElement;
								net = netInstance.getNet();
							}
						}
					}
				}
				catch (EmptyStackException e)
				{
				}

				showStatus(message);
				System.err.println(message);
				if ((net != null) && (id != FigureWithID.NOID))
				{
					CPNDrawing drawing = fDrawingLoader.getDrawing(net.getName());

					if (drawing != null)
					{
						FigureWithID figure = drawing.getFigureWithID(id);

						if (figure != null)
						{
							if (netInstance != null)
							{
								try
								{
									CPNInstanceDrawing instanceDrawing =
										CPNInstanceDrawing.getInstanceDrawing(getMode(), netInstance);
									InstanceFigure instanceFigure =
										instanceDrawing.getInstanceFigure(figure);

									processFigureException(new FigureException("Renew: Save simulation error",
										message, instanceDrawing, figure), false);
								}
								catch (RemoteException e)
								{
									e.printStackTrace(System.err);
									JOptionPane.showMessageDialog(null,
										"A problem occurred: " + e + "\nSee the console for details.",
										"Renew", JOptionPane.ERROR_MESSAGE);
								}
							}
							else
							{
								processFigureException(new FigureException("Renew: Save simulation error",
									message, drawing, figure), false);
							}
						}
					}
				}
			}
		}
		catch (RemoteException e)
		{
			e.printStackTrace(System.err);
			JOptionPane.showMessageDialog(null,
				"A problem occurred: " + e + "\n"
				+ "See the console for details.", "Renew", JOptionPane.ERROR_MESSAGE);
		}
	}

	/**
	 * Shows a load file dialog and replaces the current
	 * simulation state with the loaded one.
	 *
	 * added Apr 12 2000  Michael Duvigneau
	 */
	private void promptLoadSimulationState()
	{

		String path = getLoadPath("Load Simulation State...", null, new String[]
		{
			"rst"
		});

		if (path != null)
		{

			// Close all simulation related windows,
			// as the old simulation will be terminated.
			BindingSelectionFrame.close();
			closeAllSimulationDrawings();

			String message = null;
			ObjectInput input = null;

			try
			{
				showStatus("Importing " + path + " ...");

				FileInputStream stream = new FileInputStream(path);

				// As the recursion depth seems to be too high to
				// fit in the Java VM stack space, we try to cut
				// it down by using de.renew.util.RenewObjectInputStream.
				// Additionally, some changes to PlaceInstance are
				// required to save the tokens delayed.
				input = new RenewObjectInputStream(stream);

				// Below the original line instantiating the classic Stream.
				// input = new ObjectInputStream(stream);

				fSimulation.loadState(input, this);

				showStatus("Imported " + path + ".");
			}
			catch (ObjectStreamException e)
			{
				message = "File " + path + " is corrupted or does not "
						  + "contain state information.";
				System.err.println(e);
			}
			catch (ClassNotFoundException e)
			{
				message = "File " + path + " refers to java classes "
						  + "which are currently not available.";
				System.err.println(e);
			}
			catch (IOException e)
			{
				message = "Problem occured while reading file " + path + ":" + e;
			}
			catch (StackOverflowError e)
			{
				message = "StackOverflowError: Probably the recursion in Java's "
						  + "serialization mechanism went too deep.\n"
						  + "If you are using JDK 1.1, try setting the java options "
						  + " -ss... and/or -oss... to increase native and java stack "
						  + "sizes. Setting -ss1m was appropriate for us.";
			}
			try
			{
				if (input != null)
				{
					input.close();
				}
				if (message == null)
				{
					message = "Simulation state restored from " + path + ".";
				}
			}
			catch (StreamCorruptedException e)
			{

				// A RenewObjectInputStream can throw this exception
				// if it is not able to read delayed fields on close().
				// If loadState() was not successfull, this is a natural
				// consequence, so ignore it.
				// The stream has then been closed anyway.
				if (message == null)
				{
					message = "Problem occurred while reading file " + path
							  + ", but all relevant information was already read. "
							  + "Perhaps the simulation state is valid anyway.";
					System.err.println("Exception while closing: " + e);
				}
			}
			catch (Exception e)
			{
				if (message == null)
				{
					message = "Problem occurred while reading file " + path
							  + ", but all relevant information was already read. "
							  + "Perhaps the simulation state is valid anyway.";
				}
				System.err.println("Exception while closing: " + e);
			}
			showStatus(message);
			System.err.println(message);
		}
	}

	/**
	 * Method documentation.
	 * @param file
	 * @param useUFT
	 * @return
	 * @throws FileNotFoundException
	 */
	protected StorableInput makeStorableInput(String file,
											  boolean useUFT) throws FileNotFoundException
	{
		return new PatchingStorableInput(file, useUFT);
	}

	/**
	 * Method documentation.
	 * @param figure
	 * @return
	 */
	public ToolButton toolButtonForTextFigure(TextFigure figure)
	{
		if (figure instanceof CPNTextFigure)
		{
			if (figure instanceof DeclarationFigure)
			{
				return fDeclTB;
			}
			switch (((CPNTextFigure) figure).getType())
			{
				case CPNTextFigure.AUX:
					if (fAuxTB != null)
					{
						return fAuxTB;
					}
					else
					{
						return fInscrTB;
					}
				case CPNTextFigure.INSCRIPTION:
					return fInscrTB;
				case CPNTextFigure.NAME:
					return fNameTB;
				default:
				{
				}
			}
		}
		return super.toolButtonForTextFigure(figure);
	}

	/**
	 * Method documentation.
	 * @param toolPanel
	 */
	protected void createTools(Panel toolPanel)
	{
		super.createTools(toolPanel);

		Palette palette = new Palette(menuFrame, "Petri Net Tools");

		Tool tool;

		tool = getMode().createTransitionTool(this);
		if (tool == null)
		{

			// The ordinary tool suffices.
			tool = new TransitionFigureCreationTool(this);
		}
		palette.add(createToolButton(CPNIMAGES + "TRANS", "Transition Tool", tool));

		tool = getMode().createPlaceTool(this);
		if (tool == null)
		{

			// The ordinary tool suffices.
			tool = new PlaceFigureCreationTool(this);
		}
		palette.add(createToolButton(CPNIMAGES + "PLACE", "Place Tool", tool));

		tool = new VirtualPlaceFigureCreationTool(this);
		palette.add(createToolButton(CPNIMAGES + "VPLACE", "Virtual Place Tool", tool));

		tool = new ConnectionTool(this, ArcConnection.NormalArc);
		palette.add(createToolButton(CPNIMAGES + "ARC", "Arc Tool", tool));

		tool = new ConnectionTool(this, ArcConnection.TestArc);
		palette.add(createToolButton(IMAGES + "LINE", "Test Arc Tool", tool));

		tool = new ConnectionTool(this, ArcConnection.ReserveArc);
		palette.add(createToolButton(IMAGES + "CONN", "Reserve Arc Tool", tool));

		tool = new ConnectionTool(this, DoubleArcConnection.DoubleArc);
		palette.add(createToolButton(CPNIMAGES + "DARC", "Flexible Arc Tool", tool));

		tool = new CPNTextTool(this, CPNTextFigure.Inscription);

		// store the inscription text tool button
		// for immediate editing of errors:
		fInscrTB = createToolButton(CPNIMAGES + "INSCR", "Inscription Tool", tool);
		palette.add(fInscrTB);

		tool = new ConnectedTextTool(this, CPNTextFigure.Name);
		fNameTB = createToolButton(CPNIMAGES + "NAME", "Name Tool", tool);
		palette.add(fNameTB);

		tool = new CPNTextTool(this, new DeclarationFigure(), false);
		fDeclTB = createToolButton(CPNIMAGES + "DECL", "Declaration Tool", tool);
		palette.add(fDeclTB);

		if (getMode().wantAux())
		{
			tool = new CPNTextTool(this, CPNTextFigure.Aux);
			fAuxTB = createToolButton(getMode().getAuxFigureName(), getMode().getAuxToolName(),
									  tool);
			palette.add(fAuxTB);
		}

		// palette.add(new java.awt.Canvas());

		toolPanel.add(palette);

		getMode().createAdditionalTools(this, toolPanel, palette);
	}

	/**
	 * Creates the selection tool used in this editor.
	 */
	protected Tool createSelectionTool()
	{
		return new CPNSelectionTool(this, getMode());
	}

	/**
	 * Checks the items of all menus if they are are
	 * selection sensitive and updates them.
	 */
	public void menuStateChanged()
	{
		MenuBar mb = menuFrame.getMenuBar();
		int count = mb.getMenuCount();

		for (int i = 0; i < count; i++)
		{
			Menu menu = (mb.getMenu(i));

			if (menu instanceof CommandMenu)
			{
				((CommandMenu) menu).checkEnabled();
			}
		}
	}

	// -- main -----------------------------------------------------------

	/**
	 * Method documentation.
	 * @param args
	 */
	public static void main(String[] args)
	{
		System.out.println("The Reference Net Workshop GUI");
		System.out.println();

		ParameteredCommandLine line = new ParameteredCommandLine(args,
			new String[] {"-h"}, new int[] {0});
		args = line.getRemainingArgs();

		if (line.hasParameter("-h"))
		{
			System.out.println("Parameters: [<drawing> ...]");
			System.out.println("  <drawing> : The drawing files to be read.");

			System.exit(0);
		}

		new CPNApplication(args);
	}
}
