/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

package de.renew.gui;

import CH.ifa.draw.framework.*;
import CH.ifa.draw.standard.*;
import de.renew.database.*;
import de.renew.remote.*;
import de.renew.simulator.*;
import de.renew.util.RenewObjectInputStream;
import de.renew.util.RenewObjectOutputStream;
import de.renew.util.StringUtil;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.io.StreamCorruptedException;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import javax.swing.JOptionPane;


/**
 * Class documentation.
 */
public class CPNInstanceDrawing extends StandardDrawing implements DrawingChangeListener,
		DrawingContext
{

	public static String ID_MACRO = "$ID";

	private static Hashtable drawingsByInstance = new Hashtable();

	private NetInstanceAccessor netInstance;
	private CPNDrawing cpnDrawing;
	private RenewMode mode;
	private Hashtable instanceLookup;

	/**
	 * Constructor documentation.
	 * @param mode
	 * @param netInstance
	 * @param drawing
	 * @exception RemoteException If an RMI problem occurred.
	 */
	private CPNInstanceDrawing(RenewMode mode, NetInstanceAccessor netInstance, CPNDrawing drawing)
		throws RemoteException
	{
		this.mode = mode;
		this.cpnDrawing = drawing;
		connect(netInstance);
		cpnDrawing.addDrawingChangeListener(this);
	}

	/**
	 * Method documentation.
	 * @return
	 */
	public RenewMode getMode()
	{
		return mode;
	}

	/**
	 * Get the net instance accessor associated with this drawing.
	 */
	public NetInstanceAccessor getNetInstance()
	{
		return netInstance;
	}

	/**
	 * Method documentation.
	 * @return
	 */
	public Rectangle displayBox()
	{
		return super.displayBox().union(cpnDrawing.displayBox());
	}

	/**
	 * Returns the instance drawing associated to the given net
	 * instance accessor, creates the drawing if neccessary.
	 *
	 * @param mode needed to look up the responsible simulation
	 * @param netInstance to look up the drawing for
	 * @return CPNInstanceDrawing which displays the given netInstance.
	 * If there is currently no such drawing, one is created.
	 * If there is not even any CPNDrawing matching the net
	 * known to the active simulation, returns
	 * <code>null</code>.
	 * @exception RemoteException If an RMI problem occurred.
	 */
	static CPNInstanceDrawing getInstanceDrawing(RenewMode mode, NetInstanceAccessor netInstance)
		throws RemoteException
	{
		// forbid concurrent searches for instance drawings.
		synchronized (drawingsByInstance)
		{
			// find instance drawing:
			String key = netInstance.getNet().getName() + ":" + netInstance.getID();
			if (drawingsByInstance.containsKey(key))
			{
				return (CPNInstanceDrawing) drawingsByInstance.get(key);
			}
			else
			{
				// find drawing to base the instance drawing on
				CPNDrawing drawing = mode.getDrawingLoader().getDrawing(netInstance.getNet().getName());

				if (drawing != null)
				{
					return new CPNInstanceDrawing(mode, netInstance, drawing);
				}
				else
				{
					System.err.println("To display a net instance, an open"
									   + " drawing of the instantiated net is"
									   + " needed. No such drawing found for net \""
									   + netInstance.getNet().getName() + "\".");
				}
				return null;
			}
		}
	}

	/**
	 * Returns all instance drawings based on the specified CPNDrawing.
	 * Returns <code>null</code>, if no such drawing exists.
	 */
	public static Enumeration getDependentInstanceDrawings(CPNDrawing baseDrawing)
	{
		Enumeration allDrawings = drawingsByInstance.elements();
		Vector dependentDrawings = new Vector();

		while (allDrawings.hasMoreElements())
		{
			CPNInstanceDrawing oneDrawing = (CPNInstanceDrawing) allDrawings.nextElement();

			if (oneDrawing.cpnDrawing == baseDrawing)
			{
				dependentDrawings.addElement(oneDrawing);
			}
		}

		if (dependentDrawings.size() > 0)
		{
			return dependentDrawings.elements();
		}
		else
		{
			return null;
		}
	}

	/**
	 * Method documentation.
	 * @param g
	 */
	public void draw(Graphics g)
	{

		// g.setColor(Color.yellow.brighter());
		// Rectangle b=displayBox();
		// g.fillRect(b.x,b.y,b.width,b.height);
		cpnDrawing.draw(g, this);
		super.draw(g);
	}

	/**
	 * Method documentation.
	 * @param fig
	 * @return
	 */
	public InstanceFigure getInstanceFigure(Figure fig)
	{
		return (InstanceFigure) instanceLookup.get(fig);
	}

	/**
	 * Method documentation.
	 * @param fig
	 * @return
	 */
	private boolean isEmphasized(Figure fig)
	{
		FigureWithHighlight node = cpnDrawing.getFigureForHighlight(fig);

		if (node != null)
		{
			fig = node;
		}
		InstanceFigure instFig = getInstanceFigure(fig);

		if (instFig != null)
		{
			return instFig.isHighlighted();
		}
		return false;
	}

	/**
	 * Returns whether the CPN instance drawing shows a local net instance.
	 * Otherwise, the simulation of the shown net instance is run in a
	 * different VM, usually on another computer.
	 * @return Whether the shown net instance is local.
	 */
	public boolean isLocal()
	{
		return netInstance instanceof NetInstanceAccessorImpl;
	}

	/**
	 * Method documentation.
	 * @param fig
	 * @return
	 */
	public boolean isHighlighted(Figure fig)
	{
		return fig.isVisible() && isEmphasized(fig);
	}

	/**
	 * Method documentation.
	 * @param fig
	 * @return
	 */
	public boolean isVisible(Figure fig)
	{
		return (fig.isVisible() || isEmphasized(fig))
			&& (!(((fig instanceof CPNTextFigure
			&& ((CPNTextFigure) fig).getType() == CPNTextFigure.INSCRIPTION
			&& ((CPNTextFigure) fig).parent() instanceof PlaceFigure))
			|| fig == cpnDrawing.getIconFigure()));
	}

	/**
	 * Method documentation.
	 * @param text
	 * @return
	 */
	public String expandMacro(String text)
	{
		return expandMacro(text, netInstance);
	}

	/**
	 * Method documentation.
	 * @param text
	 * @param instance
	 * @return
	 */
	public static String expandMacro(String text, NetInstanceAccessor instance)
	{
		try
		{
			return StringUtil.replace(text, ID_MACRO, instance.getID());
		}
		catch (RemoteException e)
		{
			return e.toString();
		}
	}

	/**
	 * Creates instance figures in this instance drawing which
	 * reflect the net elements of the given net instance.
	 * Used by the (private) constructor to build an instance
	 * drawing.
	 * <p>
	 * Also... <ul>
	 * <li> builds the instance lookup table
	 * (see <code>getInstanceFigure()</code>) </li>
	 * <li> registers this instance drawing in the
	 * (private) drawingsByInstance map
	 * (see <code>getInstanceDrawing()</code>) </li>
	 * </ul></p>
	 * May be more, but that is what I could see in the code.
	 *
	 * @param netInstance the net instance to be displayed
	 * @see #getInstanceFigure
	 * @see #getInstanceDrawing
	 * @exception RemoteException If an RMI problem occurred.
	 */
	public void connect(NetInstanceAccessor netInstance)
		throws RemoteException
	{
		if (netInstance != null)
		{
			if (this.netInstance != null)
			{
				String key = netInstance.getNet().getName() + ":" + netInstance.getID();
				drawingsByInstance.remove(key);
				notifyTransactionStrategyAboutRelease();
			}

			this.netInstance = netInstance;
			String key = netInstance.getNet().getName() + ":" + netInstance.getID();
			drawingsByInstance.put(key, this);
			notifyTransactionStrategyAboutConnect();

			setName(netInstance.asString());
			instanceLookup = new Hashtable();

			NetInstanceElementLookup netElementLookup = buildNetElementLookup(netInstance);
			Enumeration figures = netElementLookup.getFigures();

			while (figures.hasMoreElements())
			{
				FigureWithID figure = (FigureWithID) figures.nextElement();

				Hashtable netElements = netElementLookup.getNetElements(figure);
				if (netElements != null)
				{
					if (figure instanceof PlaceFigure)
					{
						Figure pif = new PlaceInstanceFigure(this, (PlaceFigure) figure, netElements);
						instanceLookup.put(figure, pif);
						add(pif);
						sendToBack(pif);
					}
					else if (figure instanceof TransitionFigure)
					{
						Figure tif = ((TransitionFigure) figure)
							.createInstanceFigure(this, netElements);
						instanceLookup.put(figure, tif);
						add(tif);
					}
				}
			}
		}
	}

	/**
	 * Tries to map graphical figures of an open CPNDrawing
	 * to net elements of the given net instance.
	 * @param netInstance The net instance.
	 * @return The net element, mapping figures to mappings
	 * of group ids to net elements.
	 * @exception RemoteException If an RMI problem occurred.
	 */
	protected NetInstanceElementLookup buildNetElementLookup(NetInstanceAccessor netInstance)
		throws RemoteException
	{
		NetAccessor net = netInstance.getNet();
		String netName = net.getName();
		CPNDrawing netDrawing = mode.getDrawingLoader().getDrawing(netName);
		if (netDrawing == null)
		{
			System.err.println("No open drawing found matching the compiled net \""
				+ netName + "\"");
			return null;
		}

		// Get a lookup for the figures.
		// It maps figure ids to lists of figures.
		Hashtable figureLookup = new Hashtable();
		addToFigureLookup(netDrawing, figureLookup);

		boolean missingFigure = false;
		boolean missingNetElement = false;
		NetInstanceElementLookup netElementLookup = new NetInstanceElementLookup();

		// Fill the net element lookup with places
		NetElementID[] placeIDs = net.getPlaceIDs();
		for (int i=0; i<placeIDs.length; i++)
		{
			List figures = (List) figureLookup.get(new Integer(placeIDs[i].getFigureID()));
			if (figures == null)
			{
				System.err.println("Warning: The place with figure id "
					+ placeIDs[i].getFigureID() + " has no corresponding graphical element.");
				missingFigure = true;
			}
			else
			{
				Iterator iterator = figures.iterator();
				while (iterator.hasNext())
				{
					FigureWithID figure = (FigureWithID) iterator.next();
					netElementLookup.put(figure, netInstance.getPlaceInstance(placeIDs[i]));
				}
			}
		}

		// Fill the net element lookup with transitions
		NetElementID[] transitionIDs = net.getTransitionIDs();
		for (int i=0; i<transitionIDs.length; i++)
		{
			List figures = (List) figureLookup.get(new Integer(transitionIDs[i].getFigureID()));
			if (figures == null)
			{
				System.err.println("Warning: The transition with figure id "
					+ transitionIDs[i].getFigureID() + " has no corresponding graphical element.");
				missingFigure = true;
			}
			else
			{
				Iterator iterator = figures.iterator();
				while (iterator.hasNext())
				{
					FigureWithID figure = (FigureWithID) iterator.next();
					netElementLookup.put(figure, netInstance.getTransitionInstance(transitionIDs[i]));
				}
			}
		}

		// Check if any figure has no net elements
		Enumeration figuresEnum = figureLookup.elements();
		while (figuresEnum.hasMoreElements())
		{
			Iterator iterator = ((List) figuresEnum.nextElement()).iterator();
			while (iterator.hasNext())
			{
				FigureWithID figure = (FigureWithID) iterator.next();
				Hashtable groupIdToNetElementMap = netElementLookup.getNetElements(figure);
				if (groupIdToNetElementMap == null || groupIdToNetElementMap.isEmpty())
				{
					System.err.println("Warning: " + figure.getClass().getName()
						+ " with id " + figure.getID() + " has no corresponding net element.");
					missingNetElement = true;
				}
			}
		}

		if (missingFigure)
		{
			System.err.println("Warning: The compiled net"
				+ " contains one or more net elements with no"
				+ " corresponding graphical element in the drawing \""
				+ netName + "\".");
			System.err.println("These net elements will not be displayed.");
		}
		if (missingNetElement)
		{
			System.err.println("Warning: The drawing \""
				+ netName + "\" contains one or more graphical"
				+ " elements with no corresponding net element.");
			System.err.println("These graphical elements will not be functional.");
		}

		return netElementLookup;
	}

	/**
	 * Builds up a mapping for figure ids to figures for the elements
	 * of a given composite figure.
	 * @param container The composite figure.
	 * @param figureLookup The figure lookup to be filled with the mapping.
	 */
	private void addToFigureLookup(CompositeFigure container, Hashtable figureLookup)
	{
		Enumeration figuresEnum = container.figures();
		while (figuresEnum.hasMoreElements())
		{
			Figure figure = (Figure) figuresEnum.nextElement();
			if (figure instanceof CompositeFigure)
			{
				addToFigureLookup((CompositeFigure) figure, figureLookup);
			}
			else if (figure instanceof PlaceFigure)
			{
				Integer id = new Integer(((PlaceFigure) figure).getSemanticPlaceFigure().getID());
				List figures = (List) figureLookup.get(id);
				if (figures == null)
				{
					figures = new ArrayList();
					figureLookup.put(id, figures);
				}
				figures.add(figure);
			}
			else if (figure instanceof TransitionFigure)
			{
				Integer id = new Integer(((TransitionFigure) figure).getID());
				List figures = (List) figureLookup.get(id);
				if (figures == null)
				{
					figures = new ArrayList();
					figureLookup.put(id, figures);
				}
				figures.add(figure);
			}
		}
	}

	/**
	 * Method documentation.
	 */
	public void release()
	{
		cpnDrawing.removeDrawingChangeListener(this);
		if (netInstance != null)
		{
			try
			{
				String key = netInstance.getNet().getName() + ":" + netInstance.getID();
				drawingsByInstance.remove(key);
			}
			catch (RemoteException e)
			{
				e.printStackTrace(System.err);
				JOptionPane.showMessageDialog(null,
					"A problem occurred: " + e + "\n"
					+ "See the console for details.", "Renew", JOptionPane.ERROR_MESSAGE);
			}

			notifyTransactionStrategyAboutRelease();

			// System.err.println("Instance Drawing removed from list.");
			instanceLookup = null;
			netInstance = null;
		}
		super.release();
	}

	/**
	 * Notifies the TransactionSource about the connect of
	 * the current netInstance.
	 */
	private void notifyTransactionStrategyAboutConnect()
	{
		try
		{
			TransactionSource.netInstanceDrawingOpened(netInstance);
		}
		catch (Exception e)
		{
			e.printStackTrace(System.err);
			JOptionPane.showMessageDialog(null,
				"A problem occurred: " + e + "\n"
				+ "See the console for details.", "Renew", JOptionPane.ERROR_MESSAGE);
		}
	}

	/**
	 * Notifies the TransactionSource about the release of
	 * the current netInstance.
	 */
	private void notifyTransactionStrategyAboutRelease()
	{
		try
		{
			TransactionSource.netInstanceDrawingClosed(netInstance);
		}
		catch (Exception e)
		{
			e.printStackTrace(System.err);
			JOptionPane.showMessageDialog(null,
				"A problem occurred: " + e + "\n"
				+ "See the console for details.", "Renew", JOptionPane.ERROR_MESSAGE);
		}
	}

	/**
	 * Sent when an area is invalid
	 */
	public void drawingInvalidated(DrawingChangeEvent e)
	{

		// System.out.println("Source invalidated: "+e.getInvalidatedRectangle());
		DrawingChangeEvent myEvent = new DrawingChangeEvent(this, e.getInvalidatedRectangle());
		Enumeration listeners = drawingChangeListeners();

		while (listeners.hasMoreElements())
		{
			DrawingChangeListener listener = (DrawingChangeListener) listeners.nextElement();

			listener.drawingInvalidated(myEvent);
			listener.drawingRequestUpdate(myEvent);
		}
	}

	/**
	 * Sent when the drawing wants to be refreshed
	 */
	public void drawingRequestUpdate(DrawingChangeEvent e)
	{

		// System.out.println("Source requests update: "+e.getInvalidatedRectangle());
	}


	/**
	 * Returns whether drawing has been modified since last save.
	 */
	public boolean isModified()
	{
		return false;	 // an InstanceDrawing should not be saved!
	}


	/**
	 * Method documentation.
	 * @return
	 */
	public Dimension defaultSize()
	{
		return cpnDrawing.defaultSize();
	}

	public String getWindowCategory()
	{
		if (isLocal()) {
			return "Net instances";
		} else {
			return "Remote net instances";
		}
	}

	/**
	 * Acquires the drawing lock of this drawing and the
	 * background drawing. The background drawing is locked first.
	 */
	public void lock()
	{
		cpnDrawing.lock();
		super.lock();
	}

	/**
	 * Releases the drawing lock of this drawing and the
	 * background drawing.
	 */
	public void unlock()
	{
		super.unlock();
		cpnDrawing.unlock();
	}

	/**
	 * This class is not serializable, although inheriting
	 * serializability from its superclass.
	 * <p>
	 * Reason: Serialization is currently used for Copy&Paste
	 * and the Undo machanism only. As such modifications don't
	 * make sense for instance drawings, this class doesn't
	 * need to be serializable.
	 * </p><p>
	 * Second reason: Some fields of this class would lead to
	 * a serialization of the compiled net, which is not desirable.
	 * </p>
	 * @throws java.io.NotSerializableException always.
	 */
	private void writeObject(java.io.ObjectOutputStream out) throws java.io.IOException
	{
		throw new java.io.NotSerializableException("CPNInstanceDrawing is not serializable!");
	}

	/**
	 * Writes all currently known <code>NetInstances</code>s
	 * of the currently running local simulation
	 * to the given stream. The written information is <b>not</b>
	 * guaranteed to describe the complete current simulation
	 * state!
	 * A <code>NetInstances</code> is known, if it is displayed
	 * by a <code>CPNInstanceDrawing</code>.
	 * Net instances of remote simulations are ignored.
	 * <p>
	 * If the given <code>ObjectOutput</code> is a <code><b>
	 * de.renew.util.RenewObjectOutputStream</b></code>, its
	 * feature of cutting down the recursion depth by delaying
	 * the serialization of some fields will be used.
	 * </p><p>
	 * <b>Side effect:</b> Any open <code>BindingSelectionFrame</code>
	 * will be closed on execution of this method!
	 * <br></br>
	 * <b>Caution:</b> In order to get consistent data written
	 * to the stream, you have to ensure that there are no
	 * concurrent modifications of the simulation state.
	 * This method is not able to lock the simulation.
	 * </p>
	 *
	 * Added Feb 29 2000  Michael Duvigneau
	 *
	 * @param output target stream (see note about RenewObjectOutputStream)
	 * @see de.renew.util.RenewObjectOutputStream
	 */
	public static void saveAllInstances(ObjectOutput output) throws IOException
	{
		Enumeration enum;
		Vector saveList;
		RenewObjectOutputStream rOut = null;
		CPNInstanceDrawing drawing;
		NetInstanceAccessorImpl accessor;

		if (output instanceof RenewObjectOutputStream)
		{
			rOut = (RenewObjectOutputStream) output;

		}
		if (rOut != null)
		{
			rOut.beginDomain(CPNInstanceDrawing.class);
		}

		// Close the BindingSelectionFrame (otherwise it would
		// be stored with its transition).
		BindingSelectionFrame.close();

		// Determine which net instances are to be saved.
		// Go through all instance drawings and extract references to
		// local net instances.
		saveList = new Vector(drawingsByInstance.size());
		enum = drawingsByInstance.elements();
		while (enum.hasMoreElements()) {
			drawing = (CPNInstanceDrawing)enum.nextElement();
			if (drawing.isLocal()) {
				saveList.addElement(drawing.getNetInstance());
			}
		}

		// Save all currently known instances.
		output.writeInt(saveList.size());
		enum = saveList.elements();
		while (enum.hasMoreElements())
		{
			accessor = (NetInstanceAccessorImpl)enum.nextElement();
			accessor.writeTo(output);
		}

		// If a RenewObjectOutputStream is used, write
		// all delayed fields NOW.
		if (rOut != null)
		{
			rOut.writeDelayedObjects();

		}
		if (rOut != null)
		{
			rOut.endDomain(CPNInstanceDrawing.class);
		}
	}

	/**
	 * Reads NetInstances stored by <code>saveAllInstances()
	 * </code>.
	 * <p>
	 * This method does neither automatically add the new<code>
	 * NetInstance</code>s to the list of known instances nor
	 * open a CPNInstanceDrawing for them.
	 * The reason is that first the active simulation has to
	 * rebuild its FigureLookup.
	 * It is then left up to the caller to request the
	 * appropriate <code>CPNInstanceDrawing</code>s via
	 * <code>getInstanceDrawing()</code>.
	 * </p><p>
	 * If the given <code>ObjectInput</code> is a <code>
	 * de.renew.util.RenewObjectInputStream</code>, the
	 * neccessary steps to cover delayed serialization will
	 * be made.
	 * </p><p>
	 * The ObjectInputStream will be read using
	 * <code>de.renew.util.ClassSource</code> to provide
	 * its ability of reloading all user defined classes.
	 * </p>
	 *
	 * Added Feb 29 2000  Michael Duvigneau
	 *
	 * @return Enumeration of NetInstances
	 * @param input source stream (see note about RenewObjectInputStream)
	 * @see #saveAllInstances
	 * @see de.renew.util.ClassSource
	 * @see de.renew.util.RenewObjectInputStream
	 */
	public static Enumeration loadInstances(ObjectInput input)
			throws IOException, ClassNotFoundException
	{

		// Read all stored NetInstances.
		int count = input.readInt();
		Vector newInstances = new Vector(count);

		try
		{
			for (int i = 0; i < count; i++)
			{
				newInstances.addElement((NetInstance) de.renew.util.ClassSource.readObject(input));
			}
		}
		catch (ClassCastException e)
		{
			throw new StreamCorruptedException("Object other than NetInstance found "
											   + "when looking for net instances: " + e);
		}

		// If a RenewObjectInputStream is used, read
		// all delayed fields NOW.
		if (input instanceof RenewObjectInputStream)
		{
			((RenewObjectInputStream) input).readDelayedObjects();

			// Return the list of read NetInstances.
		}
		return newInstances.elements();
	}

	/**
	 * The net instance element lookup class maps figures to net instance
	 * elements. Since there may be several net instance elements for one
	 * figure, the figures are mapped to Hashtables of group id and net
	 * instance element pairs. The class provides functionality for x to 1
	 * mappings like for places and transitions, and functionality for x
	 * to n mappings like tasks and other aggregations. x usually is 1 in
	 * these cases, but n is also supported, of course.
	 */
	private class NetInstanceElementLookup
	{
		/**
		 * The lookup mapping figures to hashtables, which map
		 * net element group ids to net instance elements.
		 */
		Hashtable lookup;

		/**
		 * Creates a new net instance element lookup.
		 */
		private NetInstanceElementLookup()
		{
			lookup = new Hashtable();
		}

		/**
		 * Returns an enumeration of all figures in this lookup.
		 * @return The enumeration.
		 */
		private Enumeration getFigures()
		{
			return lookup.keys();
		}

		/**
		 * For a given figure, it returns a lookup that maps net
		 * element group ids to net elements.
		 * @param figure The figure.
		 * @return The lookup.
		 */
		private Hashtable getNetElements(FigureWithID figure)
		{
			return (Hashtable) lookup.get(figure);
		}

		/**
		 * Adds a figure to place instance relation to the lookup.
		 * @param figure The figure.
		 * @param placeInstance The place instance.
		 * @exception RemoteException If an RMI problem occurred.
		 */
		private void put(FigureWithID figure, PlaceInstanceAccessor placeInstance)
			throws RemoteException
		{
			Hashtable groupIdToNetElementMap = getNetElements(figure);
			if (groupIdToNetElementMap == null)
			{
				groupIdToNetElementMap = new Hashtable();
				lookup.put(figure, groupIdToNetElementMap);
			}
			groupIdToNetElementMap.put(placeInstance.getPlace().getID().getGroupID(),
				placeInstance);
		}

		/**
		 * Adds a figure to transition instance relation to the lookup.
		 * @param figure The figure.
		 * @param transitionInstance The transition instance.
		 * @exception RemoteException If an RMI problem occurred.
		 */
		private void put(FigureWithID figure, TransitionInstanceAccessor transitionInstance)
			throws RemoteException
		{
			Hashtable groupIdToNetElementMap = getNetElements(figure);
			if (groupIdToNetElementMap == null)
			{
				groupIdToNetElementMap = new Hashtable();
				lookup.put(figure, groupIdToNetElementMap);
			}
			groupIdToNetElementMap.put(transitionInstance.getTransition().getID().getGroupID(),
				transitionInstance);
		}
	}
}
