/*
 * @(#CPNTextTool.java 1.0
 *
 */

package de.renew.gui;

import java.awt.*;
import java.awt.event.*;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.standard.*;
import CH.ifa.draw.figures.*;
import CH.ifa.draw.util.FloatingTextField;
import de.renew.shadow.*;
import de.renew.formalism.java.JavaNetCompiler;

public class CPNTextTool extends ConnectedTextTool {

    private CPNTextFigure fTextFigure=null;
    private CPNDrawing fDrawing=null;

    public CPNTextTool(DrawingEditor editor, CPNTextFigure prototype) {
        super(editor, prototype);
    }

    public CPNTextTool(DrawingEditor editor, CPNTextFigure prototype,
		       boolean mustConnect) {
        super(editor, prototype, mustConnect);
    }

    public void beginEdit(TextHolder figure) {
        super.beginEdit(figure);
        if (figure instanceof CPNTextFigure &&
	    fEditor.drawing() instanceof CPNDrawing) {
           fTextFigure = (CPNTextFigure)figure;
           fDrawing=(CPNDrawing)fEditor.drawing();
        } else {
        	  fTextFigure = null;
        }
    }

    public void endEdit() {
      super.endEdit();
      if (fTextFigure!=null) {
	CPNTextFigure textFigure=fTextFigure;
	fTextFigure=null;
	int type=textFigure.getType();
	boolean special=type==CPNTextFigure.AUX;
	if (type==CPNTextFigure.INSCRIPTION || special) {
	  String newText=textFigure.getText();
	  if (newText.length()>0) {
	    //System.out.println("New inscription: "+newText);
	    try {
	      CPNSimulation sim=((CPNApplication)fEditor)
		.getMode().getSimulation();
	      sim.buildAllShadows();
	      ShadowNetSystem netSystem=sim.getNetSystem();
	      ShadowCompiler compiler=netSystem.getCompiler();

	      if (textFigure instanceof DeclarationFigure) {
		fEditor.showStatus("Checking Declaration Node...");
		compiler.checkDeclarationNode(newText,special);
	      } else {
		ShadowHolder parent = (ShadowHolder)textFigure.parent();
		ShadowNet shadowNet = fDrawing.buildShadow(netSystem);
		// quite a hack to get DeclarationNode information into
		// figures implementing the SemanticUpdateFigure interface:
		if (textFigure instanceof SemanticUpdateFigure &&
		    compiler instanceof JavaNetCompiler) {
		    ((SemanticUpdateFigure)textFigure)
			.semanticUpdate(((JavaNetCompiler)compiler)
			                .makeDeclarationNode(shadowNet));
		}
		if (parent != null && shadowNet != null) {
		  if (parent instanceof TransitionFigure) {
		    fEditor.showStatus("Checking Transition Inscription...");
		    compiler.checkTransitionInscription(newText,special,
		      shadowNet);
		  } else if (parent instanceof PlaceFigure) {
		    fEditor.showStatus("Checking Place Inscription...");
		    compiler.checkPlaceInscription(newText,special,
		      shadowNet);
		  } else if (parent instanceof ArcConnection) {
		    fEditor.showStatus("Checking Arc Inscription...");
		    compiler.checkArcInscription(newText,special,
		      shadowNet);
		  }
		}
	      }
	      ((CPNApplication)fEditor).closeSyntaxErrorFrame();
	    } catch (SyntaxException e) {
	      fEditor.showStatus("A syntax error occured.");
              FigureException fe= FigureExceptionFactory.createFigureException(e);
              // Test if the syntax error was in the edited text:
              if (fe.errorDrawing==null) {
                 fe = FigureExceptionFactory.createFigureException
		        (e,fDrawing,textFigure);
              }
	      ((CPNApplication)fEditor).processSyntaxException(fe,false);
	      //System.out.println(e.toString());
	    }
	  }
	}
      }
    }

}

