package de.renew.gui;

import de.renew.remote.Server;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;


/**
 * The password dialog is the login dialog for the agenda GUI.
 * It shows up with the user name and a password enter field.
 */
class ConnectDialog extends JDialog
{
	/**
	 * The user name field.
	 */
	private JTextField hostField;

	/**
	 * The password field.
	 */
	private JTextField serverField;

	/**
	 * Whether the dialog was cancelled.
	 */
	private boolean cancelled;

	/**
	 * Creates a new net instance selection dialog.
	 * @param owner The owning frame.
	 * @exception UnsupportedLookAndFeelException If the
	 * selected look and feel is not supported.
	 * @exception RemoteException An RMI problem occurred.
	 */
	ConnectDialog(Frame owner)
		throws UnsupportedLookAndFeelException
	{
		super(owner, "Please specify the simulation server", true);

		Box mainBox = Box.createVerticalBox();
		mainBox.add(Box.createVerticalGlue());
		mainBox.add(Box.createRigidArea(new Dimension(0, 8)));

		hostField = new JTextField("", 32);
		hostField.setMaximumSize(hostField.getPreferredSize());
		Box hostBox = Box.createHorizontalBox();
		hostBox.add(Box.createHorizontalBox());
		hostBox.add(Box.createRigidArea(new Dimension(8, 0)));
		hostBox.add(new JLabel("Host computer:"));
		hostBox.add(Box.createRigidArea(new Dimension(8, 0)));
		hostBox.add(hostField);
		hostBox.add(Box.createRigidArea(new Dimension(8, 0)));
		hostBox.add(Box.createHorizontalBox());
		mainBox.add(hostBox);
		mainBox.add(Box.createRigidArea(new Dimension(0, 8)));

		serverField = new JTextField(Server.DEFAULT_SERVER_NAME, 16);
		serverField.setMaximumSize(serverField.getPreferredSize());
		Box serverBox = Box.createHorizontalBox();
		serverBox.add(Box.createRigidArea(new Dimension(8, 0)));
		serverBox.add(Box.createHorizontalGlue());
		serverBox.add(new JLabel("Server name:"));
		serverBox.add(Box.createRigidArea(new Dimension(8, 0)));
		serverBox.add(serverField);
		serverBox.add(Box.createRigidArea(new Dimension(8, 0)));
		serverBox.add(Box.createHorizontalGlue());
		mainBox.add(serverBox);
		mainBox.add(Box.createRigidArea(new Dimension(0, 8)));

		Box buttonsBox = Box.createHorizontalBox();
		buttonsBox.add(Box.createHorizontalGlue());
		buttonsBox.add(Box.createRigidArea(new Dimension(8, 0)));

		JButton okButton = new JButton("OK");
		okButton.setDefaultCapable(true);
		okButton.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent event)
			{
				cancelled = false;
				dispose();
			}
		});
		buttonsBox.add(okButton);
		buttonsBox.add(Box.createRigidArea(new Dimension(8, 0)));

		JButton cancelButton = new JButton("Cancel");
		cancelButton.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent event)
			{
				cancelled = true;
				dispose();
			}
		});
		buttonsBox.add(cancelButton);
		buttonsBox.add(Box.createRigidArea(new Dimension(8, 0)));
		buttonsBox.add(Box.createHorizontalGlue());
		mainBox.add(buttonsBox);
		mainBox.add(Box.createRigidArea(new Dimension(0, 8)));

		mainBox.add(Box.createVerticalGlue());
		getContentPane().add(mainBox);

		setSize(500, 150);
		pack();
		show();
	}

	/**
	 * The user name string.
	 * The user name string.
	 */
	String getHost()
	{
		return hostField.getText();
	}

	/**
	 * The password string.
	 * The password string.
	 */
	String getServer()
	{
		return serverField.getText();
	}

	/**
	 * Returns whether the dialog has been dismissed with cancel.
	 * @return Whether the dialog has been dismissed with cancel.
	 */
	boolean hasCancelled()
	{
		return cancelled;
	}
}
