package de.renew.gui;

import java.util.Vector;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import collections.*;
import CH.ifa.draw.framework.*;
import de.renew.gui.*;

public class Demonstrator extends CPNApplication {
  public static void main(final String[] args) {
    System.out.println("The Renew demonstration GUI");
    System.out.println();
    if (args.length==0) {
      System.out.println("Usage: java de.renew.gui.Demonstrator files");
      System.out.println("or     java de.renew.gui.Demonstrator -conffile");
    } else if (args[0].startsWith("-")) {
      try {
	FileReader fileReader=new FileReader(args[0].substring(1));
	BufferedReader reader=new BufferedReader(fileReader);

	Vector names=new Vector();
	while (reader.ready()) {
	  String name=reader.readLine();
	  if (name!=null && !name.equals("")) {
	    names.addElement(name);
	  }
	}
	String[] arr=new String[names.size()];
	names.copyInto(arr);
	setup(arr);
      }
      catch (Exception e) {
	System.out.println("Could not access configuration file.");
      }
    } else {
      setup(args);
    }
  }
  
  public static void setup(final String[]args) {
    final CPNApplication app=new CPNApplication();
    
    final Frame frame=new Frame("Demonstration Drawings");
    frame.setSize(600,200);
    GridBagLayout gridBag=new GridBagLayout();
    frame.setLayout(gridBag);
    
    GridBagConstraints c=new GridBagConstraints();
    c.fill=GridBagConstraints.BOTH;
    c.weightx=1;
    c.weighty=1;
    c.gridwidth = GridBagConstraints.REMAINDER;
   
    final List list=new List();
    gridBag.setConstraints(list,c);
    frame.add(list);
    for (int i=0;i<args.length;i++) {
      list.add(args[i]);
    }
    
    Panel panel=new Panel();
    c.fill=GridBagConstraints.HORIZONTAL;
    c.weighty=0;
    gridBag.setConstraints(panel,c);
    frame.add(panel);
    
    gridBag=new GridBagLayout();
    panel.setLayout(gridBag);
    c=new GridBagConstraints();
    c.weightx=1;
    c.fill=GridBagConstraints.BOTH;
    c.gridwidth=1;
    
    Button button;
    button=new Button("Replace");
    button.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  int i=list.getSelectedIndex();
	  if (i>=0 && i<args.length) {
	    app.getMode().getSimulation().simulationTerminate();
	    
	    LinkedList drawings=new LinkedList();
	    drawings.prependElements(app.drawings());
	    Enumeration enum=drawings.elements();
	    while (enum.hasMoreElements()) {
	      Drawing drawing=(Drawing)enum.nextElement();
	      if (drawing instanceof CPNInstanceDrawing) {
		app.closeDrawing(drawing);
	      }
	    }
	    
	    enum=drawings.elements();
	    while (enum.hasMoreElements()) {
	      Drawing drawing=(Drawing)enum.nextElement();
	      if (!(drawing instanceof CPNInstanceDrawing)) {
		app.closeDrawing(drawing);
	      }
	    }

	    StringTokenizer tokenizer=new StringTokenizer(args[i],",",false);
	    while (tokenizer.hasMoreTokens()) {
	      app.loadAndOpenDrawing(tokenizer.nextToken());
	    }
	  }
	}
      });
    gridBag.setConstraints(button,c);
    panel.add(button);
    
    button=new Button("Include");
    button.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  int i=list.getSelectedIndex();
	  if (i>=0 && i<args.length) {
	    StringTokenizer tokenizer=new StringTokenizer(args[i],",",false);
	    while (tokenizer.hasMoreTokens()) {
	      app.loadAndOpenDrawing(tokenizer.nextToken());
	    }
	  }
	}
      });
    gridBag.setConstraints(button,c);
    panel.add(button);
    
    button=new Button("Exit");
    button.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  System.exit(0);
	}
      });
    gridBag.setConstraints(button,c);
    panel.add(button);

    frame.show();
  }
}

