package de.renew.gui;

import de.renew.shadow.ShadowNetLoader;
import de.renew.shadow.DefaultShadowNetLoader;
import de.renew.shadow.ShadowNetSystem;
import de.renew.shadow.ShadowNet;
import de.renew.simulator.NetNotFoundException;

/**
 * A shadow net loader implementation which first delegates to a
 * {@link DefaultShadowNetLoader} and then to the gui's
 * {@link CPNDrawingLoader}.
 * <p>
 * </p>
 * GuiShadowNetLoader.java
 * Created: Tue Dec  5  2001
 * @author Michael Duvigneau
 **/
class GuiShadowNetLoader implements ShadowNetLoader {

	private ShadowNetLoader shadowLoader;

	private CPNDrawingLoader drawingLoader;

	public GuiShadowNetLoader(CPNDrawingLoader drawingLoader) {
		this.shadowLoader = new DefaultShadowNetLoader();
		this.drawingLoader = drawingLoader;
	}

	public ShadowNet loadShadowNet(String netName, ShadowNetSystem netSystem)
		throws NetNotFoundException
	{
		try {
			return shadowLoader.loadShadowNet(netName, netSystem);			
		} catch (NetNotFoundException e) {
			// So there is no matching shadow net file.
			// Let's look for drawing files...
		}

		CPNDrawing drawing = drawingLoader.getDrawing(netName);
		if (drawing != null) {
			try {
				ShadowNet net = drawing.buildShadow(netSystem);
				if (net != null) {
					return net;
				}
			} catch (Exception e) {
				throw new NetNotFoundException(netName, e);
			}
		} 
		throw new NetNotFoundException(netName);
	}
}
