package de.renew.gui;

import CH.ifa.draw.framework.UndoableCommand;
import CH.ifa.draw.framework.DrawingEditor;
import CH.ifa.draw.framework.Drawing;
import CH.ifa.draw.framework.DrawingView;
import CH.ifa.draw.framework.FigureEnumeration;
import CH.ifa.draw.framework.Figure;
import java.util.Vector;

/**
 * Associates a highlight figure to a highlightable figure.
 *
 * <p></p>
 * HighlightAssociateCommand.java
 * Created: Mon Feb 26  2001
 * (Code moved from CPNApplication)
 *
 * @author Frank Wienberg, Michael Duvigneau
 */

public class HighlightAssociateCommand extends UndoableCommand {

  private DrawingEditor editor;

  public HighlightAssociateCommand (String name, DrawingEditor editor){
    super(name, editor);
    this.editor = editor;
  }

  /**
   * @return <code>true</code>, if the current drawing is a
   *         <code>CPNDrawing</code>, there are exactly two
   *         selected figures and one of them is a
   *         {@link FigureWithHighlight}.
   **/
  public boolean isExecutable() {
    Drawing drawing = editor.drawing();
    DrawingView view = editor.view();
    if (drawing instanceof CPNDrawing
      && view.selectionCount() == 2) {
      FigureEnumeration sel = view.selectionElements();
      while (sel.hasMoreElements()) {
	if (sel.nextFigure() instanceof FigureWithHighlight) {
	  return true;
	}
      } 
    }
    return false;
  }
  
  public boolean executeUndoable() {
    if (isExecutable()) {
      // The isExecutable() assertions are neccessary.
      // Otherwise, there would be errors in the following
      // code...
      CPNDrawing drawing = (CPNDrawing)editor.drawing();
      DrawingView view = editor.view();
      Vector vfig=view.selection();
      int pos=-1;
      if (vfig.elementAt(0) instanceof FigureWithHighlight) {
	pos=0;
      } else if (vfig.elementAt(1) instanceof FigureWithHighlight) {
	pos=1;
      }
      FigureWithHighlight nf = (FigureWithHighlight)vfig.elementAt(pos);
      Figure fig = (Figure)vfig.elementAt(1-pos);
      drawing.setHighlightFigure(nf,fig);
      editor.showStatus("Highlight associated!");
      return true;
    } 
    editor.showStatus("Select a node (place or transition) and another figure!");
    return false;
  }

}
