package de.renew.gui;

import CH.ifa.draw.util.Command;
import CH.ifa.draw.framework.DrawingEditor;
import CH.ifa.draw.framework.Drawing;
import CH.ifa.draw.framework.DrawingView;
import CH.ifa.draw.framework.FigureEnumeration;
import CH.ifa.draw.framework.Figure;
import java.util.Vector;

/**
 * Selects the highlight figures for all selected 
 * highlightable figures.
 *
 * <p></p>
 * HighlightSelectCommand.java
 * Created: Mon Feb 26  2001
 * (Code moved from CPNApplication)
 *
 * @author Frank Wienberg, Michael Duvigneau
 */

public class HighlightSelectCommand extends Command {

  private DrawingEditor editor;

  public HighlightSelectCommand (String name, DrawingEditor editor){
    super(name);
    this.editor = editor;
  }

  /**
   * @return <code>true</code>, if the current drawing is a
   *         <code>CPNDrawing</code> and there is at least
   *         one figure selected. If there is only one
   *         selected figure, it is checked to be a
   *         {@link FigureWithHighlight}.
   **/
  public boolean isExecutable() {
    Drawing drawing = editor.drawing();
    DrawingView view = editor.view();
    if (drawing instanceof CPNDrawing) {
      if (view.selectionCount() == 1) {
	return (view.selectionElements().nextFigure() 
		instanceof FigureWithHighlight);
      } else if (view.selectionCount() > 1) {
	return true;
      }
    }
    return false;
  }

  public void execute() {
    if (isExecutable()) {
      DrawingView view = editor.view();
      FigureEnumeration figenum=view.selectionElements();
      Vector highlights=new Vector();
      while (figenum.hasMoreElements()) {
	Figure fig=figenum.nextFigure();
	if (fig instanceof FigureWithHighlight) {
	  Figure hilight = ((FigureWithHighlight)fig)
	    .getHighlightFigure();
	  if (hilight!=null) {
	    highlights.addElement(hilight);
	  }
	}
      }

      if (highlights.isEmpty()) {
	editor.showStatus("Select some nodes "
		  +"(places or transitions) with highlight figures.");
      } else {
	view.clearSelection();
	view.addToSelectionAll(highlights);
	// Redraw the newly selected elements.
	view.repairDamage();
	editor.showStatus("Highlight(s) selected!");
      }
    }    
  }

}
