package de.renew.gui;

import CH.ifa.draw.framework.UndoableCommand;
import CH.ifa.draw.framework.DrawingEditor;
import CH.ifa.draw.framework.Drawing;
import CH.ifa.draw.framework.DrawingView;

/**
 * Breaks up the association from a highlightable figure to
 * its highlight figure.
 *
 * <p></p>
 * HighlightUnassociateCommand.java
 * Created: Mon Feb 26  2001
 * (Code moved from CPNApplication)
 *
 * @author Frank Wienberg, Michael Duvigneau
 */

public class HighlightUnassociateCommand extends UndoableCommand {

  private DrawingEditor editor;

  public HighlightUnassociateCommand (String name, DrawingEditor editor){
    super(name, editor);
    this.editor = editor;
  }

  /**
   * @return <code>true</code>, if the current drawing is a
   *         <code>CPNDrawing</code>, there is exactly one
   *         selected figure and it's a {@link FigureWithHighlight}.
   **/
  public boolean isExecutable() {
    Drawing drawing = editor.drawing();
    DrawingView view = editor.view();
    if (drawing instanceof CPNDrawing
      && view.selectionCount() == 1) {
      if (view.selectionElements().nextFigure() instanceof FigureWithHighlight) {
	  return true;
      }
    }
    return false;
  }
  
  public boolean executeUndoable() {
    if (isExecutable()) {
      // The isExecutable() assertions are neccessary.
      // Otherwise, there would be errors in the following
      // code...
      CPNDrawing drawing = (CPNDrawing)editor.drawing();
      DrawingView view = editor.view();
      FigureWithHighlight fig =
	(FigureWithHighlight)view.selectionElements().nextFigure();
      drawing.setHighlightFigure(fig,null);
      editor.showStatus("Highlight unassociated!");
      return true;
    } 
    editor.showStatus("Select a single node (place or transition)!");
    return false;
  }

}
