/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

package de.renew.gui;

import CH.ifa.draw.figures.*;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.standard.*;
import CH.ifa.draw.util.Palette;
import de.renew.formalism.java.*;
import de.renew.remote.*;
import de.renew.shadow.*;
import de.renew.simulator.NetInstance;
import de.renew.unify.Aggregate;
import de.renew.util.TextToken;
import java.awt.*;
import java.awt.image.ImageProducer;
import java.net.*;
import java.rmi.RemoteException;
import java.util.*;

/**
 * Class documentation.
 */
public class JavaMode implements RenewMode
{
	/**
	 * The client representative of the simulation.
	 */
	transient CPNSimulation fSimulation = null;

	/**
	 * The loader to get drawings.
	 */
	transient CPNDrawingLoader fDrawingLoader = null;

	/**
	 * Method documentation.
	 * @return
	 */
	public ShadowCompiler getCompiler()
	{
		return new JavaNetCompiler(false, false, false);
	}

	/**
	 * Sets a CPNDrawingLoader to be used for the simulation
	 * creation and figure mappings.
	 * If the mode already knows a loader, the request is ignored.
	 *
	 * @since Renew 1.6
	 **/
	public void setDrawingLoader(CPNDrawingLoader loader) {
		if (fDrawingLoader == null) {
			fDrawingLoader = loader;
		}
	}

	/**
	 * Returns the drawing loader, which also keeps track of all
	 * loaded drawings.
	 *
	 * @since Renew 1.6
	 **/
	public CPNDrawingLoader getDrawingLoader() {
		if (fDrawingLoader == null) {
			throw new NullPointerException("JavaMode: Drawing loader has not been set yet.");
		}
		return fDrawingLoader;
	}

	/**
	 * Returns the CPNSimulation instance for the mode.
	 * But the drawing loader had to be set before...
	 **/
	public CPNSimulation getSimulation()
	{
		if (fSimulation == null) {
			fSimulation = new CPNSimulation(getCompiler(), false, getDrawingLoader());
		}
		return fSimulation;
	}

	/**
	 * Method documentation.
	 * @param figure
	 * @return
	 */
	public CPNTextFigure newInscriptionFigure(InscribableFigure figure)
	{
		return new CPNTextFigure();
	}

	/**
	 * Method documentation.
	 * @param figure
	 * @return
	 */
	public String getDefaultInscription(InscribableFigure figure)
	{
		if (figure instanceof ArcConnection)
		{
			return "x";
		}
		if (figure instanceof PlaceFigure)
		{
			return "[]";
		}
		if (figure instanceof TransitionFigure)
		{
			return ":s()";
		}
		return "";
	}

	/**
	 * Method documentation.
	 * @param token
	 * @param expanded
	 * @return
	 */
	public Figure getTokenFigure(Object token, boolean expanded)
	{
		if (token instanceof Token)
		{
			token = ((Token) token).getTokenRepresentation(this, expanded);

			// URLs are only handled specially in the context
			// of Token tokens.
			if (token instanceof URL)
			{
				try
				{
					token = ((URL) token).getContent();
				}
				catch (java.io.IOException e)
				{
					token = "UNLOADABLE IMAGE";
				}
			}

			// Images are only handled specially in the context
			// of Token tokens.
			if (token instanceof ImageProducer)
			{
				token = Toolkit.getDefaultToolkit().createImage((ImageProducer) token);
			}
			if (token instanceof Image)
			{
				Image image = (Image) token;
				ImageFigure figure = new ImageFigure(image, null, new Point());

				figure.displayBox(new Point(),
								  new Point(image.getWidth(null), image.getHeight(null)));
				return figure;
			}

			// If a figure is returned
			if (token instanceof Figure)
			{
				return (Figure) token;
			}

			// No image. Fall back to the ordinary visualization.
		}
		if (token instanceof Aggregate)
		{
			return new AggregateFigure(this, (Aggregate) token, expanded);
		}
		if (token instanceof NetInstance)
		{
			NetInstance instance = (NetInstance) token;
			CPNDrawing drawing = null;
			Figure iconFigure = null;

			if (fDrawingLoader != null) {
				drawing = fDrawingLoader.getDrawing(instance.getNet().toString());
			}
			if (drawing != null) {
				iconFigure = drawing.getIconFigure();
			}
			if (iconFigure != null) {
				iconFigure = (Figure) iconFigure.clone();
				updateID(iconFigure, instance);
				return iconFigure;
			}
		}
		if (expanded
				&&!(token instanceof de.uni_hamburg.fs.FeatureStructure
					|| token instanceof de.renew.simulator.NetInstance
					|| token instanceof de.renew.util.Value || token instanceof String))
		{
			return new de.renew.gui.fs.FSFigure(token);
		}
		else
		{
			return new TextFigure(objectToString(token), true);
		}
	}

	/**
	 * Method documentation.
	 * @param token
	 * @return
	 */
	public static String objectToString(Object token)
	{
		StringBuffer output = new StringBuffer();

		if (token instanceof String)
		{
			output.append('"').append((String) token).append('"');
		}
		else if (token instanceof TextToken)
		{
			output.append(((TextToken) token).toTokenText());
		}
		else
		{
			output.append(token);
		}
		return output.toString();
	}

	/**
	 * Method documentation.
	 * @param figure
	 * @param instance
	 */
	private void updateID(Figure figure, NetInstance instance)
	{
		if (figure instanceof TextHolder)
		{
			TextHolder textFig = (TextHolder) figure;

			textFig.setReadOnly(true);
			try
			{
				textFig.setText(CPNInstanceDrawing.expandMacro(textFig.getText(),
					new NetInstanceAccessorImpl(instance)));
			}
			catch (RemoteException e)
			{
				textFig.setText(e.toString());
			}
		}
		else if (figure instanceof CompositeFigure)
		{

			// search recursively:
			FigureEnumeration figenum = figure.figures();

			while (figenum.hasMoreElements())
			{
				updateID(figenum.nextFigure(), instance);
			}
		}
	}


	/**
	 * Method documentation.
	 * @param editor
	 * @param toolPanel
	 * @param palette
	 */
	public void createAdditionalTools(CPNApplication editor, Panel toolPanel, Palette palette)
	{
	}

	/**
	 * Method documentation.
	 * @param editor
	 * @param menu
	 */
	public void createAdditionalMenus(CPNApplication editor, MenuBar menu)
	{
	}

	/**
	 * Method documentation.
	 * @param editor
	 * @return
	 */
	public Tool createPlaceTool(DrawingEditor editor)
	{

		// Delegate back to drawing editor.
		return null;
	}

	/**
	 * Method documentation.
	 * @param editor
	 * @return
	 */
	public Tool createTransitionTool(DrawingEditor editor)
	{

		// Delegate back to drawing editor.
		return null;
	}

	/**
	 * Method documentation.
	 * @return
	 */
	public boolean wantClock()
	{
		return false;
	}

	/**
	 * Method documentation.
	 * @return
	 */
	public boolean wantAux()
	{
		return false;
	}

	/**
	 * Method documentation.
	 * @return
	 */
	public String getAuxFigureName()
	{
		return null;
	}

	/**
	 * Method documentation.
	 * @return
	 */
	public String getAuxToolName()
	{
		return null;
	}

}
