package de.renew.gui;

import CH.ifa.draw.util.Command;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.figures.*;
import CH.ifa.draw.standard.*;
import java.util.*;
import java.awt.*;

public class LayoutCheckCommand extends Command {
    private CPNApplication application;
    private static final double OVERLAPWARNINGTHRESHOLD = 0.5;

    public LayoutCheckCommand(String name, CPNApplication application) {
	super(name);
	this.application = application;
    }

    public boolean isExecutable() {
	return true;
    }

    public synchronized void execute() {
	Enumeration drawings=application.drawings();
	while (drawings.hasMoreElements()) {
	    Drawing drawing=(Drawing)drawings.nextElement();
	    checkSingleDrawing(drawing);
	}
    }
    


    /**
     *  Computes the relative overlap factor of two rectangles
     *
     * @author Michael Koehler, Heiko Roelke
     *
     */
    private double overlap(Rectangle r1, Rectangle r2) {
	Rectangle intersection=r1.intersection(r2);
	if (intersection.isEmpty()) {
	    return 0.0;
	} else {
	    double intersectionArea = intersection.height * intersection.width;
	    double r1Area = r1.height * r1.width;
	    double r2Area = r2.height * r2.width; 
	    return Math.max(intersectionArea / r1Area, intersectionArea / r2Area);
	} 
    }

    /**
     *  Checks whether a @see CH.ifa.draw.figures.TextFigure is hidden
     *  by an overlapping TextFigure  
     *
     * @author Michael Koehler, Heiko Roelke
     * 
     *
     */
    private void checkSingleDrawing(Drawing drawing) {
	Vector allTextFigures=new Vector();
	FigureEnumeration figures=drawing.figures();
	boolean overlapFound=false;
	
	while (figures.hasMoreElements() && !overlapFound) {
	    Figure figure=figures.nextFigure();
	    if (figure instanceof TextFigure) {
		Rectangle figureRectangle=figure.displayBox();
		Enumeration knownTextFigures=allTextFigures.elements();
		while (knownTextFigures.hasMoreElements() && 
		       !overlapFound) {
		    TextFigure compareTextFigure=(TextFigure)knownTextFigures.nextElement();
		    Rectangle compareRectangle=compareTextFigure.displayBox();

		    overlapFound = (overlap(figureRectangle, compareRectangle) > OVERLAPWARNINGTHRESHOLD);
		    if (overlapFound) {
			application.processFigureException(new FigureException
			    ("Warning","Overlapping figures detected",drawing,figure),true);
		    }

		}
		if (!overlapFound) {
		    allTextFigures.addElement(figure);
		}
	    }
	}
	if (!overlapFound) {
	    application.showStatus("Layout check successful");
	}
    }
}


