package de.renew.gui;

import java.awt.*;
import java.awt.event.*;
import de.renew.util.*;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.util.*;

public class LayoutFrame implements AdjustmentListener, Animatable {
    private static int width=200;
    private static String[] buttonTexts={
        "Spring Length",
	"Spring Strength",
	"Repulsion Distance",
	"Repulsion Strength",
	"Torque Strength",
	"Friction"
    };

    private DrawingEditor editor;
    private CPNDrawing drawing;
    private Animator animator;
    private GraphLayout graphLayout = null;
    private Frame frame;
    private Scrollbar[] scroller;
  
    public LayoutFrame(DrawingEditor editor) {
        this.editor=editor;

	frame=new Frame("Automatic Net Layout");
	if (! GUIProperties.avoidFrameReshape()) {
	  frame.setSize(600,200); 
	} 
	GridBagLayout gridBag=new GridBagLayout();
	frame.setLayout(gridBag);

	GridBagConstraints c=new GridBagConstraints();
        c.fill=GridBagConstraints.HORIZONTAL;
	c.weighty=1;

	scroller=new Scrollbar[buttonTexts.length];
	for (int i=0;i<buttonTexts.length;i++) {
	    c.gridwidth=1;
	    c.weightx=0;
	    Label label=new Label(buttonTexts[i]);
	    gridBag.setConstraints(label,c);
	    frame.add(label);

	    c.gridwidth = GridBagConstraints.REMAINDER;
	    c.weightx=1;
	    scroller[i]=new Scrollbar(Scrollbar.HORIZONTAL,width/2,10,0,width);
	    scroller[i].addAdjustmentListener(this);
	    gridBag.setConstraints(scroller[i],c);
	    frame.add(scroller[i]);
	}

	Panel panel=new Panel();
	gridBag.setConstraints(panel,c);
	frame.add(panel);

	gridBag=new GridBagLayout();
	panel.setLayout(gridBag);
	c=new GridBagConstraints();
	c.weightx=1;
        c.fill=GridBagConstraints.BOTH;
	c.gridwidth=1;

	Button button;
	button=new Button("Start");
	button.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    startAnimation();
		}
	    });
	gridBag.setConstraints(button,c);
	panel.add(button);

	button=new Button("Stop");
	button.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    stopAnimation();
		}
	    });
	gridBag.setConstraints(button,c);
	panel.add(button);

	button=new Button("Close");
	button.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    stopAnimation();
		    frame.setVisible(false);
		}
	    });
	c.gridwidth = GridBagConstraints.REMAINDER;
	gridBag.setConstraints(button,c);
	panel.add(button);

	frame.addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent w) {
		    stopAnimation();
		    frame.setVisible(false);
		}
	    });
	
	frame.pack();
    }

    public Dimension getPreferredSize() {
	return new Dimension(600,200);
    }

    public Dimension getMinimumSize() {
	return getPreferredSize();
    }

    public void stopAnimation() {
	if (animator!=null) {
	    animator.end();
	    animator = null;
	}
    }

    public void startAnimation() {
	stopAnimation();

        DrawingView view = editor.view();
        Drawing drawing = view.drawing();
	if (drawing instanceof CPNDrawing) {
            // Setup the graphics.
	    frame.setTitle("Automatic Net Layout - "+drawing.getName());
	    frame.setVisible(true);
	    //Take Undo-Snapshot.
	    editor.prepareUndoSnapshot();
	    editor.commitUndoSnapshot();
	    // Setup the layout algorithm.
	    graphLayout = new GraphLayout();
	    ((CPNDrawing)drawing).fillInGraph(graphLayout);
	    fillInScrollerValues();
	    // Make sure the layout is relaxed periodically.
	    animator=new Animator(this, view);
	    animator.start();
	}
    }

    private static double calcFactor(int value) {
	double n=value/(double)width;
	return n*n;
    }

    public void adjustmentValueChanged(AdjustmentEvent e) {
      fillInScrollerValues();
    }

    private void fillInScrollerValues() {
	int num=0;
	if (graphLayout!=null) {
	    graphLayout.LENGTH_FACTOR=1.0*4*
		calcFactor(scroller[num++].getValue());
	    graphLayout.SPRING_STRENGTH=0.1*4*
		calcFactor(scroller[num++].getValue());
	    graphLayout.REPULSION_LIMIT=200.0*4*
		calcFactor(scroller[num++].getValue());
	    graphLayout.REPULSION_STRENGTH=0.5*4*
		calcFactor(scroller[num++].getValue());
	    graphLayout.TORQUE_STRENGTH=0.25*4*
		calcFactor(scroller[num++].getValue());
	    graphLayout.FRICTION_FACTOR=
	        1-calcFactor(scroller[num++].getValue());
	}
    }

    synchronized public void animationStart() {
      // Nothing to do.
    }

    synchronized public void animationStop() {
      graphLayout.remove();
      graphLayout = null;
    }

    synchronized public void animationStep() {
      if (graphLayout != null) {
	graphLayout.relax();
      }
    }

}
