/*
 * @(#)PlaceFigure.java 5.1
 *
 */

package de.renew.gui;

import java.awt.*;
import java.util.*;
import java.io.*;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.standard.*;
import CH.ifa.draw.figures.*;
import CH.ifa.draw.util.*;
import de.renew.simulator.*;
import de.renew.shadow.*;

public class PlaceFigure extends EllipseFigure
    implements PlaceNodeFigure,
	       FigureWithHighlight,
	       InscribableFigure {
    /*
     * Serialization support.
     */
    private static final long serialVersionUID = -7878886241236946512L;
    private int placeFigureSerializedDataVersion = 1;

    public static final int HIGHLIGHT = 0;
    public static final int CARDINALITY = 1;
    public static final int TOKENS = 2;
    public static final int EXPANDED_TOKENS = 3;

    /**
     * This figure will be highlighted in a instance
     * drawing in the same manner as the place figure
     * will be highlighted. May be <code>null</code>.
     * @serial
     **/
    private Figure hilightFig=null;

    public PlaceFigure() {
        super();
    }

    public Vector handles() {
        Vector handles = super.handles();
        handles.addElement(new ArcConnectionHandle(this));
	return handles;
    }

  public boolean getTraceMode() {
    Object value=getAttribute("TraceMode");
    if (value instanceof Boolean) {
      return ((Boolean)value).booleanValue();
    }
    return true;
  }

  public int getMarkingAppearance() {
      Object value=getAttribute("MarkingAppearance");
      if (value instanceof Integer) {
	return ((Integer)value).intValue();
      }
      return CARDINALITY;
  }

  /**
   * The shadow of this place figure.
   * Initially <code>null</code>, will be created
   * when needed.
   * <p>
   * This field is transient because its information
   * can be regenerated via <code>buildShadow(...)</code>.
   * </p>
   **/
  private transient ShadowPlace shadow = null;

  public void release() {
    setHighlightFigure(null);
    super.release();
    if (shadow!=null)
      shadow.discard();
  }  

  public ShadowNetElement buildShadow(ShadowNet net) {
      if (shadow!=null)
         shadow.discard();
      shadow = new ShadowPlace(net);
      shadow.context = this;
      shadow.setID(this.getID());
      shadow.setTrace(getTraceMode());
      //System.err.println("place shadow for "+this+" created!");
      return shadow;
  }

  public ShadowNetElement getShadow() {
    return shadow;
  }

  public static Dimension defaultDimension() {
    return new Dimension(20,20);
  }

  public void setHighlightFigure(Figure fig) {
      hilightFig=fig;
  }

  public Figure getHighlightFigure() {
      return hilightFig;
  }

  // This method returns this palce figure. It is overridden
  // by virtual place figures, which want to represent another
  // figure ultimately.
  public PlaceFigure getSemanticPlaceFigure() {
     return this;
  }

  /**
   * Returns all figures with dependencies of the superclass
   * plus an optional hilight figure.
   **/
  public FigureEnumeration getFiguresWithDependencies() {
      FigureEnumeration superDep = super.getFiguresWithDependencies();
      Vector myDep = new Vector(1);
      myDep.addElement(getHighlightFigure());
      return new MergedFigureEnumerator(superDep, 
					new FigureEnumerator(myDep));
  }

  /**
   * Stores the Figure to a StorableOutput.
   */
  public void write(StorableOutput dw) {
      super.write(dw);
      dw.writeStorable(hilightFig);
  }

  /**
   * Reads the Figure from a StorableInput.
   */
  public void read(StorableInput dr) throws IOException {
      super.read(dr);
      if (dr.getVersion()>=3) {
	  setHighlightFigure((Figure)dr.readStorable());
      }
  }

}
