/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

package de.renew.gui;

import java.awt.*;
import java.awt.event.*;
import java.rmi.RemoteException;
import java.util.*;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.standard.*;
import CH.ifa.draw.util.ColorMap;
import de.renew.remote.*;
import de.renew.simulator.*;
import de.renew.event.*;

/**
 * Class documentation.
 */
class PlaceInstanceFigure extends AbstractFigure implements InstanceFigure,
	de.renew.remote.EventListener, FigureChangeListener
{
	private PlaceFigure placeFigure;
	private PlaceInstanceAccessor placeInstance;
	private RemoteEventForwarder forwarder;
	private boolean isMarked;
	private int markingAppearance;
	private CPNInstanceDrawing drawing;

	/**
	 * Creates the place instance figure.
	 * @param drawing The CPN instance drawing to create for.
	 * @param tf The corresponding place figure.
	 * @param netElement A lookup mapping net element group ids to net elements.
	 */
	public PlaceInstanceFigure(CPNInstanceDrawing drawing, PlaceFigure pf,
		Hashtable netElements)
	{
		this.drawing = drawing;
		placeFigure = pf;
		initialize(netElements);
	}

	/**
	 * Initializes the figure with the net element lookup.
	 * @param netElements A lookup mapping net element group ids to net elements.
	 */
	protected void initialize(Hashtable netElements)
	{
		if (netElements.size() != 1)
		{
			System.err.println("Error: There is not exactly one place instance for the figure " + this);
			return;
		}

		placeInstance = (PlaceInstanceAccessor) netElements.elements().nextElement();

		placeFigure.addFigureChangeListener(this);
		markingAppearance = placeFigure.getMarkingAppearance();
		update();
		if (markingAppearance != PlaceFigure.HIGHLIGHT)
		{
			addTokenBagFigure();
		}

		try
		{
			forwarder = new RemoteEventForwarder(this);
			placeInstance.addRemoteEventListener(forwarder);
		}
		catch (RemoteException e)
		{
			e.printStackTrace(System.err);
		}
	}

	/**
	 * Method documentation.
	 * @return
	 */
	RenewMode getMode()
	{
		return drawing.getMode();
	}

	/**
	 * Method documentation.
	 * @return
	 */
	protected PlaceInstanceAccessor getInstance()
	{
		return placeInstance;
	}

	/**
	 * Method documentation.
	 * @return
	 */
	public boolean isHighlighted()
	{
		return isMarked;
	}

	// Allow access to drawing, so that token bag figures
	// may lock the drawing.
	/**
	 * Method documentation.
	 * @return
	 */
	CPNInstanceDrawing drawing()
	{
		return drawing;
	}

	/**
	 * Method documentation.
	 * @return
	 */
	public boolean wantSynchronousNotification()
	{
		return false;
	}

	/**
	 * Receives the de.renew.remote.EventLister events.
	 */
	public void update()
	{

		// I have dropped the plausibilty check that the
		// event must originate from the referenced place instance,
		// because that can lead to racing conditions during the
		// release phase, when the place instance is nulled.

		CPNInstanceDrawing theDrawing = drawing;

		if (theDrawing == null)
		{

			// The figure has already been released.
			return;
		}

		try
		{
			theDrawing.lock();
			if (drawing == null)
			{

				// The figure was just now concurrently released.
				return;
			}

			boolean newIsMarked;
			try
			{
				TokenCountsAccessor tokenCountsAccessor = placeInstance.getTokenCounts();
				newIsMarked = !tokenCountsAccessor.isEmpty();
			}
			catch (RemoteException e)
			{
				e.printStackTrace(System.err);
				newIsMarked = false;
			}

			if (newIsMarked != isMarked)
			{
				isMarked = newIsMarked;
				invalidate();
				Figure hilight = placeFigure.getHighlightFigure();

				if (hilight != null)
				{
					Rectangle area = hilight.displayBox();

					area.grow(5, 5);
					DrawingChangeEvent dce = new DrawingChangeEvent(drawing, area);

					drawing.drawingInvalidated(dce);
				}
				if (listener() != null)
				{
					listener().figureRequestUpdate(new FigureChangeEvent(this));
				}
			}
		}
		finally
		{
			theDrawing.unlock();
		}
	}

	/**
	 * Method documentation.
	 * @param origin
	 * @param corner
	 */
	public void basicDisplayBox(Point origin, Point corner)
	{

		// do nothing, as the DisplayBox of the Place is used!
	}

	/**
	 * Method documentation.
	 * @return
	 */
	public Vector handles()
	{
		Vector handles = new Vector();

		handles.addElement(new NullHandle(this, RelativeLocator.northWest()));
		handles.addElement(new NullHandle(this, RelativeLocator.northEast()));
		handles.addElement(new NullHandle(this, RelativeLocator.southWest()));
		handles.addElement(new NullHandle(this, RelativeLocator.southEast()));
		return handles;
	}

	/**
	 * Method documentation.
	 * @return
	 */
	public Rectangle displayBox()
	{
		return placeFigure.displayBox();
	}

	/**
	 * Method documentation.
	 * @param x
	 * @param y
	 */
	protected void basicMoveBy(int x, int y)
	{

		// do nothing, as the DisplayBox of the Place is used!
	}

	/**
	 * Method documentation.
	 * @param g
	 */
	public void draw(Graphics g)
	{

		// don't draw anything!
	}


	/**
	 * Method documentation.
	 * @return
	 */
	private TokenBagFigure addTokenBagFigure()
	{
		TokenBagFigure tbf = new TokenBagFigure(getMode(), drawing(), this, placeInstance,
												markingAppearance);

		drawing.add(tbf);
		return tbf;
	}

	/**
	 * Method documentation.
	 * @param view
	 * @param alternate
	 * @return
	 */
	public boolean inspect(DrawingView view, boolean alternate)
	{
		if (alternate)
		{
			try
			{
				((CPNApplication) view.editor()).openTokenBagDrawing(placeInstance);
				return true;
			}
			catch (RemoteException e)
			{
				return false;
			}
		}
		else
		{
			if (getTokenBagFigure() == null)
			{
				view.clearSelection();
				view.addToSelection(addTokenBagFigure());
				return true;
			}
			else
			{
				return super.inspect(view, alternate);
			}
		}
	}

	/**
	 * Method documentation.
	 * @param attribute
	 * @param value
	 */
	public void setAttribute(String attribute, Object value)
	{
		if ("MarkingAppearance".equals(attribute))
		{
			markingAppearance = ((Integer) value).intValue();
			TokenBagFigure etbf = getTokenBagFigure();

			if (markingAppearance == PlaceFigure.HIGHLIGHT)
			{
				if (etbf != null)
				{
					drawing.remove(etbf);
				}
			}
			else
			{
				if (etbf == null)
				{
					addTokenBagFigure();
				}
				else
				{
					etbf.setMarkingAppearance(markingAppearance);
				}
			}
		}
		else
		{
			super.setAttribute(attribute, value);
		}
	}

	/**
	 * Method documentation.
	 */
	public void release()
	{
		CPNInstanceDrawing theDrawing = drawing;

		if (theDrawing == null)
		{

			// This figure is released a second time.
			return;
		}

		try
		{
			theDrawing.lock();
			if (drawing == null)
			{

				// This figure has been released concurrently.
				return;
			}

			if (placeInstance != null)
			{
				try
				{
					placeInstance.removeRemoteEventListener(forwarder);
				}
				catch (RemoteException e)
				{
					e.printStackTrace(System.err);
				}

				// OK: Release reference to place instance so that it
				// can be garbage collected.
				placeInstance = null;
				drawing = null;

				// etbf=null;
				// placeFigure=null;
			}
			placeFigure.removeFigureChangeListener(this);
			super.release();
		}
		finally
		{
			theDrawing.unlock();
		}
	}

	/**
	 * Method documentation.
	 * @return
	 */
	private TokenBagFigure getTokenBagFigure()
	{
		FigureEnumeration childenum = children();

		if (childenum.hasMoreElements())
		{
			return (TokenBagFigure) childenum.nextElement();
		}
		else
		{
			return null;
		}
	}

	/**
	 * Method documentation.
	 * @param e
	 */
	public void figureInvalidated(FigureChangeEvent e)
	{
	}

	/**
	 * Method documentation.
	 * @param e
	 */
	public void figureChanged(FigureChangeEvent e)
	{

		// If my place changes, I change!
		changed();
	}

	/**
	 * Method documentation.
	 * @param e
	 */
	public void figureHandlesChanged(FigureChangeEvent e)
	{
	}

	/**
	 * Method documentation.
	 * @param e
	 */
	public void figureRemoved(FigureChangeEvent e)
	{

		// My place figure has been removed -- that's bad!!!
	}

	/**
	 * Method documentation.
	 * @param e
	 */
	public void figureRequestRemove(FigureChangeEvent e)
	{
	}

	/**
	 * Method documentation.
	 * @param e
	 */
	public void figureRequestUpdate(FigureChangeEvent e)
	{
	}


	/**
	 * This class is not serializable, although inheriting
	 * serializability from its superclass.
	 * <p>
	 * Reason: Serialization is currently used for Copy&Paste
	 * and the Undo machanism only. As such modifications don't
	 * make sense for instance drawings, this  figure, which is
	 * a part of such drawings, doesn't need to be serializable.
	 * </p><p>
	 * Second reason: Some fields of this class would lead to
	 * a serialization of the compiled net, which is not desirable.
	 * </p>
	 * @throws java.io.NotSerializableException always.
	 */
	private void writeObject(java.io.ObjectOutputStream out) throws java.io.IOException
	{
		throw new java.io.NotSerializableException("PlaceInstanceFigure is not serializable!");
	}

}
