package de.renew.gui;

import CH.ifa.draw.framework.Drawing;
import CH.ifa.draw.util.StorableInput;
import de.renew.shadow.ShadowCompiler;
import de.renew.shadow.ShadowNetSystem;
import de.renew.util.StringUtil;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectOutput;
import java.io.ObjectOutputStream;
import java.util.Iterator;
import java.util.Vector;

/**
 * Converts Renew drawings (<code>.rnw</code>) to shadow nets
 * (<code>.sns</code>).
 * <p>
 * </p>
 * ShadowTranslator.java
 * Created: Mon Jan 21  2002
 * @author Michael Duvigneau
 **/
public class ShadowTranslator {

	/**
	 * Execution entry point from the command line.
	 **/
	public static void main(String[] args) {
		System.setProperty("de.renew.noGraphics","true");

		if (args.length == 0) {
			System.err.println("Usage: java de.renew.gui.ShadowTranslator file.rnw ...");
			System.err.println("Converts Renew drawings (.rnw) to shadow nets (.sns).");
			System.err.println("No syntax check or compilation is done, just conversion.");
			System.exit(1);
		}
		
		Vector names = new Vector(args.length);
		for (int i = 0; i < args.length; ++i) {
			names.add(args[i]);
		}
		Iterator drawings = readDrawings(names.iterator());
		writeShadows(drawings);
		System.exit(0);
	}

	public static Iterator readDrawings(Iterator files) {
		Vector drawings = new Vector();
		while (files.hasNext()) {
			try {
				Drawing drawing = loadDrawing((String)files.next());
				if (drawing == null) {
					// Probably there already has been an error message...
				} else if (drawing instanceof CPNDrawing) {
					drawings.add(drawing);
				} else {
					System.out.println("  Not a CPN drawing - ignored.");
				}
			} catch (Exception e) {
				System.out.println(e);
			}
		}
		return drawings.iterator();
	}


	public static void writeShadows(Iterator drawings) {
		ShadowCompiler compiler = null;
		int count = 0;

		// Build new shadows of all nets, one net per net system.
		while (drawings.hasNext()) {
			CPNDrawing currentDrawing = (CPNDrawing) drawings.next();
			ShadowNetSystem netSystem = new ShadowNetSystem(compiler);
			currentDrawing.buildShadow(netSystem);

			String name = currentDrawing.getName();
			String path = currentDrawing.getFilename();
			if (path == null) {
				System.out.println("Skipping net "+name+" (no filename).");
			} else {
				try {
					path = StringUtil.makeCanonical(path);
					path = StringUtil.getPath(path) + File.separator + name + ".sns";
					System.out.print("Exporting net "+name+" to " + path + " ... ");
					
					FileOutputStream stream = new FileOutputStream(path);
					ObjectOutput output = new ObjectOutputStream(stream);
					output.writeObject(netSystem);
					output.close();
					System.out.println("OK");
					count++;
				} catch (Exception e) {
					System.out.println(e);
				}
			}
			currentDrawing.discardShadow();
		}
		System.out.println("Exported "+count+" shadow net"+(count==1?"":"s")+".");
	}

	// ------------------------------

	private static Drawing loadDrawing(String pathname) {
		pathname = checkAndAddExtension(pathname);
		pathname = StringUtil.makeCanonical(pathname);
		Drawing drawing = null;
		System.out.println("Loading drawing from "+pathname+"...");
		drawing = readFromStorableInput(pathname);
		if (drawing != null) {
			drawing.setName(StringUtil.getFilename(pathname));
			drawing.setFilename(pathname);
		}
		return drawing;
	}

	/**
	 * Checks if the given path ends with one of the default
	 * extensions and adds one, if this was not the case.
	 **/
	private static String checkAndAddExtension(String path) {
		String[] defaultExtensions = new String[]{"rnw"};
		boolean hasExtension = false;
		String ext=StringUtil.getExtension(path);
		for (int i = 0; i < defaultExtensions.length; i++)
			if (defaultExtensions[i].equals(ext))
				hasExtension = true;
		String result = path;
		if ((defaultExtensions.length > 0) && !hasExtension)
			result += "."+defaultExtensions[0];
		return result;
	}

	private static StorableInput makeStorableInput(String file, boolean useUFT)
		throws FileNotFoundException {
	  return new StorableInput(file,useUFT);
	}

	private static Drawing readFromStorableInput(String filename) {
		Drawing drawing = null;
		try {
			StorableInput input = makeStorableInput(filename,true);
			try {
			  drawing = readStorableDrawing(input);
			} catch (sun.io.MalformedInputException e) {
				input.close();
				input = makeStorableInput(filename,false);
				drawing = readStorableDrawing(input);
			}
			input.close();
		} catch (IOException e) {
			System.out.println("  Error: " + e);
		}
		return drawing;
	}

	private static Drawing readStorableDrawing(StorableInput input)
	  throws IOException
	{
	  if (input.canReadInt()) {
			input.setVersion(input.readInt());
	  } else {
			input.setVersion(0);
	  }
	  return (Drawing)input.readStorable();
	}

}
