package de.renew.gui;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.standard.*;
import CH.ifa.draw.figures.*;
import CH.ifa.draw.util.*;
import CH.ifa.draw.application.*;
import de.renew.shadow.*;


class SyntaxExceptionFrame extends FigureChangeAdapter {

  private final Frame frame;
  private final Button selectButton;
  private final TextArea textArea;
  private final CPNApplication editor;
  private FigureException e=null;

  SyntaxExceptionFrame(final CPNApplication editor) {
    this.editor=editor;
    frame=new Frame("Renew: Syntax Error");
    frame.setIconImage(Toolkit.getDefaultToolkit()
      .createImage(editor.menuFrame().getIconImage().getSource()));
    textArea=new TextArea();
    textArea.setEditable(false);
    frame.add("Center",textArea);
    Panel southPanel=new Panel(new GridLayout(1,2));
    frame.add("South",southPanel);
    Button closeButton=new Button("Close");
    closeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
	frame.dispose();
      }
    });
    selectButton=new Button("Select");
    selectButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
	if (e!=null && !editor.selectOffendingElements(e))
	  selectButton.setEnabled(false);
      }
    });
    southPanel.add("1",closeButton);
    southPanel.add("2",selectButton);
    frame.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent event) {
	dispose();
      }
    });
  }

  void displayException(FigureException e, boolean displayImmediately) {
    if (displayImmediately)
      selectButton.setEnabled(editor.selectOffendingElements(e));
    else
      selectButton.setEnabled(true);
    frame.setTitle(e.title);
    textArea.setText(e.getMessage());
    this.e=e;

    FigureEnumeration errorFigures=new FigureEnumerator(e.errorFigures);
    while (errorFigures.hasMoreElements()) {
        errorFigures.nextFigure().addFigureChangeListener(this);
    }

    frame.pack();
    frame.setVisible(true);
  }

    public void figureRemoved(FigureChangeEvent e) {
       dispose();
    }

    public void dispose() {
      if (e!=null) {
        frame.dispose();
        FigureEnumeration errorFigures=new FigureEnumerator(e.errorFigures);
        while (errorFigures.hasMoreElements()) {
            errorFigures.nextFigure().removeFigureChangeListener(this);
        }
        e=null;
      }
    }

}
