/*---	formatted by Jindent	2.1, (www.c-lab.de/~jindent) ---*/

package de.renew.gui;

import java.awt.*;
import java.rmi.RemoteException;
import java.util.*;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.standard.*;
import CH.ifa.draw.figures.*;
import CH.ifa.draw.util.*;
import de.renew.simulator.*;
import de.renew.event.*;
import de.renew.remote.*;
import collections.*;

/**
 *	Class	documentation.
 */
public class TokenBagFigure extends	SimpleCompositeFigure implements	ChildFigure,
	de.renew.remote.EventListener
{
	/**
	 * Constructor documentation.
	 */
	public TokenBagFigure()
	{
		super();
	}

	private static final int BORDER = 3;
	private static final int SEPARATOR = 1;

	private OffsetLocator fLocator = null;
	private RenewMode mode;
	private Drawing lockDrawing;
	private AbstractFigure figure;
	private PlaceInstanceAccessor placeInstance;
	private RemoteEventForwarder forwarder;
	private int cacheMult = -1;
	private int cacheTestMult = -1;
	private TextFigure cardFigure;
	private FigureAttributes attributes;
	private int markingAppearance = -1;	   // force first-time update

	/**
	 * Constructor documentation.
	 * @param mode
	 * @param lockDrawing
	 * @param figure
	 * @param pi
	 */
	public TokenBagFigure(RenewMode mode, Drawing lockDrawing, AbstractFigure figure,
						  PlaceInstanceAccessor pi)
	{
		this(mode, lockDrawing, figure, pi, PlaceFigure.CARDINALITY);
	}

	/**
	 * Constructor documentation.
	 * @param mode
	 * @param lockDrawing
	 * @param figure
	 * @param pi
	 * @param markingAppearance
	 */
	public TokenBagFigure(RenewMode mode, Drawing lockDrawing, AbstractFigure figure,
						  PlaceInstanceAccessor pi, int markingAppearance)
	{
		placeInstance = pi;
		this.mode = mode;
		this.lockDrawing = lockDrawing;
		cardFigure = new TextFigure();
		cardFigure.setReadOnly(true);
		cardFigure.setAttribute("FontStyle", new Integer(Font.ITALIC));
		attributes = new FigureAttributes();
		if (figure != null)
		{
			setParent(figure);
		}
		setMarkingAppearance(markingAppearance);

		try
		{
			forwarder = new RemoteEventForwarder(this);
			placeInstance.addRemoteEventListener(forwarder);
		}
		catch (RemoteException e)
		{
			e.printStackTrace(System.err);
		}
	}

	/**
	 * Method documentation.
	 * @return
	 */
	protected Vector basicHandles()
	{

		// public Vector handles() {
		Vector handles = super.basicHandles();

		handles.addElement(new NullHandle(this, RelativeLocator.northWest()));
		handles.addElement(new NullHandle(this, RelativeLocator.northEast()));
		handles.addElement(new NullHandle(this, RelativeLocator.southWest()));
		handles.addElement(new NullHandle(this, RelativeLocator.southEast()));
		return handles;
	}

	/*
	 * public void draw(Graphics g) {
	 * if (expanded) {
	 * g.setColor(Color.red);
	 * Rectangle box=displayBox();
	 * g.drawRect(box.x,box.y,box.width-1,box.height-1);
	 * }
	 * super.draw(g);
	 * }
	 */

	/**
	 * Method documentation.
	 * @param p
	 * @return
	 */
	public boolean canBeParent(ParentFigure p)
	{
		return (p instanceof AbstractFigure && (figure == null || figure == p));
	}

	/**
	 * Method documentation.
	 */
	private void unsetParent()
	{
		if (figure != null)
		{
			figure.removeChild(this);
			figure = null;
		}
	}

	/**
	 * Method documentation.
	 * @param figure
	 * @return
	 */
	public boolean setParent(ParentFigure figure)
	{
		if (!canBeParent(figure))
		{
			return false;
		}
		if (this.figure == figure)
		{
			return true;
		}
		unsetParent();
		this.figure = (AbstractFigure) figure;
		if (figure == null)
		{
			fLocator = null;
		}
		else
		{
			figure.addChild(this);
			if (fLocator != null)
			{
				fLocator.setBase(figure.connectedTextLocator(this));
			}
			else
			{
				fLocator = new OffsetLocator(figure.connectedTextLocator(this));
			}
			updateLocation();
		}
		return true;
	}

	/**
	 * Method documentation.
	 * @return
	 */
	public ParentFigure parent()
	{
		return figure;
	}

	/**
	 * Method documentation.
	 */
	protected void layout()
	{
		Point partOrigin = displayBox().getLocation();

		partOrigin.translate(BORDER, BORDER);
		Dimension extent = new Dimension(0, 0);

		FigureEnumeration k = figures();

		while (k.hasMoreElements())
		{
			Figure f = k.nextFigure();

			Dimension partExtent = f.size();
			Point corner = new Point(partOrigin.x + partExtent.width,
									 partOrigin.y + partExtent.height);

			f.basicDisplayBox(partOrigin, corner);

			extent.width = Math.max(extent.width, partExtent.width);
			extent.height += partExtent.height + SEPARATOR;
			partOrigin.y += partExtent.height + SEPARATOR;
		}
		fDisplayBox.width = extent.width + 2 * BORDER;
		fDisplayBox.height = extent.height - SEPARATOR + 2 * BORDER;
		updateLocation();
		super.layout();
	}

	/**
	 * Method documentation.
	 * @return
	 */
	protected boolean needsLayout()
	{
		Dimension extent = new Dimension(0, -SEPARATOR + 2 * BORDER);

		FigureEnumeration k = figures();

		while (k.hasMoreElements())
		{
			Figure f = k.nextFigure();

			extent.width = Math.max(extent.width, f.size().width);
			extent.height += f.size().height + SEPARATOR;
		}
		extent.width += 2 * BORDER;
		return !extent.equals(fDisplayBox.getSize());
	}

	/**
	 * Method documentation.
	 * @param x
	 * @param y
	 */
	public void moveBy(int x, int y)
	{
		super.moveBy(x, y);
		if (fLocator != null)
		{
			fLocator.moveBy(x, y);
		}
	}

	/**
	 * Updates the location relative to the connected figure.
	 * The Figure is centered around the located point.
	 */
	public void updateLocation()
	{
		if (fLocator != null)
		{
			Point p = fLocator.locate(figure);
			Rectangle box = displayBox();

			p.x -= box.x + box.width / 2;
			p.y -= box.y + box.height / 2;

			if (p.x != 0 || p.y != 0)
			{
				willChange();
				basicMoveBy(p.x, p.y);
				changed();
			}
		}
	}

	/**
	 * Receives the de.renew.remote.EventLister events.
	 */
	public void update()
	{
		// I have dropped the plausibilty check that the
		// event must originate from the referenced place instance,
		// because that can lead to racing conditions during the
		// discard phase, when the place instance is nulled.

		Drawing theLockDrawing = lockDrawing;

		if (theLockDrawing == null)
		{

			// This figure has already been released.
			return;
		}

		try
		{

			// Lock the drawing during the entire update.
			theLockDrawing.lock();
			if (lockDrawing == null)
			{

				// A figure was just now concurrently released.
				return;
			}

			if (markingAppearance == PlaceFigure.CARDINALITY)
			{
				updateCardFigure();
			}
			else
			{
				updateTokenFigures();
			}

			if (listener() != null)
			{
				listener().figureRequestUpdate(new FigureChangeEvent(this));

				// Is it wise to lock the drawing while doing the
				// notifications? Probably yes.
			}
		}
		finally
		{
			theLockDrawing.unlock();
		}
	}

	/**
	 * Method documentation.
	 */
	private void updateCardFigure()
	{
		try
		{
			TokenCountsAccessor tokenCounts = placeInstance.getTokenCounts();
			int newTestMult = tokenCounts.getTestedTokenCount();
			int newMult = tokenCounts.getFreeTokenCount() + newTestMult;

			if ((newMult != cacheMult) || (newTestMult != cacheTestMult))
			{
				cacheMult = newMult;
				cacheTestMult = newTestMult;
				if (newTestMult > 0)
				{
					if (newMult > 0)
					{
						cardFigure.setText(newMult + "(" + newTestMult + ")");
					}
					else
					{
						cardFigure.setText("(" + newTestMult + ")");
					}
				}
				else if (newMult > 0)
				{
					cardFigure.setText(String.valueOf(newMult));
				}
				else
				{
					cardFigure.setText(" ");
				}
			}
		}
		catch (RemoteException e)
		{
			e.printStackTrace(System.err);
		}
	}

	/**
	 * Method documentation.
	 */
	private void updateTokenFigures()
	{
		/*Vector tempTokens = new Vector();
		Enumeration tokens = placeInstance.getDistinctTestableTokens();

		while (tokens.hasMoreElements())
		{
			tempTokens.addElement(tokens.nextElement());
		}
		Object[] newTokens = new Object[tempTokens.size()];
		TimeSet[] newTimeSets = new TimeSet[newTokens.length];
		boolean[] newIsTested = new boolean[newTokens.length];

		tempTokens.copyInto(newTokens);
		for (int i = 0; i < newTokens.length; i++)
		{
			newTimeSets[i] = placeInstance.getFreeTimeSet(newTokens[i]);
			newIsTested[i] = placeInstance.containsTestedToken(newTokens[i]);
		}*/

		// Render marking information.
		removeAll();
		willChange();

		try
		{
			MarkingAccessor marking = placeInstance.getMarking();

			boolean expanded = markingAppearance == PlaceFigure.EXPANDED_TOKENS;
			int distinctTokenCount = marking.getDistinctTokenCount();
			for (int i = 0; i < distinctTokenCount; i++)
			{
				ObjectAccessor token = marking.getToken(i);
				boolean isTested = marking.getTokenTested(i);
				double[] times = marking.getTokenTimes(i);
				int[] mults = marking.getTokenTimeMultiplicities(i);

				for (int j = 0; j < times.length; j++)
				{
					double time = times[j];
					int mult = mults[j];
					boolean showTested = isTested && time == marking.getCurrentTime();

					Figure tokenFigure = new MultipleTokenFigure(mode, mult, showTested, time, token, expanded);
					Enumeration attrenum = attributes.definedAttributes();
					while (attrenum.hasMoreElements())
					{
						String attribute = (String) attrenum.nextElement();
						tokenFigure.setAttribute(attribute, attributes.get(attribute));
					}
					add(tokenFigure);
				}
			}
		}
		catch (RemoteException e)
		{
			e.printStackTrace(System.err);
		}

		layout();
		changed();
	}

	/**
	 * Method documentation.
	 */
	public void release()
	{

		// Later on, I will not even know my place,
		// let alone the drawing. So keep a note.
		Drawing theDrawing = lockDrawing;

		if (theDrawing == null)
		{

			// This figure is released a second time.
			return;
		}

		try
		{
			theDrawing.lock();
			if (lockDrawing == null)
			{

				// This figure has been released concurrently.
			}

			if (placeInstance != null)
			{
				try
				{
					placeInstance.removeRemoteEventListener(forwarder);
				}
				catch (RemoteException e)
				{
					e.printStackTrace(System.err);
				}

				// OK: Release reference to place instance so that the place
				// instance can be garbage collected.
				placeInstance = null;
			}

			// OK: Forget about the place.
			unsetParent();

			// Indicate release.
			lockDrawing = null;

			// OK: Forget about all tokens.
			removeAll();

			super.release();
		}
		finally
		{
			theDrawing.unlock();
		}
	}

	/**
	 * Method documentation.
	 * @return
	 */
	public String toString()
	{
		if (placeInstance == null)
		{
			return "Detached TokenBagFigure@" + System.identityHashCode(this);
		}

		StringBuffer output = new StringBuffer().append('{');

		try
		{
			MarkingAccessor marking = placeInstance.getMarking();

			int distinctTokenCount = marking.getDistinctTokenCount();
			for (int i=0; i<distinctTokenCount; i++)
			{
				ObjectAccessor token = (ObjectAccessor) marking.getToken(i);
				int mult = marking.getTokenFreeCount(i);
				boolean isTested = marking.getTokenTested(i);

				output.append(MultipleTokenFigure.getMultString(mult, isTested));
				output.append(token.asString());

				if (i < distinctTokenCount - 1)
				{
					output.append(", ");
				}
			}

			output.append('}');
			return output.toString();
		}
		catch (RemoteException e)
		{
			e.printStackTrace(System.err);
			return "";
		}
	}

	/**
	 * Method documentation.
	 * @return
	 */
	public boolean wantSynchronousNotification()
	{
		return false;
	}

	/**
	 * Method documentation.
	 * @param markingAppearance
	 */
	public void setMarkingAppearance(int markingAppearance)
	{
		if (this.markingAppearance != markingAppearance
				&& markingAppearance != PlaceFigure.HIGHLIGHT)
		{
			willChange();
			this.markingAppearance = markingAppearance;
			if (markingAppearance == PlaceFigure.CARDINALITY)
			{
				removeAll();
				add(cardFigure);
			}
			update();
			layout();
			changed();
		}
	}

	/**
	 * Returns whether this figure can be selected.
	 */

	// public boolean isSelectable() { return false; }

	public boolean inspect(DrawingView view, boolean alternate)
	{

		// undo support can be implemented here
		// (currently disabled for instance drawings)
		boolean done = super.inspect(view, alternate);

		if (alternate &&!done)
		{
			view.clearSelection();
			int newMA;

			if (markingAppearance == PlaceFigure.CARDINALITY)
			{
				if (figure == null)
				{

					// in seperate window:
					newMA = PlaceFigure.EXPANDED_TOKENS;
				}
				else
				{
					newMA = PlaceFigure.TOKENS;
				}
			}
			else
			{
				newMA = PlaceFigure.CARDINALITY;
			}
			setAttribute("MarkingAppearance", new Integer(newMA));
			view.addToSelection(this);
		}
		return done;
	}

	/**
	 * Method documentation.
	 * @return
	 */
	protected PlaceInstanceAccessor getPlaceInstance()
	{
		return placeInstance;
	}


	/**
	 * Method documentation.
	 * @param e
	 */
	public void update(FigureChangeEvent e)
	{
		updateLocation();
		super.update(e);
	}

	/**
	 * Method documentation.
	 * @param name
	 * @param value
	 */
	public void setAttribute(String name, Object value)
	{
		if (name.equals("MarkingAppearance"))
		{
			if (figure != null)
			{
				figure.setAttribute(name, value);
			}

			// if we want TokenBagFigures to also
			// be switchable if there is no PlaceInstanceFigure
			// assigned, we need:
			else
			{
				setMarkingAppearance(((Integer) value).intValue());
			}
		}
		else
		{
			if (markingAppearance != PlaceFigure.CARDINALITY)
			{
				attributes.set(name, value);
			}
			super.setAttribute(name, value);
		}
	}

	/**
	 * Returns the figures with dependencies of superclasses
	 * plus the parent figure of this figure.
	 */
	public FigureEnumeration getFiguresWithDependencies()
	{
		FigureEnumeration superDep = super.getFiguresWithDependencies();
		Vector myDep = new Vector(1);

		myDep.addElement(figure);
		return new MergedFigureEnumerator(superDep, new FigureEnumerator(myDep));
	}

	/**
	 * This class is not serializable, although inheriting
	 * serializability from its superclass.
	 * <p>
	 * Reason: Serialization is currently used for Copy&Paste
	 * and the Undo machanism only. As such modifications don't
	 * make sense for instance drawings, this  figure, which is
	 * a part of such drawings, doesn't need to be serializable.
	 * </p><p>
	 * Second reason: Some fields of this class would lead to
	 * a serialization of the compiled net, which is not desirable.
	 * </p>
	 * @throws java.io.NotSerializableException always.
	 */
	private void writeObject(java.io.ObjectOutputStream out) throws java.io.IOException
	{
		throw new java.io.NotSerializableException("TokenBagFigure is not serializable!");
	}

	/**
	 * Deserialization method, behaves like default readObjec
	 * method except restoring the connection to the parent of
	 * this child figure.
	 */
	private void readObject(java.io.ObjectInputStream in)
			throws java.io.IOException, ClassNotFoundException
	{
		in.defaultReadObject();
		if (figure != null)
		{
			figure.addChild(this);
		}

	}

}

