/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

package de.renew.gui;

import java.awt.*;
import java.awt.event.*;
import java.rmi.RemoteException;
import java.util.*;
import collections.CollectionEnumeration;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.standard.*;
import de.renew.simulator.*;
import de.renew.event.*;
import de.renew.remote.*;
import de.renew.unify.Variable;

/**
 * Class documentation.
 */
public class TransitionInstanceFigure extends AbstractFigure implements InstanceFigure,
	de.renew.remote.EventListener
{
	private TransitionFigure transitionFigure;
	private TransitionInstanceAccessor transitionInstance;
	private boolean afterglow = false;
	private Thread afterglowThread = null;
	private UpdateStatus updateStatus = new UpdateStatus();
	private RemoteEventForwarder forwarder;
	protected CPNInstanceDrawing drawing;

	/**
	 * Creates the transition instance figure.
	 * @param drawing The CPN instance drawing to create for.
	 * @param tf The corresponding transition figure.
	 * @param netElement A lookup mapping net element group ids to net elements.
	 */
	public TransitionInstanceFigure(CPNInstanceDrawing drawing, TransitionFigure tf,
		Hashtable netElements)
	{
		this.drawing = drawing;
		transitionFigure = tf;
		initialize(netElements);
	}

	/**
	 * Initializes the figure with the net element lookup.
	 * @param netElements A lookup mapping net element group ids to net elements.
	 */
	protected void initialize(Hashtable netElements)
	{
		if (netElements.size() != 1)
		{
			System.err.println("Error: There is not exactly one transition instance for the figure " + this);
			return;
		}

		transitionInstance = (TransitionInstanceAccessor) netElements.elements().nextElement();

		try
		{
			forwarder = new RemoteEventForwarder(this);
			transitionInstance.addRemoteEventListener(forwarder);
		}
		catch (RemoteException e)
		{
			e.printStackTrace(System.err);
		}
	}

	/**
	 * Returns whether the figure shall display firing state.
	 * @return Whether the figure is firing.
	 */
	public boolean isFiring()
	{
		try
		{
			return transitionInstance.isFiring();
		}
		catch (RemoteException e)
		{
			return false;
		}
	}

	/**
	 * Method documentation.
	 * @return
	 */
	public boolean isHighlighted()
	{
		return isFiring() || afterglow;
	}

	/**
	 * Method documentation.
	 */
	private void invalidateHighlight()
	{
		Figure highlight = transitionFigure.getHighlightFigure();

		if (highlight != null)
		{
			Rectangle area = highlight.displayBox();
			DrawingChangeEvent dce = new DrawingChangeEvent(drawing, area);
			drawing.drawingInvalidated(dce);
		}
	}


	/**
	 * Method documentation.
	 * @return
	 */
	public boolean wantSynchronousNotification()
	{
		return false;
	}

	/**
	 * This event signals that a Transition Occurrence has started or completed.
	 */
	public void update()
	{
		synchronized (updateStatus)
		{
			if (updateStatus.inUpdate)
			{
				updateStatus.anotherTurn = true;
				return;
			}
			else
			{
				updateStatus.inUpdate = true;
			}
		}

		while (true)
		{
			try
			{
				drawing.lock();
	
				if (listener() != null)
				{
					if (afterglowThread != null && afterglowThread.isAlive())
					{
						try
						{
							afterglowThread.notifyAll();
						}
						catch (Exception e)
						{
						}
					}
	
					try
					{
						drawing.lock();
						afterglow = true;
						invalidate();
						invalidateHighlight();
						listener().figureRequestUpdate(new FigureChangeEvent(this));
					}
					finally
					{
						drawing.unlock();
					}
	
					final TransitionInstanceFigure thizz = this;
					afterglowThread = new Thread()
					{
						/**
						 * Method documentation.
						 */
						public void run()
						{
							try
							{
								synchronized (this)
								{
									this.wait(100);
								}
	
								// If another afterglow is instantiated, the current one is
								// interrupted. So at this point, we were not.
								try
								{
									drawing.lock();
									afterglow = false;
									invalidate();
									invalidateHighlight();
									listener().figureRequestUpdate(new FigureChangeEvent(thizz));
								}
								finally
								{
									drawing.unlock();
								}
							}
							catch (InterruptedException e)
							{
							}
						}
	
					};
					afterglowThread.start();
				}
			}
			finally
			{
				drawing.unlock();
			}

			synchronized (updateStatus)
			{
				if (updateStatus.anotherTurn)
				{
					updateStatus.anotherTurn = false;
				}
				else
				{
					updateStatus.inUpdate = false;
					break;
				}
			}
		}
	}

	/**
	 * Method documentation.
	 * @param origin
	 * @param corner
	 */
	public void basicDisplayBox(Point origin, Point corner)
	{

		// do nothing, as the DisplayBox of the Transition is used!
	}

	/**
	 * Method documentation.
	 * @return
	 */
	public Vector handles()
	{
		Vector handles = new Vector();

		handles.addElement(new NullHandle(this, RelativeLocator.northWest()));
		handles.addElement(new NullHandle(this, RelativeLocator.northEast()));
		handles.addElement(new NullHandle(this, RelativeLocator.southWest()));
		handles.addElement(new NullHandle(this, RelativeLocator.southEast()));
		return handles;
	}

	/**
	 * Method documentation.
	 * @return
	 */
	public Rectangle displayBox()
	{
		return transitionFigure.displayBox();
	}

	/**
	 * Method documentation.
	 * @param x
	 * @param y
	 */
	protected void basicMoveBy(int x, int y)
	{

		// do nothing, as the DisplayBox of the Transition is used!
	}

	/**
	 * Method documentation.
	 * @param g
	 */
	public void draw(Graphics g)
	{

		/*
		 * if (actionCounter > 0) {
		 * g.setColor(Color.red);
		 * Rectangle r = displayBox();
		 * g.fillRect(r.x, r.y, r.width, r.height);
		 * }
		 */
	}


	/**
	 * Method documentation.
	 * @param view
	 * @param alternate
	 * @return
	 */
	public boolean inspect(DrawingView view, boolean alternate)
	{

		// System.out.println("Searching bindings for "+transitionInstance+"...");
		if (alternate)
		{
			if (fire())
			{
				String name;
				try
				{
					name = transitionInstance.asString();
				}
				catch (RemoteException e)
				{
					name = e.toString();
				}
				view.editor().showStatus("Transition " + name + " fired.");
			}
			else
			{
				view.editor().showStatus("No binding for this transition.");
			}
		}
		else
		{
			BindingSelectionFrame.open(transitionInstance, getSimulation());
		}
		return true;
	}

	/**
	 * Method documentation.
	 * @return
	 */
	protected CPNSimulation getSimulation()
	{
		return drawing.getMode().getSimulation();
	}

	/**
	 * Method documentation.
	 * @return
	 */
	public boolean fire()
	{
		CPNSimulation simulation = getSimulation();

		simulation.getBreakpointManager().clearLog();
		boolean result;
		try
		{
			result = transitionInstance.fireOneBinding(
				new TracerAccessorImpl(new ConsoleTracer("Synchronously", "(Automatically) ")),
				!simulation.isStrictlySequential());
		}
		catch (RemoteException e)
		{
			e.printStackTrace(System.err);
			result = false;
		}

		simulation.simulationRefresh();
		return result;
	}

	/**
	 * Method documentation.
	 */
	public void release()
	{
		if (transitionInstance != null)
		{
			try
			{
				transitionInstance.removeRemoteEventListener(forwarder);
			}
			catch (RemoteException e)
			{
				e.printStackTrace(System.err);
			}

			// OK: Release reference to transition instance so that it
			// can be garbage collected.
			transitionInstance = null;
		}
		super.release();
	}

	/**
	 * Method documentation.
	 * @return
	 */
	protected TransitionInstanceAccessor getInstance()
	{
		return transitionInstance;
	}

	/**
	 * This class is not serializable, although inheriting
	 * serializability from its superclass.
	 * <p>
	 * Reason: Serialization is currently used for Copy&Paste
	 * and the Undo machanism only. As such modifications don't
	 * make sense for instance drawings, this  figure, which is
	 * a part of such drawings, doesn't need to be serializable.
	 * </p><p>
	 * Second reason: Some fields of this class would lead to
	 * a serialization of the compiled net, which is not desirable.
	 * </p>
	 * @throws java.io.NotSerializableException always.
	 */
	private void writeObject(java.io.ObjectOutputStream out) throws java.io.IOException
	{
		throw new java.io.NotSerializableException("TransitionInstanceFigure is not serializable!");
	}

	private static class UpdateStatus
	{
		private boolean inUpdate = false;
		private boolean anotherTurn = false;
	}
}
