/*
 * @(#)VirtualPlaceFigureCreationTool.java 5.1
 *
 */

package de.renew.gui;

import java.awt.*;
import java.awt.event.MouseEvent;

import CH.ifa.draw.framework.*;
import CH.ifa.draw.standard.*;

/**
 * A more efficient version of the generic Trans creation
 * tool that is not based on cloning.
 */

public  class VirtualPlaceFigureCreationTool extends CreationTool {

    public VirtualPlaceFigureCreationTool(DrawingEditor editor) {
        super(editor);
    }

    /**
     * Creates a new VirtualPlaceFigure.
     */
    protected Figure createFigure() {
        return new VirtualPlaceFigure(null);
    }

    /**
     * the anchor point of the interaction
     */
    private Point anchor = null;

    /**
     * the currently created figure
     */
    private VirtualPlaceFigure  virtualPlace = null;

    /**
     * Handles mouse move events in the drawing view.
     */
    public void mouseMove(MouseEvent e, int x, int y) {
        // highlight PlaceFigures?
    }

    /**
     * If the mouse down hits a PlaceFigure create a new VirtualPlaceFigure.
     */
    public void mouseDown(MouseEvent e, int x, int y)
    {
        int ex = e.getX();
        int ey = e.getY();
        Figure fTarget = drawing().findFigure(ex, ey);
        if (fTarget instanceof PlaceFigure) {
           anchor = new Point(ex,ey);
           virtualPlace = new VirtualPlaceFigure((PlaceFigure)fTarget);
           Rectangle r = fTarget.displayBox();
           virtualPlace.displayBox(new Rectangle(r.x,r.y,r.width,r.height));
           view().add(virtualPlace);
	   changesMade();
        }
    }

    /**
     * Adjust the created connection or split segment.
     */
    public void mouseDrag(MouseEvent e, int x, int y) {
        if (virtualPlace != null) {
           Rectangle r = virtualPlace.displayBox();
           virtualPlace.displayBox(new Rectangle(e.getX(),e.getY(), r.width, r.height));
        }
    }

    /**
     * Drop new VirtualPlaceFigure.
     */
    public void mouseUp(MouseEvent e, int x, int y) {
        virtualPlace = null;
        editor().toolDone();
    }

    public void deactivate() {
        super.deactivate();
        virtualPlace = null;
    }

}
