/*
 * @(#)FSMode.java 5.1
 *
 */

package de.renew.gui.fs;

import java.awt.*;
import java.util.Properties;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.standard.*;
import CH.ifa.draw.util.*;
import CH.ifa.draw.figures.*;
import CH.ifa.draw.application.DrawApplication;
import de.renew.gui.*;
import de.renew.formalism.fs.*;
import de.renew.remote.*;
import de.renew.shadow.ShadowCompiler;
import de.uni_hamburg.fs.*;

/**
 * The mode for FSNets
 */
public class FSMode extends /*Sequential*/JavaMode {

  public final static String IMAGES="/de/renew/gui/fs/images/";

  public ShadowCompiler getCompiler() {
      return new FSNetCompiler();
  }


/*
    public void promptExportConceptsXML() {
      toolDone();
      String path = getSavePath("Export All Concepts as XML...",null);

      if (path != null) {
	try {
            PrintWriter p = new PrintWriter(new FileOutputStream(path));
            p.println("<?xml version=\"1.0\"?>");
            p.println("<!DOCTYPE FS SYSTEM \"FS.dtd\">");
            p.println("<FS>");
            p.print(fs.ConceptImpl.allToXML());
            p.println("</FS>");
            p.close();
	} catch (Exception e) {
	  showStatus(e.toString());
	}
      }
    }
*/

  public Figure getTokenFigure(Object token, boolean expanded) {
      if (token instanceof FeatureStructure) {
	  return new FSFigure((FeatureStructure)token,expanded);
      } else {
	  return super.getTokenFigure(token,expanded);
      }
  }

    public void createAdditionalTools(CPNApplication editor, Panel toolPanel,
				      Palette superPalette) {
	super.createAdditionalTools(editor,toolPanel,superPalette);

        Panel palette = new Palette(editor.menuFrame(),
				    "Information Modeling Tools");

        Tool tool;

	tool = new CPNTextTool(editor, new FSFigure(), false);
	palette.add(editor.createToolButton(IMAGES+"FS",
					    "Feature Structure Tool", tool));

	tool = new TextTool(editor, new FSNodeFigure());
	palette.add(editor.createToolButton(IMAGES+"FSNODE",
					    "Feature Structure Node Tool", tool));

	tool = new ConnectionTool(editor, new FeatureConnection());
	palette.add(editor.createToolButton(IMAGES+"FEATURE",
					    "Feature Tool", tool));

	tool = new TextTool(editor, new ConceptFigure());
	palette.add(editor.createToolButton(IMAGES+"CONCEPT",
					    "Concept Tool", tool));

	tool = new ConnectionTool(editor, new IsaConnection());
	palette.add(editor.createToolButton(IMAGES+"DISISA",
					    "Disjunctive Is-A Tool", tool));

	tool = new ConnectionTool(editor, new IsaConnection(false));
	palette.add(editor.createToolButton(IMAGES+"ISA",
					    "Is-A Tool", tool));

	tool = new ConnectionTool(editor, new AssocConnection());
	palette.add(editor.createToolButton(IMAGES+"ASSOC",
					    "Association Tool", tool));

        tool = new TextTool(editor, new UMLNoteFigure());
        palette.add(editor.createToolButton(IMAGES+"UMLNOTE",
					    "UML Note Tool", tool));

	tool = new ConnectionTool(editor, new LineConnection(null,null,
				       PolyLineFigure.LINE_STYLE_DASHED));
	palette.add(editor.createToolButton(IMAGES+"ANCHOR",
					    "UML Note Anchor Tool", tool));

	toolPanel.add(palette);
    }

    public void createAdditionalMenus(CPNApplication editor, MenuBar mb) {
	modifyAttributesMenu(editor,
			     mb.getMenu(DrawApplication.ATTRIBUTES_MENU));
	modifyLayoutMenu(editor,
			 mb.getMenu(DrawApplication.LAYOUT_MENU));
    }


    protected void modifyAttributesMenu(CPNApplication editor, Menu attrMenu) {
	CommandMenu isaTypeMenu = editor.createCommandMenu("Is-A Type");
	isaTypeMenu.add(new ChangeAttributeCommand("disjunctive",
						   "IsaType",
						   Boolean.TRUE,
						   editor));
	isaTypeMenu.add(new ChangeAttributeCommand("multiple",
						   "IsaType",
						   Boolean.FALSE,
						   editor));
	attrMenu.add(isaTypeMenu);

    }

    protected void modifyLayoutMenu(CPNApplication editor, Menu layoutMenu) {
	CommandMenu renderModeMenu = editor.createCommandMenu("Object Rendering");
	renderModeMenu.add(new SetFSRenderModeCommand("Feature Structures",
						      editor,
						      false));
	renderModeMenu.add(new SetFSRenderModeCommand("UML Notation",
						      editor,
						      true));
	layoutMenu.add(renderModeMenu);

    }

    /** Get the default text for a new inscription child.
     */
    public String getDefaultInscription(InscribableFigure figure) {
	if (figure instanceof AssocConnection) {
	    AssocConnection assoc=(AssocConnection)figure;
	    ConceptFigure concept=(ConceptFigure)assoc.endFigure();
	    if (concept==null || "".equals(concept.getName())) {
		return "relatesTo";
	    } else {
		return concept.getName().toLowerCase();
	    }
	}
	if (figure instanceof FeatureConnection) {
	    return "feat";
	}
	return super.getDefaultInscription(figure);
    }

}
