package de.renew.gui.fs;

import CH.ifa.draw.framework.Figure;
import CH.ifa.draw.figures.*;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.standard.*;
import CH.ifa.draw.util.*;
import de.renew.formalism.fs.*;
import de.renew.shadow.*;
import de.renew.gui.*;
import java.awt.*;
import java.util.*;
import java.io.*;

public class IsaConnection extends ConceptConnection {

  /**
   * Determines type and appearance of the is-a relation.
   * <p>
   * <code>true</code>:  ??? <br>
   * <code>false</code>: ???
   * </p>
   * @serial
   **/
  private boolean isDisjunctive;

  public IsaConnection() {
	this(true);
  }

  public IsaConnection(boolean isDisjunctive) {
      super(new IsaArrowTip());
      this.isDisjunctive=isDisjunctive;
      setFillColor(Color.white);
  }

  protected ShadowNetElement createShadow(ShadowConcept from,
					  ShadowConcept to) {
      //System.out.println("creating shadow for is-a relation between "+from.getName()+" and "+to.getName());
      return new ShadowIsa(from,to,isDisjunctive);
  }

  public boolean isDisjunctive() {
      return isDisjunctive;
  }

  public void setAttribute(String name, Object value) {
    if (name.equals("IsaType")) {
	isDisjunctive=((Boolean)value).booleanValue();
	willChange();
	if (!isDisjunctive && fPoints.size()>2) {
	    // remove all intermediate points
	    Point start = startPoint();
	    Point end = endPoint();
	    fPoints.removeAllElements();
	    fPoints.addElement(start);
	    fPoints.addElement(end);
	}
	updateConnection();
	changed();
    } else {
       super.setAttribute(name,value);
    }
  }

    public void write(StorableOutput dw) {
        super.write(dw);
	dw.writeBoolean(isDisjunctive);
    }

    public void read(StorableInput dr) throws IOException {
	isDisjunctive = false; // prevent updatePoints()
	setFillColor(Color.white);
        super.read(dr);
	isDisjunctive = dr.readBoolean();
    } 

    public void updateConnection() {
        super.updateConnection();
	if (isDisjunctive)
	    updatePoints();
    }

    public void layoutConnection() {
	if (!isDisjunctive)
	    super.layoutConnection();
    }

    /**
     * Gets the handles of the figure.
     */
    public Vector handles() {
	if (isDisjunctive) {
	    Vector handles = new Vector(fPoints.size()*2);
	    handles.addElement(new ChangeConnectionStartHandle(this));
	    for (int i = 1; i < fPoints.size()-1; i++)
		handles.addElement(new NullHandle(this, locator(i)));
	    handles.addElement(new ChangeConnectionEndHandle(this));
	    for (int i = 0; i < fPoints.size()-1; i++)
		handles.addElement(new ElbowHandle(this, i));
	    return handles;
	} else {
	    return super.handles();
	}
    }

    public Locator connectedTextLocator(Figure f) {
	if (isDisjunctive)
	    return new ElbowTextLocator();
	else
	    return super.connectedTextLocator(f);
    }

    protected void updatePoints() {
        willChange();
        Point start = startPoint();
        Point end = endPoint();
        fPoints.removeAllElements();
        fPoints.addElement(start);

        if (start.x != end.x && start.y != end.y) {
            Rectangle r1 = start().owner().displayBox();
            Rectangle r2 = end().owner().displayBox();

            int x1, y1, x2, y2;
	    fPoints.addElement(new Point(start.x, (start.y + end.y)/2));
	    fPoints.addElement(new Point(end.x, (start.y + end.y)/2));
	}
	fPoints.addElement(end);
        changed();
    }

    /**
     * Handles the connection of a connection.
     * Override this method to handle this event.
     */
    protected void handleConnect(Figure start, Figure end) {
	if (end instanceof ConceptFigure // should be!
	    && "interface".equals(((ConceptFigure)end).getStereotype())) {
	   setAttribute("LineStyle", PolyLineFigure.LINE_STYLE_DASHED);
	}
    }

}
