/*
 * @(#)SetFSRenderModeCommand.java 5.1
 *
 */

package de.renew.gui.fs;

import java.util.*;
import CH.ifa.draw.util.Command;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.standard.*;
import CH.ifa.draw.application.DrawApplication;

/**
 * A command to set the mode in which Feature Structures are rendered,
 * either to "normal" or to "UML".
 * <p>
 * Clears the undo history for all drawings!
 * </p>
 */
public class SetFSRenderModeCommand extends Command {

    protected DrawApplication fEditor;
    private boolean fUML;

   /**
    * Constructs a toggle grid command.
    * @param name the command name
    * @param editor the editor
    * @param grid the grid size. A grid size of 1,1 turns grid snapping off.
    */
    public SetFSRenderModeCommand(String name, DrawingEditor editor,
				  boolean renderUML) {
        super(name);
        fEditor = (DrawApplication)editor;
        fUML =renderUML;
    }

    public void execute() {
	FSFigure.umlMode=fUML;
	// Now mark dirty all FSFigures in all drawings:
	Enumeration drawings=fEditor.drawings();
	while (drawings.hasMoreElements()) {
	    StandardDrawing drawing=(StandardDrawing)drawings.nextElement();
	    changeFSFigures(drawing);
	    DrawingView view=fEditor.getView(drawing);
	    if (view!=null) {
		view.clearSelection();
		view.checkDamage();
	    }
	    // To avoid inconsistencies: 
	    fEditor.getUndoRedoManager().clearUndoHistory(drawing);
	}
    }

    private void changeFSFigures(CompositeFigure figure) {
	FigureEnumeration figs=figure.figures();
	while (figs.hasMoreElements()) {
	    Figure fig=figs.nextFigure();
	    if (fig instanceof FSFigure) {
		fig.willChange();
		((FSFigure)fig).markDirty();
		fig.changed();
	    } else if (fig instanceof CompositeFigure) {
		changeFSFigures((CompositeFigure)fig);
	    }
	}
    }
}


