/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

package de.renew.remote;

import collections.CollectionEnumeration;
import collections.CorruptedEnumerationException;
import de.renew.simulator.*;
import java.rmi.*;
import java.rmi.server.*;


/**
 * This class implements the <code>NetAccessor</code> interface
 * and nothing more.
 * <p>
 * </p>
 * NetAccessorImpl.java
 * Created: Thu Jul 13  2000
 * @author Michael Duvigneau
 */

public class NetAccessorImpl extends ObjectAccessorImpl implements NetAccessor
{
	/**
	 * Creates a new net accessor for the given net.
	 *
	 * @param net the net to access
	 * @exception java.rmi.RemoteException if an RMI failure occured.
	 */
	public NetAccessorImpl(Net net) throws RemoteException
	{
		super(net);
	}

	/* This method is specified by the NetAccessor interface. */
	/**
	 * Method documentation.
	 * @return
	 * @throws RemoteException
	 */
	public String getName() throws RemoteException
	{
		return ((Net) object).getName();
	}

	// -------------------------------------------- Place handling ----

	/* This method is specified by the NetAccessor interface. */
	/**
	 * Method documentation.
	 * @return
	 * @throws RemoteException
	 */
	public NetElementID[] getPlaceIDs() throws RemoteException
	{

		boolean corrupted;
		CollectionEnumeration placeEnum;
		NetElementID[] placeIDs = null;

		// Corruption may occur if the net's set of places changes
		// while copying data into the ID array.
		// As this situation should be very rare, the copying of
		// data is in a loop which will then be restarted.
		// Synchronization is not needed this way.
		// It is important that the first action is getting the
		// enumeration to cover the whole computation with its
		// corruption detection.

		do
		{
			placeEnum = ((Net) object).places();
			corrupted = false;
			placeIDs = new NetElementID[((Net) object).placeCount()];
			try
			{
				for (int index = 0; index < placeIDs.length; index++)
				{
					placeIDs[index] = ((Place) placeEnum.nextElement()).getID();
				}
			}
			catch (CorruptedEnumerationException e)
			{
				System.err.println("NetAccessor(" + object
								   + "): place enumeration corrupted, restarting.");
				corrupted = true;
			}
		}
		while (corrupted);

		return placeIDs;
	}

	/**
	 * Return a place accessor for the place with the given
	 * ID. Return <code>null</code>, if no such place exists.
	 *
	 * This method is specified by the NetAccessor interface.
	 *
	 * <p>
	 * An additional remark to the validity of the given id:
	 * It is not checked if the set of places of the accessed
	 * net has changed since the last call to <code>getPlaceIDs()
	 * </code>.
	 * </p>
	 *
	 * @see NetAccessor#getPlace
	 */
	public PlaceAccessor getPlace(NetElementID id) throws RemoteException
	{
		Place result = ((Net) object).getPlaceWithID(id);

		if (result == null)
		{
			return null;
		}
		return new PlaceAccessorImpl(result);
	}

	// --------------------------------------- Transition handling ----

	/* This method is specified by the NetAccessor interface. */
	/**
	 * Method documentation.
	 * @return
	 * @throws RemoteException
	 */
	public NetElementID[] getTransitionIDs() throws RemoteException
	{

		boolean corrupted;
		CollectionEnumeration transitionEnum;
		NetElementID[] transitionIDs = null;

		// Corruption may occur if the net's set of transitions changes
		// while copying data into the ID array.
		// As this situation should be very rare, the copying of
		// data is in a loop which will then be restarted.
		// Synchronization is not needed this way.
		// It is important that the first action is getting the
		// enumeration to cover the whole computation with its
		// corruption detection.

		do
		{
			transitionEnum = ((Net) object).transitions();
			corrupted = false;
			transitionIDs = new NetElementID[((Net) object).transitionCount()];
			try
			{
				for (int index = 0; index < transitionIDs.length; index++)
				{
					transitionIDs[index] = ((Transition) transitionEnum.nextElement()).getID();
				}
			}
			catch (CorruptedEnumerationException e)
			{
				System.err.println("NetAccessor(" + object
								   + "): transition enumeration corrupted, restarting.");
				corrupted = true;
			}
		}
		while (corrupted);

		return transitionIDs;
	}

	/**
	 * Return a transition accessor for the transition with the given
	 * ID. Return <code>null</code>, if no such transition exists.
	 *
	 * This method is specified by the NetAccessor interface.
	 *
	 * <p>
	 * An additional remark to the validity of the given id:
	 * It is not checked if the set of transitions of the accessed
	 * net has changed since the last call to <code>getTransitionIDs()
	 * </code>.
	 * </p>
	 *
	 * @see NetAccessor#getTransition
	 */
	public TransitionAccessor getTransition(NetElementID id) throws RemoteException
	{
		Transition result = ((Net) object).getTransitionWithID(id);

		if (result == null)
		{
			return null;
		}
		return new TransitionAccessorImpl(result);
	}
}
