package de.renew.remote;

import de.renew.simulator.Tracer;
import java.rmi.RemoteException;


/**
 * The remote tracer knows a TracerAccessor and passes
 * all trace messages to its tracer.
 * @author Thomas Jacob
 */
public class RemoteTracer implements Tracer
{
	/**
	 * The tracer accessor to pass messages to.
	 */
	private TracerAccessor tracer;

	/**
	 * Creates a new remote tracer.
	 * @param tracer The tracer accessor to pass messages to.
	 */
	public RemoteTracer(TracerAccessor tracer)
	{
		this.tracer = tracer;
	}

	/**
	 * Process a trace message.
	 * @param message The message to be output
	 */
	public void trace(String message)
	{
		try
		{
			tracer.trace(message);
		}
		catch (RemoteException e)
		{
			// Ignore any problems
		}
	}

	/**
	 * This tells me that all trace messages have arrived.
	 * My descendents will perform an appropriate action, if any.
	 */
	public void complete()
	{
		try
		{
			tracer.complete();
		}
		catch (RemoteException e)
		{
			// Ignore any problems
		}
	}
}
