package de.renew.remote;

import de.renew.simulator.Tracer;
import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;


/**
 * Implements the tracer accessor interface.
 * @author Thomas Jacob
 */
public class TracerAccessorImpl extends UnicastRemoteObject implements TracerAccessor
{
	/**
	 * The tracer to pass messages to.
	 */
	private Tracer tracer;

	/**
	 * Creates a new tracer accessor.
	 * @param tracer The tracer accessor to pass messages to.
	 * @exception RemoteException If any RMI problem occurs.
	 */
	public TracerAccessorImpl(Tracer tracer)
		throws RemoteException
	{
		super(0, SocketFactoryDeterminer.getInstance(), SocketFactoryDeterminer.getInstance());
		this.tracer = tracer;
	}

	/**
	 * Process a trace message.
	 * @param message The message to be output
	 * @exception RemoteException If any RMI problem occurs.
	 */
	public void trace(String message)
		throws RemoteException
	{
		tracer.trace(message);
	}

	/**
	 * This tells me that all trace messages have arrived.
	 * My descendents will perform an appropriate action, if any.
	 * @exception RemoteException If any RMI problem occurs.
	 */
	public void complete()
		throws RemoteException
	{
		tracer.complete();
	}
}
