/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

package de.renew.remote;

import collections.CollectionEnumeration;
import de.renew.simulator.NetElementID;
import de.renew.simulator.Tracer;
import de.renew.simulator.Triggerable;
import java.rmi.*;

/**
 * A transition instance accessor allows to view and modify the
 * state of a transition instance.
 * <p>
 * This is a remote interface that allows an implementation
 * that can be used via RMI.
 * </p>
 *
 * TransitionInstanceAccessor.java
 * Created: Mon Jul 10  2000
 * @author Michael Duvigneau
 */

public interface TransitionInstanceAccessor extends RemoteEventProducer, ObjectAccessor
{
	/**
	 * Returns all bindings for the transition instance
	 * and registeres a remote triggerable for search proposals.
	 * @param triggerableForwarder The triggerable forwarder.
	 * @return The bindings.
	 * @exception java.rmi.RemoteException if an RMI failure occured.
	 */
	public BindingAccessor[] findAllBindings(TriggerableForwarder triggerableForwarder)
		throws RemoteException;

	/**
	 * Fires one binding for the accessor's transition.
	 * @param tracer The tracer to be used.
	 * @param asynchronous Whether to perform the firing asynchronous.
	 * @return Whether a binding could be fired.
	 * @exception RemoteException If an RMI problem occurred.
	 */
	public boolean fireOneBinding(TracerAccessor tracer, boolean asynchronous)
		throws RemoteException;

	/**
	 * Get the ID of the transition instance that is being accessed with
	 * this object. The ID of a transition instance is the same as the
	 * ID of the transition the instance is based on.
	 *
	 * @return the ID of the transition instance
	 * @exception java.rmi.RemoteException if an RMI failure occured.
	 */
	public NetElementID getID() throws RemoteException;

	/**
	 * Get a transition accessor that corresponds to this accessor.
	 *
	 * @return the transition accessor
	 * @exception java.rmi.RemoteException if an RMI failure occured.
	 */
	public TransitionAccessor getTransition() throws RemoteException;

	/**
	 * Get a net instance accessor that corresponds to this accessor.
	 *
	 * @return the net instance accessor
	 * @exception java.rmi.RemoteException if an RMI failure occured.
	 */
	public NetInstanceAccessor getNetInstance() throws RemoteException;

	/**
	 * Returns whether the transition instance is currently firing
	 * at least one occurrance.
	 * @return Whether it is firing.
	 * @exception java.rmi.RemoteException if an RMI failure occured.
	 */
	public boolean isFiring()
		throws RemoteException;
}
