package de.renew.shadow;

import collections.*;

public abstract class ShadowInscribable extends ShadowNetElement {

	static final long serialVersionUID = 1696327514729295533L;

	/**
	 * References all inscriptions (text inscriptions, arcs, etc.)
	 * of this inscribable shadow net element.
	 * <p>
	 * This field is not serialized because we have to avoid deep
	 * recursion on serialization. (This field would write arcs
	 * attached to a node, the arc would write its other adjacent
	 * node, the node would write another arc and so on and so on.)
	 * Instead, its contents has to be rebuild on deserialization.
	 * All inscription net elements have to reregister themselves
	 * on deserialization in the same way they do at their creation
	 * time.
	 * </p>
	 **/
  transient UpdatableSet elements;

  protected ShadowInscribable(ShadowNet shadowNet) {
    super(shadowNet);
    elements=new HashedSet();
  }

  public void add(ShadowNetElement element) {
    elements.include(element);
  }
  public void remove(ShadowNetElement element) {
    elements.exclude(element);
  }

  public CollectionEnumeration elements() {
    return elements.elements();
  }

	/**
	 * Deserialization method, behaves like default readObject
	 * method, additionally re-initializes the set of elements.
	 **/
	private void readObject(java.io.ObjectInputStream in)
		throws java.io.IOException, ClassNotFoundException
	{
		in.defaultReadObject();
    elements=new HashedSet();		
	}

}
