package de.renew.shadow;

import collections.*;
import java.util.Hashtable;

public class ShadowNet implements java.io.Serializable {
  static final long serialVersionUID = 5180248532101141648L;

  public transient Object context=null;

  private UpdatableSet elements;
  private String name;
  ShadowNetSystem netSystem;

  public ShadowNet(String name, ShadowNetSystem netSystem) {
    elements=new HashedSet();
    this.name=name;
    this.netSystem=netSystem;
    netSystem.add(this);
  }

  public void setName(String name) {
    this.name=name;
  }
  public String getName() {
    return name;
  }
  public CollectionEnumeration elements() {
    return elements.elements();
  }

  public void discard() {
    netSystem.remove(this);
  }

	/**
	 * Inserts this shadow net into another shadow net system. This
	 * immediately implies removal from the previous net system.
	 * <p>
	 * This method should not be executed after compilation of the
	 * shadow net: The results and relations to the compiled net
	 * and other shadow nets of the previous net system are not
	 * defined.
	 * </p>
	 **/
	public void switchNetSystem(ShadowNetSystem newNetSystem) {
    netSystem.remove(this);
		netSystem = newNetSystem;
		netSystem.add(this);
	}

  void add(ShadowNetElement element) {
    elements.include(element);
  }
  void remove(ShadowNetElement element) {
    elements.exclude(element);
    element.context=null;
  }

	public String toString() {
		return "ShadowNet \""+name+"\" ("+elements.size()+" elements)";
	}
}

