package de.renew.simulator;

import de.renew.expression.*;
import de.renew.unify.*;
import collections.*;

class ActionExecutable implements LateExecutable {
  ActionInscription actionInscription;
  VariableMapper mapper;
  
  ActionExecutable(ActionInscription actionInscription,
    VariableMapper mapper)
  {
    this.actionInscription=actionInscription;
    this.mapper=mapper;
  }

  // Get the phase during which this executable should fire.
  public int phase() {
    return ACTION;
  }

  // An action might take very long.
  public boolean isLong() {
    return true;
  }

  // Execute this executable.
  public void execute(Tracer tracer) {
    try {
      // No undo past this point, so we may use null
      // as the current state recorder. Also, no
      // calculations should be registered with the checker.
      actionInscription.expression.startEvaluation(mapper,null,null);
    }
    catch (Impossible e) {
      // That hurts. Probably a method threw an exception.
      // This should have been caught by the compiler earlier on,
      // unless the error is a RuntimeException. So let's
      // be rigrous and throw a RuntimeException again.
      throw new RuntimeException("Action exception: "+e.getMessage());
    }
  }
}
