package de.renew.simulator;

import java.util.*;
import java.lang.reflect.Array;
import collections.*;
import de.renew.unify.*;
import de.renew.util.Value;

class ClearArcExecutable implements EarlyExecutable {
  PlaceInstance placeInstance;
  Variable variable;
  ClearArc arc;

  Vector removedTokens;
  Vector removedTimeStamps;

  ClearArcExecutable(PlaceInstance placeInstance, Variable variable,
    ClearArc arc)
  {
    this.placeInstance=placeInstance;
    this.variable=variable;
    this.arc=arc;
  }

  public long lockPriority() {
    return placeInstance.lockOrder;
  }

  public int phase() {
    return CLEAR;
  }

  public void lock() {
    placeInstance.lock.lock();
  }

  public void verify() throws Impossible {
    // Ensure large size increments.
    removedTokens=new Vector(8,0);
    removedTimeStamps=new Vector(8,0);

    placeInstance.extractAllTokens(removedTokens,removedTimeStamps);
    
    Object result=Array.newInstance(arc.elementType,removedTokens.size());
    for (int i=0;i<removedTokens.size();i++) {
      Object value=removedTokens.elementAt(i);
      if (arc.elementType.isPrimitive()) {
	value=Value.unvalueAndCast(value,arc.elementType);
      }
      Array.set(result,i,value);
    }

    Unify.unify(variable,result,null);
  }

  public void execute(Tracer tracer) {
    if (arc.getTrace()) {
      tracer.trace("    Clearing "+placeInstance);
    }     
  }

  public void rollback() {
    // We have to undo the previous removal. We cannot do this
    // without notifying the observers and listeners, because
    // it was not done silently. However, the database must not log
    // this modification.

    for (int i=0;i<removedTokens.size();i++) {
      Object token=removedTokens.elementAt(i);
      double time=((Double)removedTimeStamps.elementAt(i)).doubleValue();
      placeInstance.internallyInsertToken(token,time,false);
    }
    removedTokens=null;
  }

  public void unlock() {
    placeInstance.lock.unlock();
  }
}
