package de.renew.simulator;

import java.util.Hashtable;
import java.util.Enumeration;
import java.io.IOException;
import de.renew.util.Null;
import de.renew.util.RenewObjectOutputStream;

public class IDRegistry implements java.io.Serializable {
  private transient Hashtable table;

  public IDRegistry() {
    table=new Hashtable();
  }

  /** The single well-known instance of the IDRegistry. */
  private static IDRegistry _instance=null;

  /**
   * Get the single well-known instance of the IDRegistry class.
   * Although other instances are possible, this instance is
   * specifically created to be used whenever a local
   * IDRegistry is not available.
   */
  public static synchronized IDRegistry getInstance() {
    if (_instance==null) {
      _instance=new IDRegistry();
    }
    return _instance;
  }
    
  // Needn't be synchronized, because only called from
  // synchronized methods.
  private IDCounter register(Object elem) {
   elem=Null.convert(elem);
    IDCounter counter=(IDCounter)table.get(elem);
    if (counter==null) {
      counter=new IDCounter(IDSource.createID());
      table.put(elem,counter);
    }
    return counter;
  }

  public synchronized String getID(Object elem) {
    return register(elem).getID();
  }

  public synchronized void reserve(Object elem) {
    register(elem).reserve();
  }

  /**
   * Reduce the reservation count by 1.
   * If the reservation count reaches 0, the current ID
   * of the object is discarded.
   *
   * @param elem the object to be unreserved.
   */
  public synchronized void unreserve(Object elem) {
    IDCounter counter=register(elem);
    counter.unreserve();
    if (counter.isDiscardable()) {
      table.remove(elem);
    }
  }

  public synchronized void setAndReserveID(Object elem, String id) {
    elem=Null.convert(elem);
    IDCounter counter=(IDCounter)table.get(elem);
    if (counter==null) {
      counter=new IDCounter(id);
      table.put(elem,counter);
    } else if (!counter.getID().equals(id)) {
      throw new RuntimeException("Token already in use.");
    }
    counter.reserve();
  }

    private synchronized void readObject(java.io.ObjectInputStream in)
	throws IOException, ClassNotFoundException
    {
	in.defaultReadObject();
	table=new Hashtable();
	boolean finished=false;
	do {
	    Object key=in.readObject();
	    if (key==null) {
		finished=true;
	    } else {
		Object counter=in.readObject();
		table.put(key,counter);
	    }
	} while (!finished);
    }
    
    private synchronized void writeObject(java.io.ObjectOutputStream out)
	throws IOException
    {
        RenewObjectOutputStream rOut=null;
	if (out instanceof RenewObjectOutputStream)
	    rOut=(RenewObjectOutputStream)out;
	if (rOut != null) rOut.beginDomain(this);
	out.defaultWriteObject();
	Enumeration enum=table.keys();
	while (enum.hasMoreElements()) {
	    Object key=enum.nextElement();
	    Object counter=table.get(key);
	    out.writeObject(key);
	    out.writeObject(counter);
	}
	out.writeObject(null);
	if (rOut != null) rOut.endDomain(this);
    }

    public synchronized static void reset() {
	_instance=null;
    }

    public synchronized static void save(java.io.ObjectOutput out)
	throws IOException
    {
	out.writeObject(_instance);
    }

    public synchronized static void load(java.io.ObjectInput in)
	throws IOException, ClassNotFoundException
    {
	_instance=(IDRegistry)in.readObject();
    }
}
