package de.renew.util;

import collections.*;
import java.io.Serializable;

public class HashedRelation implements Serializable {
	private UpdatableMap hashtable;
	public HashedRelation() {
		hashtable=new HashedMap();
	}

	public synchronized void put(Object key, Object elem) {
		UpdatableSet set;
		if (hashtable.includesKey(key)) {
			set=(UpdatableSet)hashtable.at(key);
		} else {
			set=new HashedSet();
			hashtable.putAt(key,set);
		}
		set.include(elem);
	}

	public synchronized void remove(Object key, Object elem) {
		UpdatableSet set=(UpdatableSet)hashtable.at(key);
		set.exclude(elem);
		if (set.isEmpty()) {
			hashtable.removeAt(key);
		}
	}

	public synchronized CollectionEnumeration keys() {
		return hashtable.keys();
	}

	public synchronized CollectionEnumeration elementsAt(Object key) {
		if (hashtable.includesKey(key)) {
			UpdatableSet set=(UpdatableSet)hashtable.at(key);
			return set.elements();
		} else {
			return EmptyEnumeration.INSTANCE;
		}
	}

	public synchronized int sizeAt(Object key) {
		if (hashtable.includesKey(key)) {
			UpdatableSet set=(UpdatableSet)hashtable.at(key);
			return set.size();
		} else {
			return 0;
		}
	}

	public String toString() {
		StringBuffer buffer = new StringBuffer();
		buffer.append("de.renew.util.HashedRelation( ");
		CollectionEnumeration keys = hashtable.keys();
		while (keys.hasMoreElements()) {
			Object key = keys.nextElement();
			buffer.append(key);
			buffer.append(" -> (");
			CollectionEnumeration values = ((Set)hashtable.at(key)).elements();
			boolean first = true;
			while (values.hasMoreElements()) {
				if (first) {
					first = false;
				} else {
					buffer.append(", ");
				}
				buffer.append(values.nextElement());
			}
			buffer.append(") ");
		}
		buffer.append(")");
		return buffer.toString();
	}
}
