package de.renew.util;

import collections.CircularList;

/**
 * A <code>RunQueue</code> is responsible for executing
 * <code>Runnable</code> objects in a separate thread fifo-style.
 * The thread is never stopped.
 *
 * @author <a href="mailto:kummer@informatik.uni-hamburg.de">Olaf Kummer</a>
 */
public class RunQueue implements Runnable {
  private CircularList runnables=new CircularList();

  public RunQueue() {
    new Thread(this).start();
  }

  public synchronized void add(Runnable runnable) {
    runnables.insertFirst(runnable);
    notify();
  }

  public void run() {
    Runnable runnable;
    while (true) {
      synchronized(this) {
				while (runnables.isEmpty()) {
					try {
						wait();
					} catch (InterruptedException e) {}
				}
				runnable=(Runnable)runnables.take();
      }
      
      runnable.run();
    }
  }
}
